@@ -17,6 +17,7 @@ use clap_complete::ArgValueCompleter;
 use itertools::Itertools as _;
 use jj_lib::object_id::ObjectId as _;
 use jj_lib::rewrite::merge_commit_trees;
+use pollster::FutureExt as _;
 use tracing::instrument;
 
 use crate::cli_util::CommandHelper;
@@ -106,14 +107,14 @@ pub(crate) fn cmd_backout(
             .collect_vec()
     };
     let mut tx = workspace_command.start_transaction();
-    let mut new_base_tree = merge_commit_trees(tx.repo(), &parents)?;
+    let mut new_base_tree = merge_commit_trees(tx.repo(), &parents).block_on()?;
 
     for (commit_to_back_out, new_commit_description) in
         commits_to_back_out_with_new_commit_descriptions
     {
         let old_base_tree = commit_to_back_out.parent_tree(tx.repo())?;
         let old_tree = commit_to_back_out.tree()?;
-        let new_tree = new_base_tree.merge(&old_tree, &old_base_tree)?;
+        let new_tree = new_base_tree.merge(old_tree, old_base_tree).block_on()?;
         let new_parent_ids = parents.iter().map(|commit| commit.id().clone()).collect();
         let new_commit = tx
             .repo_mut()
