@@ -18,19 +18,19 @@ use clap_complete::ArgValueCandidates;
 use jj_lib::commit::Commit;
 use jj_lib::config::ConfigNamePathBuf;
 use jj_lib::config::ConfigValue;
-use jj_lib::repo::Repo;
+use jj_lib::repo::Repo as _;
 use tracing::instrument;
 
 use super::ConfigLevelArgs;
 use crate::cli_util::CommandHelper;
 use crate::cli_util::WorkspaceCommandHelper;
-use crate::command_error::user_error_with_message;
 use crate::command_error::CommandError;
+use crate::command_error::user_error_with_message;
 use crate::complete;
 use crate::config::parse_value_or_bare_string;
 use crate::ui::Ui;
 
-/// Update config file to set the given option to a given value.
+/// Update a config file to set the given option to a given value.
 #[derive(clap::Args, Clone, Debug)]
 pub struct ConfigSetArgs {
     #[arg(required = true, add = ArgValueCandidates::new(complete::leaf_config_keys))]
@@ -38,12 +38,12 @@ pub struct ConfigSetArgs {
     /// New value to set
     ///
     /// The value should be specified as a TOML expression. If string value
-    /// doesn't contain any TOML constructs (such as apostrophes or array
+    /// isn't enclosed by any TOML constructs (such as apostrophes or array
     /// notation), quotes can be omitted. Note that the value may also need
     /// shell quoting. TOML multi-line strings can be useful if the value
     /// contains apostrophes. For example, to set `foo.bar` to the string
-    /// "don't" use `jj config set --user foo.bar "'''don't'''"`. This
-    /// is valid in both Bash and Fish.
+    /// "{don't}" use `jj config set --user foo.bar "'''{don't}'''"`. This is
+    /// valid in both Bash and Fish.
     ///
     /// Alternative, e.g. to avoid dealing with shell quoting, use `jj config
     /// edit` to edit the TOML file directly.
@@ -65,7 +65,7 @@ pub fn cmd_config_set(
     command: &CommandHelper,
     args: &ConfigSetArgs,
 ) -> Result<(), CommandError> {
-    let mut file = args.level.edit_config_file(command)?;
+    let mut file = args.level.edit_config_file(ui, command)?;
 
     // If the user is trying to change the author config, we should warn them that
     // it won't affect the working copy author
