@@ -15,10 +15,10 @@
 mod copy_detection;
 mod fileset;
 mod index;
+mod index_changed_paths;
 mod init_simple;
 mod local_working_copy;
 mod object;
-mod operation;
 mod reindex;
 mod revset;
 mod snapshot;
@@ -39,14 +39,14 @@ use self::fileset::DebugFilesetArgs;
 use self::fileset::cmd_debug_fileset;
 use self::index::DebugIndexArgs;
 use self::index::cmd_debug_index;
+use self::index_changed_paths::DebugIndexChangedPathsArgs;
+use self::index_changed_paths::cmd_debug_index_changed_paths;
 use self::init_simple::DebugInitSimpleArgs;
 use self::init_simple::cmd_debug_init_simple;
 use self::local_working_copy::DebugLocalWorkingCopyArgs;
 use self::local_working_copy::cmd_debug_local_working_copy;
 use self::object::DebugObjectArgs;
 use self::object::cmd_debug_object;
-use self::operation::DebugOperationArgs;
-use self::operation::cmd_debug_operation;
 use self::reindex::DebugReindexArgs;
 use self::reindex::cmd_debug_reindex;
 use self::revset::DebugRevsetArgs;
@@ -73,12 +73,11 @@ pub enum DebugCommand {
     CopyDetection(CopyDetectionArgs),
     Fileset(DebugFilesetArgs),
     Index(DebugIndexArgs),
+    IndexChangedPaths(DebugIndexChangedPathsArgs),
     InitSimple(DebugInitSimpleArgs),
     LocalWorkingCopy(DebugLocalWorkingCopyArgs),
     #[command(subcommand)]
     Object(DebugObjectArgs),
-    #[command(visible_alias = "view")]
-    Operation(DebugOperationArgs),
     Reindex(DebugReindexArgs),
     Revset(DebugRevsetArgs),
     Snapshot(DebugSnapshotArgs),
@@ -98,10 +97,10 @@ pub fn cmd_debug(
         DebugCommand::CopyDetection(args) => cmd_debug_copy_detection(ui, command, args),
         DebugCommand::Fileset(args) => cmd_debug_fileset(ui, command, args),
         DebugCommand::Index(args) => cmd_debug_index(ui, command, args),
+        DebugCommand::IndexChangedPaths(args) => cmd_debug_index_changed_paths(ui, command, args),
         DebugCommand::InitSimple(args) => cmd_debug_init_simple(ui, command, args),
         DebugCommand::LocalWorkingCopy(args) => cmd_debug_local_working_copy(ui, command, args),
         DebugCommand::Object(args) => cmd_debug_object(ui, command, args),
-        DebugCommand::Operation(args) => cmd_debug_operation(ui, command, args),
         DebugCommand::Reindex(args) => cmd_debug_reindex(ui, command, args),
         DebugCommand::Revset(args) => cmd_debug_revset(ui, command, args),
         DebugCommand::Snapshot(args) => cmd_debug_snapshot(ui, command, args),
