@@ -32,11 +32,12 @@ pub fn cmd_git_import(
     _args: &GitImportArgs,
 ) -> Result<(), CommandError> {
     let mut workspace_command = command.workspace_helper(ui)?;
+    let git_settings = workspace_command.settings().git_settings()?;
     let mut tx = workspace_command.start_transaction();
     // In non-colocated repo, Git HEAD will never be moved internally by jj.
     // That's why cmd_git_export() doesn't export the HEAD ref.
     git::import_head(tx.repo_mut())?;
-    let stats = git::import_refs(tx.repo_mut(), &command.settings().git_settings()?)?;
+    let stats = git::import_refs(tx.repo_mut(), &git_settings)?;
     print_git_import_stats(ui, tx.repo(), &stats, true)?;
     tx.finish(ui, "import git refs")?;
     Ok(())
