@@ -37,10 +37,10 @@ use jj_lib::ref_name::RefNameBuf;
 use jj_lib::ref_name::RemoteName;
 use jj_lib::ref_name::RemoteNameBuf;
 use jj_lib::ref_name::RemoteRefSymbol;
-use jj_lib::refs::classify_bookmark_push_action;
 use jj_lib::refs::BookmarkPushAction;
 use jj_lib::refs::BookmarkPushUpdate;
 use jj_lib::refs::LocalAndRemoteRef;
+use jj_lib::refs::classify_bookmark_push_action;
 use jj_lib::repo::Repo;
 use jj_lib::revset::RevsetExpression;
 use jj_lib::settings::UserSettings;
@@ -48,18 +48,18 @@ use jj_lib::signing::SignBehavior;
 use jj_lib::str_util::StringPattern;
 use jj_lib::view::View;
 
-use crate::cli_util::has_tracked_remote_bookmarks;
-use crate::cli_util::short_commit_hash;
 use crate::cli_util::CommandHelper;
 use crate::cli_util::RevisionArg;
 use crate::cli_util::WorkspaceCommandHelper;
 use crate::cli_util::WorkspaceCommandTransaction;
+use crate::cli_util::has_tracked_remote_bookmarks;
+use crate::cli_util::short_commit_hash;
+use crate::command_error::CommandError;
 use crate::command_error::cli_error;
 use crate::command_error::cli_error_with_message;
 use crate::command_error::user_error;
 use crate::command_error::user_error_with_hint;
 use crate::command_error::user_error_with_message;
-use crate::command_error::CommandError;
 use crate::commands::git::get_single_remote;
 use crate::complete;
 use crate::formatter::Formatter;
@@ -606,8 +606,9 @@ fn sign_commits_before_push(
     let commit_ids: IndexSet<CommitId> = commits_to_sign.iter().ids().cloned().collect();
     let mut old_to_new_commits_map: HashMap<CommitId, CommitId> = HashMap::new();
     let mut num_rebased_descendants = 0;
-    tx.repo_mut()
-        .transform_descendants(commit_ids.iter().cloned().collect_vec(), |rewriter| {
+    tx.repo_mut().transform_descendants(
+        commit_ids.iter().cloned().collect_vec(),
+        async |rewriter| {
             let old_commit_id = rewriter.old_commit().id().clone();
             if commit_ids.contains(&old_commit_id) {
                 let commit = rewriter
@@ -621,7 +622,8 @@ fn sign_commits_before_push(
                 old_to_new_commits_map.insert(old_commit_id, commit.id().clone());
             }
             Ok(())
-        })?;
+        },
+    )?;
 
     let bookmark_updates = bookmark_updates
         .into_iter()
