@@ -34,12 +34,14 @@ mod git;
 mod help;
 mod interdiff;
 mod log;
+mod metaedit;
 mod new;
 mod next;
 mod operation;
 mod parallelize;
 mod prev;
 mod rebase;
+mod redo;
 mod resolve;
 mod restore;
 mod revert;
@@ -53,6 +55,7 @@ mod split;
 mod squash;
 mod status;
 mod tag;
+mod undo;
 mod unsign;
 mod util;
 mod version;
@@ -116,6 +119,7 @@ enum Command {
     Help(help::HelpArgs),
     Interdiff(interdiff::InterdiffArgs),
     Log(log::LogArgs),
+    Metaedit(metaedit::MetaeditArgs),
     New(new::NewArgs),
     Next(next::NextArgs),
     #[command(subcommand)]
@@ -124,6 +128,7 @@ enum Command {
     Parallelize(parallelize::ParallelizeArgs),
     Prev(prev::PrevArgs),
     Rebase(rebase::RebaseArgs),
+    Redo(redo::RedoArgs),
     Resolve(resolve::ResolveArgs),
     Restore(restore::RestoreArgs),
     Revert(revert::RevertArgs),
@@ -141,8 +146,7 @@ enum Command {
     Status(status::StatusArgs),
     #[command(subcommand)]
     Tag(tag::TagCommand),
-    /// Undo an operation (shortcut for `jj op undo`)
-    Undo(operation::undo::OperationUndoArgs),
+    Undo(undo::UndoArgs),
     Unsign(unsign::UnsignArgs),
     #[command(subcommand)]
     Util(util::UtilCommand),
@@ -173,6 +177,7 @@ pub fn run_command(ui: &mut Ui, command_helper: &CommandHelper) -> Result<(), Co
         Command::Diffedit(args) => diffedit::cmd_diffedit(ui, command_helper, args),
         Command::Duplicate(args) => duplicate::cmd_duplicate(ui, command_helper, args),
         Command::Edit(args) => edit::cmd_edit(ui, command_helper, args),
+        Command::Evolog(args) => evolog::cmd_evolog(ui, command_helper, args),
         Command::File(args) => file::cmd_file(ui, command_helper, args),
         Command::Fix(args) => fix::cmd_fix(ui, command_helper, args),
         #[cfg(feature = "git")]
@@ -180,13 +185,14 @@ pub fn run_command(ui: &mut Ui, command_helper: &CommandHelper) -> Result<(), Co
         Command::Help(args) => help::cmd_help(ui, command_helper, args),
         Command::Interdiff(args) => interdiff::cmd_interdiff(ui, command_helper, args),
         Command::Log(args) => log::cmd_log(ui, command_helper, args),
+        Command::Metaedit(args) => metaedit::cmd_metaedit(ui, command_helper, args),
         Command::New(args) => new::cmd_new(ui, command_helper, args),
         Command::Next(args) => next::cmd_next(ui, command_helper, args),
-        Command::Evolog(args) => evolog::cmd_evolog(ui, command_helper, args),
         Command::Operation(args) => operation::cmd_operation(ui, command_helper, args),
         Command::Parallelize(args) => parallelize::cmd_parallelize(ui, command_helper, args),
         Command::Prev(args) => prev::cmd_prev(ui, command_helper, args),
         Command::Rebase(args) => rebase::cmd_rebase(ui, command_helper, args),
+        Command::Redo(args) => redo::cmd_redo(ui, command_helper, args),
         Command::Resolve(args) => resolve::cmd_resolve(ui, command_helper, args),
         Command::Restore(args) => restore::cmd_restore(ui, command_helper, args),
         Command::Revert(args) => revert::cmd_revert(ui, command_helper, args),
@@ -202,7 +208,7 @@ pub fn run_command(ui: &mut Ui, command_helper: &CommandHelper) -> Result<(), Co
         Command::Squash(args) => squash::cmd_squash(ui, command_helper, args),
         Command::Status(args) => status::cmd_status(ui, command_helper, args),
         Command::Tag(args) => tag::cmd_tag(ui, command_helper, args),
-        Command::Undo(args) => operation::undo::cmd_op_undo(ui, command_helper, args),
+        Command::Undo(args) => undo::cmd_undo(ui, command_helper, args),
         Command::Unsign(args) => unsign::cmd_unsign(ui, command_helper, args),
         Command::Util(args) => util::cmd_util(ui, command_helper, args),
         Command::Version(args) => version::cmd_version(ui, command_helper, args),
@@ -210,6 +216,25 @@ pub fn run_command(ui: &mut Ui, command_helper: &CommandHelper) -> Result<(), Co
     }
 }
 
+/// Wraps deprecated command of `old_name` which has been renamed to `new_name`.
+pub(crate) fn renamed_cmd<Args>(
+    old_name: &'static str,
+    new_name: &'static str,
+    cmd: impl Fn(&mut Ui, &CommandHelper, &Args) -> Result<(), CommandError>,
+) -> impl Fn(&mut Ui, &CommandHelper, &Args) -> Result<(), CommandError> {
+    move |ui: &mut Ui, command: &CommandHelper, args: &Args| -> Result<(), CommandError> {
+        writeln!(
+            ui.warning_default(),
+            "`jj {old_name}` is deprecated; use `jj {new_name}` instead, which is equivalent"
+        )?;
+        writeln!(
+            ui.warning_default(),
+            "`jj {old_name}` will be removed in a future version, and this will be a hard error"
+        )?;
+        cmd(ui, command, args)
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
