@@ -16,8 +16,8 @@ mod abandon;
 mod diff;
 mod log;
 mod restore;
+pub mod revert;
 mod show;
-pub mod undo;
 
 use abandon::OperationAbandonArgs;
 use abandon::cmd_op_abandon;
@@ -28,13 +28,14 @@ use log::OperationLogArgs;
 use log::cmd_op_log;
 use restore::OperationRestoreArgs;
 use restore::cmd_op_restore;
+use revert::OperationRevertArgs;
+use revert::cmd_op_revert;
 use show::OperationShowArgs;
 use show::cmd_op_show;
-use undo::OperationUndoArgs;
-use undo::cmd_op_undo;
 
 use crate::cli_util::CommandHelper;
 use crate::command_error::CommandError;
+use crate::commands::renamed_cmd;
 use crate::ui::Ui;
 
 /// Commands for working with the operation log
@@ -49,8 +50,11 @@ pub enum OperationCommand {
     Diff(OperationDiffArgs),
     Log(OperationLogArgs),
     Restore(OperationRestoreArgs),
+    Revert(OperationRevertArgs),
     Show(OperationShowArgs),
-    Undo(OperationUndoArgs),
+    // TODO: Delete in jj 0.39.0+
+    #[command(hide = true)]
+    Undo(OperationRevertArgs),
 }
 
 pub fn cmd_operation(
@@ -63,13 +67,18 @@ pub fn cmd_operation(
         OperationCommand::Diff(args) => cmd_op_diff(ui, command, args),
         OperationCommand::Log(args) => cmd_op_log(ui, command, args),
         OperationCommand::Restore(args) => cmd_op_restore(ui, command, args),
+        OperationCommand::Revert(args) => cmd_op_revert(ui, command, args),
         OperationCommand::Show(args) => cmd_op_show(ui, command, args),
-        OperationCommand::Undo(args) => cmd_op_undo(ui, command, args),
+        OperationCommand::Undo(args) => {
+            let cmd = renamed_cmd("op undo", "op revert", cmd_op_revert);
+            cmd(ui, command, args)
+        }
     }
 }
 
+// pub for `jj undo`
 #[derive(Copy, Clone, Debug, PartialEq, Eq, PartialOrd, Ord, clap::ValueEnum)]
-enum UndoWhatToRestore {
+pub(crate) enum RevertWhatToRestore {
     /// The jj repo state and local bookmarks
     Repo,
     /// The remote-tracking bookmarks. Do not restore these if you'd like to
@@ -77,21 +86,24 @@ enum UndoWhatToRestore {
     RemoteTracking,
 }
 
-const DEFAULT_UNDO_WHAT: [UndoWhatToRestore; 2] =
-    [UndoWhatToRestore::Repo, UndoWhatToRestore::RemoteTracking];
+// pub for `jj undo`
+pub(crate) const DEFAULT_REVERT_WHAT: [RevertWhatToRestore; 2] = [
+    RevertWhatToRestore::Repo,
+    RevertWhatToRestore::RemoteTracking,
+];
 
 /// Restore only the portions of the view specified by the `what` argument
-fn view_with_desired_portions_restored(
+pub(crate) fn view_with_desired_portions_restored(
     view_being_restored: &jj_lib::op_store::View,
     current_view: &jj_lib::op_store::View,
-    what: &[UndoWhatToRestore],
+    what: &[RevertWhatToRestore],
 ) -> jj_lib::op_store::View {
-    let repo_source = if what.contains(&UndoWhatToRestore::Repo) {
+    let repo_source = if what.contains(&RevertWhatToRestore::Repo) {
         view_being_restored
     } else {
         current_view
     };
-    let remote_source = if what.contains(&UndoWhatToRestore::RemoteTracking) {
+    let remote_source = if what.contains(&RevertWhatToRestore::RemoteTracking) {
         view_being_restored
     } else {
         current_view
