@@ -23,8 +23,8 @@ use tracing::instrument;
 
 use crate::cli_util::CommandHelper;
 use crate::cli_util::RevisionArg;
-use crate::command_error::user_error;
 use crate::command_error::CommandError;
+use crate::command_error::user_error;
 use crate::complete;
 use crate::ui::Ui;
 
@@ -58,7 +58,7 @@ pub(crate) struct RestoreArgs {
         long,
         short,
         value_name = "REVSET",
-        add = ArgValueCandidates::new(complete::all_revisions)
+        add = ArgValueCompleter::new(complete::revset_expression_all),
     )]
     from: Option<RevisionArg>,
     /// Revision to restore into (destination)
@@ -66,7 +66,7 @@ pub(crate) struct RestoreArgs {
         long, short = 't',
         visible_alias = "to",
         value_name = "REVSET",
-        add = ArgValueCandidates::new(complete::mutable_revisions)
+        add = ArgValueCompleter::new(complete::revset_expression_mutable),
     )]
     into: Option<RevisionArg>,
     /// Undo the changes in a revision as compared to the merge of its parents.
@@ -81,7 +81,7 @@ pub(crate) struct RestoreArgs {
         long, short,
         value_name = "REVSET",
         conflicts_with_all = ["into", "from"],
-        add = ArgValueCandidates::new(complete::all_revisions),
+        add = ArgValueCompleter::new(complete::revset_expression_all),
     )]
     changes_in: Option<RevisionArg>,
     /// Prints an error. DO NOT USE.
@@ -97,7 +97,11 @@ pub(crate) struct RestoreArgs {
     #[arg(long, short)]
     interactive: bool,
     /// Specify diff editor to be used (implies --interactive)
-    #[arg(long, value_name = "NAME")]
+    #[arg(
+        long,
+        value_name = "NAME",
+        add = ArgValueCandidates::new(complete::diff_editors),
+    )]
     tool: Option<String>,
     /// Preserve the content (not the diff) when rebasing descendants
     #[arg(long)]
@@ -113,11 +117,13 @@ pub(crate) fn cmd_restore(
     let mut workspace_command = command.workspace_helper(ui)?;
     let (from_commits, from_tree, to_commit);
     if args.revision.is_some() {
-        return Err(user_error(
-            "`jj restore` does not have a `--revision`/`-r` option. If you'd like to modify\nthe \
-             *current* revision, use `--from`. If you'd like to modify a *different* \
-             revision,\nuse `--into` or `--changes-in`.",
-        ));
+        return Err(
+            user_error("`jj restore` does not have a `--revision`/`-r` option.")
+                .hinted("To modify the current revision, use `--from`.")
+                .hinted(
+                    "To undo changes in a revision compared to its parents, use `--changes-in`.",
+                ),
+        );
     }
     if args.from.is_some() || args.into.is_some() {
         to_commit = workspace_command
@@ -161,12 +167,11 @@ pub(crate) fn cmd_restore(
         writeln!(ui.status(), "Nothing changed.")?;
     } else {
         let mut tx = workspace_command.start_transaction();
-        let new_commit = tx
-            .repo_mut()
+        tx.repo_mut()
             .rewrite_commit(&to_commit)
             .set_tree_id(new_tree_id)
             .write()?;
-        // rebase_descendants early; otherwise `new_commit` would always have
+        // rebase_descendants early; otherwise the new commit would always have
         // a conflicted change id at this point.
         let (num_rebased, extra_msg) = if args.restore_descendants {
             (
@@ -177,9 +182,6 @@ pub(crate) fn cmd_restore(
             (tx.repo_mut().rebase_descendants()?, "")
         };
         if let Some(mut formatter) = ui.status_formatter() {
-            write!(formatter, "Created ")?;
-            tx.write_commit_summary(formatter.as_mut(), &new_commit)?;
-            writeln!(formatter)?;
             if num_rebased > 0 {
                 writeln!(
                     formatter,
