@@ -1,6 +1,6 @@
 use std::collections::HashSet;
 
-use clap_complete::ArgValueCandidates;
+use clap_complete::ArgValueCompleter;
 use itertools::Itertools as _;
 use jj_lib::backend::BackendError;
 use jj_lib::revset::RevsetExpression;
@@ -28,7 +28,7 @@ pub(crate) struct SimplifyParentsArgs {
         long,
         short,
         value_name = "REVSETS",
-        add = ArgValueCandidates::new(complete::mutable_revisions)
+        add = ArgValueCompleter::new(complete::revset_expression_mutable),
     )]
     source: Vec<RevisionArg>,
     /// Simplify specified revision(s) (can be repeated)
@@ -40,7 +40,7 @@ pub(crate) struct SimplifyParentsArgs {
         long,
         short,
         value_name = "REVSETS",
-        add = ArgValueCandidates::new(complete::mutable_revisions)
+        add = ArgValueCompleter::new(complete::revset_expression_mutable),
     )]
     revisions: Vec<RevisionArg>,
 }
@@ -85,7 +85,7 @@ pub(crate) fn cmd_simplify_parents(
     let mut reparented_descendants = 0;
 
     tx.repo_mut()
-        .transform_descendants(commit_ids, |mut rewriter| {
+        .transform_descendants(commit_ids, async |mut rewriter| {
             let num_old_heads = rewriter.new_parents().len();
             if commit_ids_set.contains(rewriter.old_commit().id()) && num_old_heads > 1 {
                 // TODO: BackendError is not the right error here because
