@@ -38,6 +38,7 @@ use jj_lib::config::StackedConfig;
 use jj_lib::dsl_util;
 use regex::Captures;
 use regex::Regex;
+use serde::Serialize as _;
 use tracing::instrument;
 
 use crate::command_error::CommandError;
@@ -71,14 +72,40 @@ fn is_bare_string(value_str: &str) -> bool {
     }
 }
 
+/// Converts [`ConfigValue`] (or [`toml_edit::Value`]) to [`toml::Value`] which
+/// implements [`serde::Serialize`].
+pub fn to_serializable_value(value: ConfigValue) -> toml::Value {
+    match value {
+        ConfigValue::String(v) => toml::Value::String(v.into_value()),
+        ConfigValue::Integer(v) => toml::Value::Integer(v.into_value()),
+        ConfigValue::Float(v) => toml::Value::Float(v.into_value()),
+        ConfigValue::Boolean(v) => toml::Value::Boolean(v.into_value()),
+        ConfigValue::Datetime(v) => toml::Value::Datetime(v.into_value()),
+        ConfigValue::Array(array) => {
+            let array = array.into_iter().map(to_serializable_value).collect();
+            toml::Value::Array(array)
+        }
+        ConfigValue::InlineTable(table) => {
+            let table = table
+                .into_iter()
+                .map(|(k, v)| (k, to_serializable_value(v)))
+                .collect();
+            toml::Value::Table(table)
+        }
+    }
+}
+
 /// Configuration variable with its source information.
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, serde::Serialize)]
 pub struct AnnotatedValue {
     /// Dotted name path to the configuration variable.
+    #[serde(serialize_with = "serialize_name")]
     pub name: ConfigNamePathBuf,
     /// Configuration value.
+    #[serde(serialize_with = "serialize_value")]
     pub value: ConfigValue,
     /// Source of the configuration value.
+    #[serde(serialize_with = "serialize_source")]
     pub source: ConfigSource,
     /// Path to the source file, if available.
     pub path: Option<PathBuf>,
@@ -86,6 +113,27 @@ pub struct AnnotatedValue {
     pub is_overridden: bool,
 }
 
+fn serialize_name<S>(name: &ConfigNamePathBuf, serializer: S) -> Result<S::Ok, S::Error>
+where
+    S: serde::Serializer,
+{
+    name.to_string().serialize(serializer)
+}
+
+fn serialize_value<S>(value: &ConfigValue, serializer: S) -> Result<S::Ok, S::Error>
+where
+    S: serde::Serializer,
+{
+    to_serializable_value(value.clone()).serialize(serializer)
+}
+
+fn serialize_source<S>(source: &ConfigSource, serializer: S) -> Result<S::Ok, S::Error>
+where
+    S: serde::Serializer,
+{
+    source.to_string().serialize(serializer)
+}
+
 /// Collects values under the given `filter_prefix` name recursively, from all
 /// layers.
 pub fn resolved_config_values(
@@ -530,7 +578,7 @@ fn config_files_for(
 /// 4. Repo config `.jj/repo/config.toml`
 /// 5. TODO: Workspace config `.jj/config.toml`
 /// 6. Override environment variables
-/// 7. Command-line arguments `--config`, `--config-toml`, `--config-file`
+/// 7. Command-line arguments `--config` and `--config-file`
 ///
 /// This function sets up 1, 2, and 6.
 pub fn config_from_environment(default_layers: impl IntoIterator<Item = ConfigLayer>) -> RawConfig {
@@ -633,8 +681,6 @@ fn env_overrides_layer() -> ConfigLayer {
 pub enum ConfigArgKind {
     /// `--config=NAME=VALUE`
     Item,
-    /// `--config-toml=TOML`
-    Toml,
     /// `--config-file=PATH`
     File,
 }
@@ -659,11 +705,6 @@ pub fn parse_config_args(
                 }
                 layers.push(layer);
             }
-            ConfigArgKind::Toml => {
-                for (_, text) in chunk {
-                    layers.push(ConfigLayer::parse(source, text)?);
-                }
-            }
             ConfigArgKind::File => {
                 for (_, path) in chunk {
                     layers.push(ConfigLayer::load_from_file(source, path.into())?);
@@ -695,8 +736,6 @@ fn parse_config_arg_item(item_str: &str) -> Result<(ConfigNamePathBuf, ConfigVal
 /// List of rules to migrate deprecated config variables.
 pub fn default_config_migrations() -> Vec<ConfigMigrationRule> {
     vec![
-        // TODO: Delete in jj 0.32+
-        ConfigMigrationRule::rename_value("git.auto-local-branch", "git.auto-local-bookmark"),
         // TODO: Delete in jj 0.33+
         ConfigMigrationRule::rename_update_value(
             "signing.sign-all",
