@@ -13,6 +13,7 @@
 // limitations under the License.
 
 use std::borrow::Borrow;
+use std::borrow::Cow;
 use std::cmp::max;
 use std::io;
 use std::iter;
@@ -37,6 +38,7 @@ use jj_lib::backend::TreeValue;
 use jj_lib::commit::Commit;
 use jj_lib::config::ConfigGetError;
 use jj_lib::conflicts::ConflictMarkerStyle;
+use jj_lib::conflicts::ConflictMaterializeOptions;
 use jj_lib::conflicts::MaterializedFileValue;
 use jj_lib::conflicts::MaterializedTreeDiffEntry;
 use jj_lib::conflicts::MaterializedTreeValue;
@@ -60,6 +62,7 @@ use jj_lib::files::DiffLineIterator;
 use jj_lib::files::DiffLineNumber;
 use jj_lib::matchers::Matcher;
 use jj_lib::merge::Merge;
+use jj_lib::merge::MergeBuilder;
 use jj_lib::merge::MergedTreeValue;
 use jj_lib::merged_tree::MergedTree;
 use jj_lib::object_id::ObjectId as _;
@@ -422,39 +425,29 @@ impl<'a> DiffRenderer<'a> {
         }
     }
 
-    /// Generates diff between `from_tree` and `to_tree`.
-    #[expect(clippy::too_many_arguments)]
-    pub fn show_diff(
+    /// Generates diff between `trees`.
+    pub async fn show_diff(
         &self,
         ui: &Ui, // TODO: remove Ui dependency if possible
         formatter: &mut dyn Formatter,
-        from_tree: &MergedTree,
-        to_tree: &MergedTree,
+        trees: [&MergedTree; 2],
         matcher: &dyn Matcher,
         copy_records: &CopyRecords,
         width: usize,
     ) -> Result<(), DiffRenderError> {
-        formatter.with_label("diff", |formatter| {
-            self.show_diff_inner(
-                ui,
-                formatter,
-                from_tree,
-                to_tree,
-                matcher,
-                copy_records,
-                width,
-            )
-            .block_on()
-        })
+        formatter
+            .with_label_async("diff", async |formatter| {
+                self.show_diff_trees(ui, formatter, trees, matcher, copy_records, width)
+                    .await
+            })
+            .await
     }
 
-    #[expect(clippy::too_many_arguments)]
-    async fn show_diff_inner(
+    async fn show_diff_trees(
         &self,
         ui: &Ui,
         formatter: &mut dyn Formatter,
-        from_tree: &MergedTree,
-        to_tree: &MergedTree,
+        [from_tree, to_tree]: [&MergedTree; 2],
         matcher: &dyn Matcher,
         copy_records: &CopyRecords,
         width: usize,
@@ -547,10 +540,58 @@ impl<'a> DiffRenderer<'a> {
         Ok(())
     }
 
+    fn show_diff_commit_descriptions(
+        &self,
+        formatter: &mut dyn Formatter,
+        [from_description, to_description]: [&Merge<&str>; 2],
+    ) -> Result<(), DiffRenderError> {
+        if from_description == to_description {
+            return Ok(());
+        }
+        const DUMMY_PATH: &str = "JJ-COMMIT-DESCRIPTION";
+        let materialize_options = ConflictMaterializeOptions {
+            marker_style: self.conflict_marker_style,
+            marker_len: None,
+        };
+        for format in &self.formats {
+            match format {
+                // Omit diff from "short" formats. Printing dummy file path
+                // wouldn't be useful.
+                DiffFormat::Summary
+                | DiffFormat::Stat(_)
+                | DiffFormat::Types
+                | DiffFormat::NameOnly => {}
+                DiffFormat::Git(options) => {
+                    // Git format must be parsable, so use dummy file path.
+                    show_git_diff_texts(
+                        formatter,
+                        [DUMMY_PATH, DUMMY_PATH],
+                        [from_description, to_description],
+                        options,
+                        &materialize_options,
+                    )?;
+                }
+                DiffFormat::ColorWords(options) => {
+                    writeln!(formatter.labeled("header"), "Modified commit description:")?;
+                    show_color_words_diff_hunks(
+                        formatter,
+                        [from_description, to_description],
+                        options,
+                        &materialize_options,
+                    )?;
+                }
+                DiffFormat::Tool(_) => {
+                    // TODO: materialize commit description as file?
+                }
+            }
+        }
+        Ok(())
+    }
+
     /// Generates diff between `from_commits` and `to_commit` based off their
     /// parents. The `from_commits` will temporarily be rebased onto the
     /// `to_commit` parents to exclude unrelated changes.
-    pub fn show_inter_diff(
+    pub async fn show_inter_diff(
         &self,
         ui: &Ui,
         formatter: &mut dyn Formatter,
@@ -559,22 +600,42 @@ impl<'a> DiffRenderer<'a> {
         matcher: &dyn Matcher,
         width: usize,
     ) -> Result<(), DiffRenderError> {
+        let from_description = if from_commits.is_empty() {
+            Merge::resolved("")
+        } else {
+            // TODO: use common predecessor as the base description?
+            MergeBuilder::from_iter(itertools::intersperse(
+                from_commits.iter().map(|c| c.description()),
+                "",
+            ))
+            .build()
+            .simplify()
+        };
+        let to_description = Merge::resolved(to_commit.description());
         let from_tree = rebase_to_dest_parent(self.repo, from_commits, to_commit)?;
-        let to_tree = to_commit.tree()?;
+        let to_tree = to_commit.tree_async().await?;
         let copy_records = CopyRecords::default(); // TODO
-        self.show_diff(
-            ui,
-            formatter,
-            &from_tree,
-            &to_tree,
-            matcher,
-            &copy_records,
-            width,
-        )
+        formatter
+            .with_label_async("diff", async |formatter| {
+                self.show_diff_commit_descriptions(
+                    formatter,
+                    [&from_description, &to_description],
+                )?;
+                self.show_diff_trees(
+                    ui,
+                    formatter,
+                    [&from_tree, &to_tree],
+                    matcher,
+                    &copy_records,
+                    width,
+                )
+                .await
+            })
+            .await
     }
 
     /// Generates diff of the given `commit` compared to its parents.
-    pub fn show_patch(
+    pub async fn show_patch(
         &self,
         ui: &Ui,
         formatter: &mut dyn Formatter,
@@ -583,7 +644,7 @@ impl<'a> DiffRenderer<'a> {
         width: usize,
     ) -> Result<(), DiffRenderError> {
         let from_tree = commit.parent_tree(self.repo)?;
-        let to_tree = commit.tree()?;
+        let to_tree = commit.tree_async().await?;
         let mut copy_records = CopyRecords::default();
         for parent_id in commit.parent_ids() {
             let records = get_copy_records(self.repo.store(), parent_id, commit.id(), matcher)?;
@@ -592,12 +653,12 @@ impl<'a> DiffRenderer<'a> {
         self.show_diff(
             ui,
             formatter,
-            &from_tree,
-            &to_tree,
+            [&from_tree, &to_tree],
             matcher,
             &copy_records,
             width,
         )
+        .await
     }
 }
 
@@ -716,11 +777,11 @@ impl ColorWordsDiffOptions {
     }
 }
 
-fn show_color_words_diff_hunks(
+fn show_color_words_diff_hunks<T: AsRef<[u8]>>(
     formatter: &mut dyn Formatter,
-    [lefts, rights]: [&Merge<BString>; 2],
+    [lefts, rights]: [&Merge<T>; 2],
     options: &ColorWordsDiffOptions,
-    conflict_marker_style: ConflictMarkerStyle,
+    materialize_options: &ConflictMaterializeOptions,
 ) -> io::Result<()> {
     let line_number = DiffLineNumber { left: 1, right: 1 };
     let labels = ["removed", "added"];
@@ -731,8 +792,8 @@ fn show_color_words_diff_hunks(
     }
     match options.conflict {
         ConflictDiffMethod::Materialize => {
-            let left = materialize_merge_result_to_bytes(lefts, conflict_marker_style);
-            let right = materialize_merge_result_to_bytes(rights, conflict_marker_style);
+            let left = materialize_merge_result_to_bytes(lefts, materialize_options);
+            let right = materialize_merge_result_to_bytes(rights, materialize_options);
             let contents = [&left, &right].map(BStr::new);
             show_color_words_resolved_hunks(formatter, contents, line_number, labels, options)?;
         }
@@ -1192,13 +1253,13 @@ fn file_content_for_diff<T>(
 fn diff_content(
     path: &RepoPath,
     value: MaterializedTreeValue,
-    conflict_marker_style: ConflictMarkerStyle,
+    materialize_options: &ConflictMaterializeOptions,
 ) -> BackendResult<FileContent<BString>> {
     diff_content_with(
         path,
         value,
         |content| content,
-        |contents| materialize_merge_result_to_bytes(&contents, conflict_marker_style),
+        |contents| materialize_merge_result_to_bytes(&contents, materialize_options),
     )
 }
 
@@ -1279,8 +1340,12 @@ pub async fn show_color_words_diff(
     tree_diff: BoxStream<'_, CopiesTreeDiffEntry>,
     path_converter: &RepoPathUiConverter,
     options: &ColorWordsDiffOptions,
-    conflict_marker_style: ConflictMarkerStyle,
+    marker_style: ConflictMarkerStyle,
 ) -> Result<(), DiffRenderError> {
+    let materialize_options = ConflictMaterializeOptions {
+        marker_style,
+        marker_len: None,
+    };
     let empty_content = || Merge::resolved(BString::default());
     let mut diff_stream = materialized_diff_stream(store, tree_diff);
     while let Some(MaterializedTreeDiffEntry { path, values }) = diff_stream.next().await {
@@ -1325,7 +1390,7 @@ pub async fn show_color_words_diff(
                     formatter,
                     [&empty_content(), &right_content.contents],
                     options,
-                    conflict_marker_style,
+                    &materialize_options,
                 )?;
             }
         } else if right_value.is_present() {
@@ -1392,7 +1457,7 @@ pub async fn show_color_words_diff(
                     formatter,
                     [&left_content.contents, &right_content.contents],
                     options,
-                    conflict_marker_style,
+                    &materialize_options,
                 )?;
             }
         } else {
@@ -1411,7 +1476,7 @@ pub async fn show_color_words_diff(
                     formatter,
                     [&left_content.contents, &empty_content()],
                     options,
-                    conflict_marker_style,
+                    &materialize_options,
                 )?;
             }
         }
@@ -1426,15 +1491,19 @@ pub async fn show_file_by_file_diff(
     tree_diff: BoxStream<'_, CopiesTreeDiffEntry>,
     path_converter: &RepoPathUiConverter,
     tool: &ExternalMergeTool,
-    conflict_marker_style: ConflictMarkerStyle,
+    marker_style: ConflictMarkerStyle,
 ) -> Result<(), DiffRenderError> {
+    let materialize_options = ConflictMaterializeOptions {
+        marker_style,
+        marker_len: None,
+    };
     let create_file = |path: &RepoPath,
                        wc_dir: &Path,
                        value: MaterializedTreeValue|
      -> Result<PathBuf, DiffRenderError> {
         let fs_path = path.to_fs_path(wc_dir)?;
         std::fs::create_dir_all(fs_path.parent().unwrap())?;
-        let content = diff_content(path, value, conflict_marker_style)?;
+        let content = diff_content(path, value, &materialize_options)?;
         std::fs::write(&fs_path, content.contents)?;
         Ok(fs_path)
     };
@@ -1497,7 +1566,7 @@ struct GitDiffPart {
 fn git_diff_part(
     path: &RepoPath,
     value: MaterializedTreeValue,
-    conflict_marker_style: ConflictMarkerStyle,
+    materialize_options: &ConflictMaterializeOptions,
 ) -> Result<GitDiffPart, DiffRenderError> {
     const DUMMY_HASH: &str = "0000000000";
     let mode;
@@ -1551,7 +1620,7 @@ fn git_diff_part(
             hash = DUMMY_HASH.to_owned();
             content = FileContent {
                 is_binary: false, // TODO: are we sure this is never binary?
-                contents: materialize_merge_result_to_bytes(&file.contents, conflict_marker_style),
+                contents: materialize_merge_result_to_bytes(&file.contents, materialize_options),
             };
         }
         MaterializedTreeValue::OtherConflict { id } => {
@@ -1827,8 +1896,12 @@ pub async fn show_git_diff(
     store: &Store,
     tree_diff: BoxStream<'_, CopiesTreeDiffEntry>,
     options: &UnifiedDiffOptions,
-    conflict_marker_style: ConflictMarkerStyle,
+    marker_style: ConflictMarkerStyle,
 ) -> Result<(), DiffRenderError> {
+    let materialize_options = ConflictMaterializeOptions {
+        marker_style,
+        marker_len: None,
+    };
     let mut diff_stream = materialized_diff_stream(store, tree_diff);
     while let Some(MaterializedTreeDiffEntry { path, values }) = diff_stream.next().await {
         let left_path = path.source();
@@ -1837,8 +1910,8 @@ pub async fn show_git_diff(
         let right_path_string = right_path.as_internal_file_string();
         let (left_value, right_value) = values?;
 
-        let left_part = git_diff_part(left_path, left_value, conflict_marker_style)?;
-        let right_part = git_diff_part(right_path, right_value, conflict_marker_style)?;
+        let left_part = git_diff_part(left_path, left_value, &materialize_options)?;
+        let right_part = git_diff_part(right_path, right_value, &materialize_options)?;
 
         formatter.with_label("file_header", |formatter| {
             writeln!(
@@ -1915,6 +1988,29 @@ pub async fn show_git_diff(
     Ok(())
 }
 
+/// Generates diff of non-binary contents in Git format.
+fn show_git_diff_texts<T: AsRef<[u8]>>(
+    formatter: &mut dyn Formatter,
+    [left_path, right_path]: [&str; 2],
+    contents: [&Merge<T>; 2],
+    options: &UnifiedDiffOptions,
+    materialize_options: &ConflictMaterializeOptions,
+) -> io::Result<()> {
+    formatter.with_label("file_header", |formatter| {
+        writeln!(formatter, "diff --git a/{left_path} b/{right_path}")?;
+        writeln!(formatter, "--- {left_path}")?;
+        writeln!(formatter, "+++ {right_path}")
+    })?;
+    let [left, right] = contents.map(|content| match content.as_resolved() {
+        Some(text) => Cow::Borrowed(BStr::new(text)),
+        None => Cow::Owned(materialize_merge_result_to_bytes(
+            content,
+            materialize_options,
+        )),
+    });
+    show_unified_diff_hunks(formatter, [left.as_ref(), right.as_ref()], options)
+}
+
 #[instrument(skip_all)]
 pub async fn show_diff_summary(
     formatter: &mut dyn Formatter,
@@ -1977,18 +2073,18 @@ impl DiffStats {
         store: &Store,
         tree_diff: BoxStream<'_, CopiesTreeDiffEntry>,
         options: &DiffStatOptions,
-        conflict_marker_style: ConflictMarkerStyle,
+        marker_style: ConflictMarkerStyle,
     ) -> BackendResult<Self> {
+        let materialize_options = ConflictMaterializeOptions {
+            marker_style,
+            marker_len: None,
+        };
         let entries = materialized_diff_stream(store, tree_diff)
             .map(|MaterializedTreeDiffEntry { path, values }| {
                 let (left, right) = values?;
-                let left_content = diff_content(path.source(), left, conflict_marker_style)?;
-                let right_content = diff_content(path.target(), right, conflict_marker_style)?;
-                let stat = get_diff_stat_entry(
-                    path,
-                    [&left_content.contents, &right_content.contents].map(BStr::new),
-                    options,
-                );
+                let left_content = diff_content(path.source(), left, &materialize_options)?;
+                let right_content = diff_content(path.target(), right, &materialize_options)?;
+                let stat = get_diff_stat_entry(path, [&left_content, &right_content], options);
                 BackendResult::Ok(stat)
             })
             .try_collect()
@@ -2001,49 +2097,64 @@ impl DiffStats {
         &self.entries
     }
 
-    /// Total number of insertions.
+    /// Total number of inserted lines.
     pub fn count_total_added(&self) -> usize {
-        self.entries.iter().map(|stat| stat.added).sum()
+        self.entries
+            .iter()
+            .filter_map(|stat| stat.added_removed.map(|(added, _)| added))
+            .sum()
     }
 
-    /// Total number of deletions.
+    /// Total number of deleted lines.
     pub fn count_total_removed(&self) -> usize {
-        self.entries.iter().map(|stat| stat.removed).sum()
+        self.entries
+            .iter()
+            .filter_map(|stat| stat.added_removed.map(|(_, removed)| removed))
+            .sum()
     }
 }
 
 #[derive(Clone, Debug)]
 pub struct DiffStatEntry {
     pub path: CopiesTreeDiffEntryPath,
-    pub added: usize,
-    pub removed: usize,
+    /// Lines added and removed; None for binary files.
+    pub added_removed: Option<(usize, usize)>,
+    /// Change in file size in bytes.
+    pub bytes_delta: isize,
 }
 
 fn get_diff_stat_entry(
     path: CopiesTreeDiffEntryPath,
-    contents: [&BStr; 2],
+    contents: [&FileContent<BString>; 2],
     options: &DiffStatOptions,
 ) -> DiffStatEntry {
-    // TODO: this matches git's behavior, which is to count the number of newlines
-    // in the file. but that behavior seems unhelpful; no one really cares how
-    // many `0x0a` characters are in an image.
-    let diff = diff_by_line(contents, &options.line_diff);
-    let mut added = 0;
-    let mut removed = 0;
-    for hunk in diff.hunks() {
-        match hunk.kind {
-            DiffHunkKind::Matching => {}
-            DiffHunkKind::Different => {
-                let [left, right] = hunk.contents[..].try_into().unwrap();
-                removed += left.split_inclusive(|b| *b == b'\n').count();
-                added += right.split_inclusive(|b| *b == b'\n').count();
+    let [left_content, right_content] = contents;
+    let added_removed = if left_content.is_binary || right_content.is_binary {
+        None
+    } else {
+        let diff = diff_by_line(
+            contents.map(|content| &content.contents),
+            &options.line_diff,
+        );
+        let mut added = 0;
+        let mut removed = 0;
+        for hunk in diff.hunks() {
+            match hunk.kind {
+                DiffHunkKind::Matching => {}
+                DiffHunkKind::Different => {
+                    let [left, right] = hunk.contents[..].try_into().unwrap();
+                    removed += left.split_inclusive(|b| *b == b'\n').count();
+                    added += right.split_inclusive(|b| *b == b'\n').count();
+                }
             }
         }
-    }
+        Some((added, removed))
+    };
+
     DiffStatEntry {
         path,
-        added,
-        removed,
+        added_removed,
+        bytes_delta: right_content.contents.len() as isize - left_content.contents.len() as isize,
     }
 }
 
@@ -2067,60 +2178,110 @@ pub fn show_diff_stats(
 
     // Entries format like:
     //   path/to/file | 123 ++--
+    // or, for binary files:
+    //   path/to/file | (binary) +1234 bytes
     //
     // Depending on display widths, we can elide part of the path,
     // and the the ++-- bar will adjust its scale to fill the rest.
 
-    let max_path_width = ui_paths.iter().map(|s| s.width()).max().unwrap_or(0);
+    // Choose how many columns to use for the path.  The right side will use the
+    // rest. Start with the longest path.  The code below might shorten it.
+    let mut max_path_width = ui_paths.iter().map(|s| s.width()).max().unwrap_or(0);
+
+    // Fit to the available display width, but always assume at least a tiny bit of
+    // room.
+    let available_width = max(display_width.saturating_sub(" | ".len()), 8);
+
+    // Measure the widest right side for line diffs and reduce max_path_width if
+    // needed.
     let max_diffs = stats
         .entries()
         .iter()
-        .map(|stat| stat.added + stat.removed)
-        .max()
-        .unwrap_or(0);
+        .filter_map(|stat| {
+            let (added, removed) = stat.added_removed?;
+            Some(added + removed)
+        })
+        .max();
+    let diff_number_width = max_diffs.map_or(0, |n| n.to_string().len());
+    if max_diffs.is_some() {
+        let width = diff_number_width + " ".len();
+        // Ensure 30% of the space for displaying the ++-- graph:
+        max_path_width =
+            max_path_width.min((0.7 * available_width.saturating_sub(width) as f64) as usize);
+    };
 
-    let number_width = max_diffs.to_string().len();
-    // Available width to distribute between path and bar:
-    let available_width = display_width.saturating_sub(" | ".len() + number_width + " ".len());
-    // Always use at least a tiny bit of room
-    let available_width = max(available_width, 5);
+    // Measure widest right side for binary diffs and reduce max_path_width if
+    // needed.
+    let max_bytes = stats
+        .entries
+        .iter()
+        .filter(|stat| stat.added_removed.is_none())
+        .map(|stat| stat.bytes_delta.abs())
+        .max();
+    if let Some(max) = max_bytes {
+        let width = if max > 0 {
+            format!("(binary) {max:+} bytes").len()
+        } else {
+            "(binary)".len()
+        };
+        max_path_width = max_path_width.min(available_width.saturating_sub(width));
+    }
 
-    let max_path_width = max_path_width.min((0.7 * available_width as f64) as usize);
-    let max_bar_length = available_width.saturating_sub(max_path_width);
-    let factor = if max_diffs < max_bar_length {
-        1.0
-    } else {
-        max_bar_length as f64 / max_diffs as f64
+    // Now that we've chosen the path width, use the rest of the space for the ++--
+    // bar.
+    let max_bar_width =
+        available_width.saturating_sub(max_path_width + diff_number_width + " ".len());
+    let factor = match max_diffs {
+        Some(max) if max > max_bar_width => max_bar_width as f64 / max as f64,
+        _ => 1.0,
     };
 
     for (stat, ui_path) in iter::zip(stats.entries(), &ui_paths) {
-        let bar_length = ((stat.added + stat.removed) as f64 * factor) as usize;
-        // If neither adds nor removes are present, bar length should be zero.
-        // If only one is present, bar length should be at least 1.
-        // If both are present, bar length should be at least 2.
-        //
-        // Fractional space after scaling is given to whichever of adds/removes is
-        // smaller, to show at least one tick for small (but nonzero) counts.
-        let bar_length = bar_length.max((stat.added > 0) as usize + (stat.removed > 0) as usize);
-        let (bar_added, bar_removed) = if stat.added < stat.removed {
-            let len = (stat.added as f64 * factor).ceil() as usize;
-            (len, bar_length - len)
-        } else {
-            let len = (stat.removed as f64 * factor).ceil() as usize;
-            (bar_length - len, len)
-        };
         // replace start of path with ellipsis if the path is too long
         let (path, path_width) = text_util::elide_start(ui_path, "...", max_path_width);
         let path_pad_width = max_path_width - path_width;
         write!(
             formatter,
-            "{path}{:path_pad_width$} | {:>number_width$}{}",
+            "{path}{:path_pad_width$} | ",
             "", // pad to max_path_width
-            stat.added + stat.removed,
-            if bar_added + bar_removed > 0 { " " } else { "" },
         )?;
-        write!(formatter.labeled("added"), "{}", "+".repeat(bar_added))?;
-        writeln!(formatter.labeled("removed"), "{}", "-".repeat(bar_removed))?;
+        if let Some((added, removed)) = stat.added_removed {
+            let bar_length = ((added + removed) as f64 * factor) as usize;
+            // If neither adds nor removes are present, bar length should be zero.
+            // If only one is present, bar length should be at least 1.
+            // If both are present, bar length should be at least 2.
+            //
+            // Fractional space after scaling is given to whichever of adds/removes is
+            // smaller, to show at least one tick for small (but nonzero) counts.
+            let bar_length = bar_length.max((added > 0) as usize + (removed > 0) as usize);
+            let (bar_added, bar_removed) = if added < removed {
+                let len = (added as f64 * factor).ceil() as usize;
+                (len, bar_length - len)
+            } else {
+                let len = (removed as f64 * factor).ceil() as usize;
+                (bar_length - len, len)
+            };
+            write!(
+                formatter,
+                "{:>diff_number_width$}{}",
+                added + removed,
+                if bar_added + bar_removed > 0 { " " } else { "" },
+            )?;
+            write!(formatter.labeled("added"), "{}", "+".repeat(bar_added))?;
+            writeln!(formatter.labeled("removed"), "{}", "-".repeat(bar_removed))?;
+        } else {
+            write!(formatter.labeled("binary"), "(binary)")?;
+            if stat.bytes_delta != 0 {
+                let label = if stat.bytes_delta < 0 {
+                    "removed"
+                } else {
+                    "added"
+                };
+                write!(formatter.labeled(label), " {:+}", stat.bytes_delta)?;
+                write!(formatter, " bytes")?;
+            }
+            writeln!(formatter)?;
+        }
     }
 
     let total_added = stats.count_total_added();
