@@ -17,7 +17,7 @@ mod config_schema_defaults;
 mod test_environment;
 
 pub use self::command_output::CommandOutput;
-pub use self::config_schema_defaults::default_toml_from_schema;
+pub use self::config_schema_defaults::default_config_from_schema;
 pub use self::test_environment::TestEnvironment;
 pub use self::test_environment::TestWorkDir;
 
