@@ -20,6 +20,7 @@ use std::path::PathBuf;
 
 use bstr::BString;
 use indoc::formatdoc;
+use itertools::Itertools as _;
 use regex::Captures;
 use regex::Regex;
 use tempfile::TempDir;
@@ -198,20 +199,6 @@ impl TestEnvironment {
         self.env_vars.insert(key.into(), val.into());
     }
 
-    // TODO: Remove with the `git.subprocess` setting.
-    pub fn with_git_subprocess(self, subprocess: bool) -> Self {
-        #[cfg(feature = "git2")]
-        if !subprocess {
-            self.add_config(formatdoc! {"
-                git.subprocess = false
-                debug.suppress-git2-deprecation-warning = true
-            "});
-            return self;
-        }
-        assert!(subprocess);
-        self
-    }
-
     /// Sets up the fake editor to read an edit script from the returned path
     /// Also sets up the fake editor as a merge tool named "fake-editor"
     pub fn set_up_fake_editor(&mut self) -> PathBuf {
@@ -302,12 +289,20 @@ impl TestWorkDir<'_> {
         }
     }
 
+    /// Reads the current operation id without resolving divergence nor
+    /// snapshotting. `TestWorkDir` must be at the workspace root.
     #[track_caller]
     pub fn current_operation_id(&self) -> String {
-        let output = self
-            .run_jj(["debug", "operation", "--display=id"])
-            .success();
-        output.stdout.raw().trim_end().to_owned()
+        let heads_dir = self
+            .root()
+            .join(PathBuf::from_iter([".jj", "repo", "op_heads", "heads"]));
+        let head_entry = heads_dir
+            .read_dir()
+            .expect("TestWorkDir must point to the workspace root")
+            .exactly_one()
+            .expect("divergence not supported")
+            .unwrap();
+        head_entry.file_name().into_string().unwrap()
     }
 
     /// Returns test helper for the specified sub directory.
