@@ -52,8 +52,10 @@ mod test_help_command;
 mod test_immutable_commits;
 mod test_interdiff_command;
 mod test_log_command;
+mod test_metaedit_command;
 mod test_new_command;
 mod test_next_prev_commands;
+mod test_op_revert_command;
 mod test_operations;
 mod test_parallelize_command;
 mod test_rebase_command;
@@ -72,7 +74,7 @@ mod test_squash_command;
 mod test_status_command;
 mod test_tag_command;
 mod test_templater;
-mod test_undo;
+mod test_undo_redo_commands;
 mod test_util_command;
 mod test_working_copy;
 mod test_workspaces;
