@@ -128,42 +128,29 @@ fn test_absorb_simple() {
     ");
     insta::assert_snapshot!(get_evolog(&work_dir, "description(1)"), @r"
     ○    kkmpptxz 5810eb0f 1
-    ├─╮  -- operation 5876e0f3d35d (2001-02-03 08:05:14) absorb changes into 1 commits
+    ├─╮
     │ ○  yqosqzyt hidden 39b42898 (no description set)
-    │ │  -- operation a2c449e239df (2001-02-03 08:05:14) snapshot working copy
     │ ○  yqosqzyt hidden 977269ac (empty) (no description set)
-    │    -- operation 4a9cb11bbdd5 (2001-02-03 08:05:13) absorb changes into 2 commits
     ○    kkmpptxz hidden bd7d4016 1
-    ├─╮  -- operation 4a9cb11bbdd5 (2001-02-03 08:05:13) absorb changes into 2 commits
+    ├─╮
     │ ○  mzvwutvl hidden 0b307741 (no description set)
-    │ │  -- operation 51ebffcb116e (2001-02-03 08:05:13) snapshot working copy
     │ ○  mzvwutvl hidden f2709b4e (empty) (no description set)
-    │    -- operation b92e661fdac1 (2001-02-03 08:05:11) new empty commit
     ○  kkmpptxz hidden 1553c5e8 1
-    │  -- operation 35926ea345b0 (2001-02-03 08:05:10) snapshot working copy
     ○  kkmpptxz hidden eb943711 (empty) 1
-       -- operation da1318a72167 (2001-02-03 08:05:09) new empty commit
     [EOF]
     ");
     insta::assert_snapshot!(get_evolog(&work_dir, "description(2)"), @r"
     ○    zsuskuln dd109863 2
-    ├─╮  -- operation fc078244f126 (2001-02-03 08:05:15) absorb changes into 1 commits
+    ├─╮
     │ ○  vruxwmqv hidden 761492a8 (no description set)
-    │ │  -- operation b15694dee324 (2001-02-03 08:05:15) snapshot working copy
     │ ○  vruxwmqv hidden 48c7d8fa (empty) (no description set)
-    │    -- operation 5876e0f3d35d (2001-02-03 08:05:14) absorb changes into 1 commits
     ○  zsuskuln hidden 8edd60a2 2
-    │  -- operation 5876e0f3d35d (2001-02-03 08:05:14) absorb changes into 1 commits
     ○    zsuskuln hidden 95568809 2
-    ├─╮  -- operation 4a9cb11bbdd5 (2001-02-03 08:05:13) absorb changes into 2 commits
+    ├─╮
     │ ○  mzvwutvl hidden 0b307741 (no description set)
-    │ │  -- operation 51ebffcb116e (2001-02-03 08:05:13) snapshot working copy
     │ ○  mzvwutvl hidden f2709b4e (empty) (no description set)
-    │    -- operation b92e661fdac1 (2001-02-03 08:05:11) new empty commit
     ○  zsuskuln hidden 36fad385 2
-    │  -- operation ab92cd8883ff (2001-02-03 08:05:11) snapshot working copy
     ○  zsuskuln hidden 561fbce9 (empty) 2
-       -- operation 9339ac427cfc (2001-02-03 08:05:10) new empty commit
     [EOF]
     ");
 }
@@ -962,6 +949,6 @@ fn get_diffs(work_dir: &TestWorkDir, revision: &str) -> CommandOutput {
 
 #[must_use]
 fn get_evolog(work_dir: &TestWorkDir, revision: &str) -> CommandOutput {
-    let template = r#"format_commit_summary_with_refs(self, "") ++ "\n""#;
+    let template = r#"format_commit_summary_with_refs(commit, "") ++ "\n""#;
     work_dir.run_jj(["evolog", "-r", revision, "-T", template])
 }
