@@ -56,6 +56,8 @@ fn test_commit_with_editor() {
         std::fs::read_to_string(test_env.env_root().join("editor0")).unwrap(), @r#"
     initial
 
+    JJ: Change ID: qpvuntsm
+    JJ:
     JJ: Lines starting with "JJ:" (like this one) will be removed.
     "#);
 
@@ -69,6 +71,7 @@ fn test_commit_with_editor() {
         std::fs::read_to_string(test_env.env_root().join("editor1")).unwrap(), @r#"
     add files
 
+    JJ: Change ID: kkmpptxz
     JJ: This commit contains the following changes:
     JJ:     A file1
     JJ:     A file2
@@ -134,8 +137,11 @@ fn test_commit_with_empty_description_from_editor() {
     ");
     insta::assert_snapshot!(
         std::fs::read_to_string(test_env.env_root().join("editor0")).unwrap(),
-        @r#"JJ: Lines starting with "JJ:" (like this one) will be removed."#
-    );
+        @r#"
+    JJ: Change ID: qpvuntsm
+    JJ:
+    JJ: Lines starting with "JJ:" (like this one) will be removed.
+    "#);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Hint: The commit message was left empty.
@@ -162,6 +168,7 @@ fn test_commit_interactive() {
 
     let diff_script = ["rm file2", "dump JJ-INSTRUCTIONS instrs"].join("\0");
     std::fs::write(diff_editor, diff_script).unwrap();
+    let setup_opid = work_dir.current_operation_id();
 
     // Create a commit interactively and select only file1
     work_dir.run_jj(["commit", "-i"]).success();
@@ -179,6 +186,7 @@ fn test_commit_interactive() {
         std::fs::read_to_string(test_env.env_root().join("editor")).unwrap(), @r#"
     add files
 
+    JJ: Change ID: qpvuntsm
     JJ: This commit contains the following changes:
     JJ:     A file1
     JJ:
@@ -186,7 +194,7 @@ fn test_commit_interactive() {
     "#);
 
     // Try again with --tool=<name>, which implies --interactive
-    work_dir.run_jj(["undo"]).success();
+    work_dir.run_jj(["op", "restore", &setup_opid]).success();
     work_dir
         .run_jj([
             "commit",
@@ -199,6 +207,7 @@ fn test_commit_interactive() {
         std::fs::read_to_string(test_env.env_root().join("editor")).unwrap(), @r#"
     add files
 
+    JJ: Change ID: qpvuntsm
     JJ: This commit contains the following changes:
     JJ:     A file1
     JJ:
@@ -255,6 +264,7 @@ fn test_commit_interactive_with_paths() {
         std::fs::read_to_string(test_env.env_root().join("editor")).unwrap(), @r#"
     edit
 
+    JJ: Change ID: rlvkpnrz
     JJ: This commit contains the following changes:
     JJ:     A file1
     JJ:
@@ -305,6 +315,7 @@ fn test_commit_with_default_description() {
         std::fs::read_to_string(test_env.env_root().join("editor")).unwrap(), @r#"
     TESTED=TODO
 
+    JJ: Change ID: qpvuntsm
     JJ: This commit contains the following changes:
     JJ:     A file1
     JJ:     A file2
@@ -564,6 +575,8 @@ fn test_commit_trailers() {
 
     Reviewed-by: foo@bar.org
 
+    JJ: Change ID: zsuskuln
+    JJ:
     JJ: Lines starting with "JJ:" (like this one) will be removed.
     -----
     "#);
