@@ -977,7 +977,7 @@ fn test_operations() {
     [EOF]
     ");
 
-    let output = work_dir.complete_fish(["op", "undo", "8e"]);
+    let output = work_dir.complete_fish(["op", "revert", "8e"]);
     insta::assert_snapshot!(output, @r"
     8ed8c16786e6	(2001-02-03 08:05:11) describe commit 3725536d0ae06d69e46911258cee591dbdb66478
     [EOF]
@@ -1066,6 +1066,7 @@ fn test_template_alias() {
     builtin_config_list
     builtin_config_list_detailed
     builtin_draft_commit_description
+    builtin_evolog_compact
     builtin_log_comfortable
     builtin_log_compact
     builtin_log_compact_full_description
@@ -1073,11 +1074,13 @@ fn test_template_alias() {
     builtin_log_node
     builtin_log_node_ascii
     builtin_log_oneline
+    builtin_log_redacted
     builtin_op_log_comfortable
     builtin_op_log_compact
     builtin_op_log_node
     builtin_op_log_node_ascii
     builtin_op_log_oneline
+    builtin_op_log_redacted
     commit_summary_separator
     default_commit_description
     description_placeholder
@@ -1467,3 +1470,25 @@ fn test_files() {
     [EOF]
     ");
 }
+
+#[test]
+fn test_command_alias_with_exec() {
+    let test_env = TestEnvironment::default();
+    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
+    let work_dir = test_env.work_dir("repo");
+
+    test_env.add_config(r#"aliases.my-script = ["util", "exec", "--", "my-jj-script"]"#);
+
+    work_dir.write_file("file1", "contents");
+    work_dir.write_file("file2", "contents");
+    work_dir.create_dir("folder");
+    work_dir.write_file("folder/subfile", "contents");
+
+    let output = work_dir.complete_fish(["my-script", "f"]);
+    insta::assert_snapshot!(output.normalize_backslash(), @r"
+    file1
+    file2
+    folder/
+    [EOF]
+    ");
+}
