@@ -20,7 +20,7 @@ use itertools::Itertools as _;
 use regex::Regex;
 
 use crate::common::TestEnvironment;
-use crate::common::default_toml_from_schema;
+use crate::common::default_config_from_schema;
 use crate::common::fake_editor_path;
 use crate::common::force_interactive;
 use crate::common::to_toml_value;
@@ -233,7 +233,6 @@ fn test_config_list_layer() {
 
     let output = work_dir.run_jj(["config", "list", "--user"]);
     insta::assert_snapshot!(output, @r#"
-    "$schema" = "https://jj-vcs.github.io/jj/latest/config-schema.json"
     test-key = "test-val"
     test-layered-key = "test-original-val"
     [EOF]
@@ -258,7 +257,6 @@ fn test_config_list_layer() {
 
     let output = work_dir.run_jj(["config", "list", "--repo"]);
     insta::assert_snapshot!(output, @r#"
-    "$schema" = "https://jj-vcs.github.io/jj/latest/config-schema.json"
     test-layered-key = "test-layered-val"
     [EOF]
     "#);
@@ -308,7 +306,6 @@ fn test_config_list_origin() {
     ]);
     insta::assert_snapshot!(output, @r#"
     test-key = "test-val" # user $TEST_ENV/config/config.toml
-    "$schema" = "https://jj-vcs.github.io/jj/latest/config-schema.json" # repo $TEST_ENV/repo/.jj/repo/config.toml
     test-layered-key = "test-layered-val" # repo $TEST_ENV/repo/.jj/repo/config.toml
     user.name = "Test User" # env
     user.email = "test.user@example.com" # env
@@ -336,6 +333,27 @@ fn test_config_list_origin() {
     [38;5;2m<<config_list name::test-key>>[39m<<config_list:: = >>[38;5;3m<<config_list value::"test-cli-val">>[39m<<config_list:: # >>[38;5;4m<<config_list source::cli>>[39m<<config_list::>>
     [EOF]
     "#);
+
+    let output = work_dir.run_jj([
+        "config",
+        "list",
+        r#"-Tjson(self) ++ "\n""#,
+        "--include-defaults",
+        "--include-overridden",
+        "--config=test-key=test-cli-val",
+        "test-key",
+    ]);
+    insta::with_settings!({
+        // Windows paths will be escaped in JSON syntax, which cannot be
+        // normalized as $TEST_ENV.
+        filters => [(r#""path":"[^"]*","#, r#""path":"<redacted>","#)],
+    }, {
+        insta::assert_snapshot!(output, @r#"
+        {"name":"test-key","value":"test-val","source":"user","path":"<redacted>","is_overridden":true}
+        {"name":"test-key","value":"test-cli-val","source":"cli","path":null,"is_overridden":false}
+        [EOF]
+        "#);
+    });
 }
 
 #[test]
@@ -554,7 +572,7 @@ fn test_config_set_bad_opts() {
       |
     1 | 
       | ^
-    invalid key
+    unquoted keys cannot be empty, expected letters, numbers, `-`, `_`
 
 
     For more information, try '--help'.
@@ -569,8 +587,7 @@ fn test_config_set_bad_opts() {
       |
     1 | ['typo'}
       |        ^
-    invalid array
-    expected `]`
+    missing comma between array elements, expected `,`
 
 
     For more information, try '--help'.
@@ -602,7 +619,8 @@ fn test_config_set_for_user() {
     let user_config_toml = std::fs::read_to_string(&user_config_path)
         .unwrap_or_else(|_| panic!("Failed to read file {}", user_config_path.display()));
     insta::assert_snapshot!(user_config_toml, @r#"
-    "$schema" = "https://jj-vcs.github.io/jj/latest/config-schema.json"
+    #:schema https://jj-vcs.github.io/jj/latest/config-schema.json
+
     test-key = "test-val"
 
     [test-table]
@@ -669,7 +687,8 @@ fn test_config_set_for_repo() {
     // Ensure test-key successfully written to user config.
     let repo_config_toml = work_dir.read_file(".jj/repo/config.toml");
     insta::assert_snapshot!(repo_config_toml, @r#"
-    "$schema" = "https://jj-vcs.github.io/jj/latest/config-schema.json"
+    #:schema https://jj-vcs.github.io/jj/latest/config-schema.json
+
     test-key = "test-val"
 
     [test-table]
@@ -698,7 +717,7 @@ fn test_config_set_toml_types() {
     set_value("test-table.string", r#""foo""#);
     set_value("test-table.invalid", r"a + b");
     insta::assert_snapshot!(std::fs::read_to_string(&user_config_path).unwrap(), @r#"
-    "$schema" = "https://jj-vcs.github.io/jj/latest/config-schema.json"
+    #:schema https://jj-vcs.github.io/jj/latest/config-schema.json
 
     [test-table]
     integer = 42
@@ -793,10 +812,11 @@ fn test_config_unset_inline_table_key() {
         .run_jj(["config", "unset", "--user", "inline-table.foo"])
         .success();
     let user_config_toml = std::fs::read_to_string(&user_config_path).unwrap();
-    insta::assert_snapshot!(user_config_toml, @r#"
-    "$schema" = "https://jj-vcs.github.io/jj/latest/config-schema.json"
+    insta::assert_snapshot!(user_config_toml, @r"
+    #:schema https://jj-vcs.github.io/jj/latest/config-schema.json
+
     inline-table = {}
-    "#);
+    ");
 }
 
 #[test]
@@ -868,11 +888,7 @@ fn test_config_unset_for_user() {
         .success();
 
     let user_config_toml = std::fs::read_to_string(&user_config_path).unwrap();
-    insta::assert_snapshot!(user_config_toml, @r#"
-    "$schema" = "https://jj-vcs.github.io/jj/latest/config-schema.json"
-
-    [table]
-    "#);
+    insta::assert_snapshot!(user_config_toml, @"[table]");
 }
 
 #[test]
@@ -889,7 +905,7 @@ fn test_config_unset_for_repo() {
         .success();
 
     let repo_config_toml = work_dir.read_file(".jj/repo/config.toml");
-    insta::assert_snapshot!(repo_config_toml, @r#""$schema" = "https://jj-vcs.github.io/jj/latest/config-schema.json""#);
+    insta::assert_snapshot!(repo_config_toml, @"");
 }
 
 #[test]
@@ -1066,7 +1082,7 @@ fn test_config_edit_invalid_config() {
       |
     1 | invalid config here
       |         ^
-    expected `.`, `=`
+    key with no value, expected `=`
 
     Do you want to keep editing the file? If not, previous config will be restored. (Yn): [EOF]
     ");
@@ -1095,7 +1111,7 @@ fn test_config_edit_invalid_config() {
       |
     1 | invalid config here
       |         ^
-    expected `.`, `=`
+    key with no value, expected `=`
 
     Do you want to keep editing the file? If not, previous config will be restored. (Yn): [EOF]
     ");
@@ -1263,29 +1279,31 @@ fn test_config_get_yields_values_consistent_with_schema_defaults() {
     let test_env = TestEnvironment::default();
     let get_true_default = move |key: &str| {
         let output = test_env.run_jj_in(".", ["config", "get", key]).success();
-        let output_doc =
-            toml_edit::ImDocument::parse(format!("test={}", output.stdout.normalized()))
-                .unwrap_or_else(|_| {
-                    // Unfortunately for this test, `config get` is "lossy" and does not print
-                    // quoted strings. This means that e.g. `false` and `"false"` are not
-                    // distinguishable. If value couldn't be parsed, it's probably a string, so
-                    // let's parse its Debug string instead.
-                    toml_edit::ImDocument::parse(format!(
-                        "test={:?}",
-                        output.stdout.normalized().trim()
-                    ))
+        let output_doc = toml_edit::Document::parse(format!("test={}", output.stdout.normalized()))
+            .unwrap_or_else(|_| {
+                // Unfortunately for this test, `config get` is "lossy" and does not print
+                // quoted strings. This means that e.g. `false` and `"false"` are not
+                // distinguishable. If value couldn't be parsed, it's probably a string, so
+                // let's parse its Debug string instead.
+                toml_edit::Document::parse(format!("test={:?}", output.stdout.normalized().trim()))
                     .unwrap()
-                });
+            });
         output_doc.get("test").unwrap().as_value().unwrap().clone()
     };
 
-    let Some(schema_defaults) = default_toml_from_schema() else {
-        testutils::ensure_running_outside_ci("`jq` must be in the PATH");
-        eprintln!("Skipping test because jq is not installed on the system");
-        return;
-    };
+    let mut schema_defaults = toml_edit::ser::to_document(&default_config_from_schema()).unwrap();
 
-    for (key, schema_default) in schema_defaults.as_table().get_values() {
+    // Ensure that `get_values()` flattens the entire configuration.
+    struct SetDotted;
+    impl toml_edit::visit_mut::VisitMut for SetDotted {
+        fn visit_table_like_mut(&mut self, node: &mut dyn toml_edit::TableLike) {
+            node.set_dotted(true);
+            toml_edit::visit_mut::visit_table_like_mut(self, node);
+        }
+    }
+    toml_edit::visit_mut::visit_document_mut(&mut SetDotted, &mut schema_defaults);
+
+    for (key, schema_default) in schema_defaults.into_table().get_values() {
         let key = key.iter().join(".");
         match key.as_str() {
             // These keys technically don't have a default value, but they exhibit a default
@@ -1312,7 +1330,7 @@ fn test_config_get_yields_values_consistent_with_schema_defaults() {
             // The `immutable_heads()` revset actually defaults to `builtin_immutable_heads()` but
             // this would be a poor starting point for a custom revset, so the schema "inlines"
             // `builtin_immutable_heads()`.
-            "revset-aliases.'immutable_heads()'" => {
+            r#"revset-aliases."immutable_heads()""# => {
                 let builtin_default =
                     get_true_default("revset-aliases.'builtin_immutable_heads()'");
                 assert!(
@@ -1421,7 +1439,7 @@ fn test_config_path_syntax() {
       |
     1 | .
       | ^
-    invalid key
+    unquoted keys cannot be empty, expected letters, numbers, `-`, `_`
 
 
     For more information, try '--help'.
@@ -1437,7 +1455,7 @@ fn test_config_path_syntax() {
       |
     1 | b c
       |   ^
-
+    unexpected content, expected nothing
 
 
     For more information, try '--help'.
@@ -1451,7 +1469,7 @@ fn test_config_path_syntax() {
       |
     1 | 
       | ^
-    invalid key
+    unquoted keys cannot be empty, expected letters, numbers, `-`, `_`
 
 
     For more information, try '--help'.
