@@ -136,12 +136,11 @@ fn test_config_multiple_tools_with_same_name() {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Config error: Configuration cannot be parsed as TOML document
-    Caused by: TOML parse error at line 6, column 9
+    Caused by: TOML parse error at line 6, column 20
       |
     6 |         [fix.tools.my-tool]
-      |         ^
-    invalid table header
-    duplicate key `my-tool` in table `fix.tools`
+      |                    ^^^^^^^
+    duplicate key
 
     Hint: Check the config file: $TEST_ENV/config/config0002.toml
     For help, see https://jj-vcs.github.io/jj/latest/config/ or use `jj help -k config`.
@@ -540,13 +539,15 @@ fn test_relative_tool_path_from_subdirectory() {
     work_dir.write_file("test.txt", "hello world\n");
     let sub_dir = work_dir.create_dir("subdir");
     work_dir.write_file("subdir/nested.txt", "nested content\n");
+    work_dir.run_jj(["debug", "snapshot"]).success();
+    let setup_opid = work_dir.current_operation_id();
 
     // Run fix from workspace root
     let output = work_dir.run_jj(["fix"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Fixed 1 commits of 1 checked.
-    Working copy  (@) now at: qpvuntsm 58961e98 (no description set)
+    Working copy  (@) now at: qpvuntsm 57a27b36 (no description set)
     Parent commit (@-)      : zzzzzzzz 00000000 (empty) (no description set)
     Added 0 files, modified 2 files, removed 0 files
     [EOF]
@@ -564,14 +565,14 @@ fn test_relative_tool_path_from_subdirectory() {
     "###);
 
     // Reset so the fix tools should have an effect again
-    work_dir.run_jj(["undo"]).success();
+    work_dir.run_jj(["op", "restore", &setup_opid]).success();
 
     // Run fix from the subdirectory
     let output = sub_dir.run_jj(["fix"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Fixed 1 commits of 1 checked.
-    Working copy  (@) now at: qpvuntsm 380c1b78 (no description set)
+    Working copy  (@) now at: qpvuntsm 05404d5b (no description set)
     Parent commit (@-)      : zzzzzzzz 00000000 (empty) (no description set)
     Added 0 files, modified 2 files, removed 0 files
     [EOF]
