@@ -12,6 +12,8 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+use std::io::Write as _;
+
 use testutils::git;
 
 use crate::common::CommandOutput;
@@ -19,14 +21,22 @@ use crate::common::TestEnvironment;
 use crate::common::TestWorkDir;
 use crate::common::create_commit;
 
-fn add_commit_to_branch(git_repo: &gix::Repository, branch: &str) -> gix::ObjectId {
+fn add_commit_to_branch(git_repo: &gix::Repository, branch: &str, message: &str) -> gix::ObjectId {
+    // Get current commit ID of the branch if it exists
+    let parents = git_repo
+        .find_reference(&format!("refs/heads/{branch}"))
+        .ok()
+        .and_then(|mut r| r.peel_to_commit().ok())
+        .map(|c| vec![c.id().detach()])
+        .unwrap_or_default();
+
     git::add_commit(
         git_repo,
         &format!("refs/heads/{branch}"),
         branch,            // filename
         branch.as_bytes(), // content
-        "message",
-        &[],
+        message,
+        &parents,
     )
     .commit_id
 }
@@ -35,7 +45,7 @@ fn add_commit_to_branch(git_repo: &gix::Repository, branch: &str) -> gix::Object
 fn init_git_remote(test_env: &TestEnvironment, remote: &str) -> gix::Repository {
     let git_repo_path = test_env.env_root().join(remote);
     let git_repo = git::init(git_repo_path);
-    add_commit_to_branch(&git_repo, remote);
+    add_commit_to_branch(&git_repo, remote, "message");
 
     git_repo
 }
@@ -220,6 +230,54 @@ fn test_git_fetch_multiple_remotes() {
     ");
 }
 
+#[test]
+fn test_git_fetch_with_ignored_refspecs() {
+    let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-bookmark = true");
+    test_env
+        .run_jj_in(".", ["git", "init", "--colocate", "repo"])
+        .success();
+    let source_repo = init_git_remote(&test_env, "origin");
+
+    for branch in ["main", "foo", "foobar", "foobaz", "bar"] {
+        add_commit_to_branch(&source_repo, &format!("refs/heads/{branch}"), branch);
+    }
+
+    let work_dir = test_env.work_dir("repo");
+    std::fs::OpenOptions::new()
+        .append(true)
+        .open(work_dir.root().join("./.git/config"))
+        .expect("failed to open config file")
+        .write_all(
+            br#"
+            [remote "origin"]
+            url = ../origin/.git
+            fetch = +refs/heads/main:refs/remotes/origin/main
+            fetch = +refs/heads/foo*:refs/remotes/origin/baz*
+            fetch = +refs/heads/bar*:refs/tags/bar*
+            fetch = refs/heads/bar
+            "#,
+        )
+        .expect("failed to update config file");
+
+    let output = work_dir.run_jj(["git", "fetch"]).success();
+
+    insta::assert_snapshot!(output.stdout, @r"");
+    insta::assert_snapshot!(output.stderr, @r"
+    Warning: Ignored refspec `refs/heads/bar` from `origin`: fetch-only refspecs are not supported
+    Warning: Ignored refspec `+refs/heads/bar*:refs/tags/bar*` from `origin`: only refs/remotes/ is supported for fetch destinations
+    Warning: Ignored refspec `+refs/heads/foo*:refs/remotes/origin/baz*` from `origin`: renaming is not supported
+    bookmark: main@origin [new] tracked
+    [EOF]
+    ");
+    insta::assert_snapshot!(get_bookmark_output(&work_dir), @r"
+    main: kpukqsrq f803461e main
+      @git: kpukqsrq f803461e main
+      @origin: kpukqsrq f803461e main
+    [EOF]
+    ");
+}
+
 #[test]
 fn test_git_fetch_with_glob() {
     let test_env = TestEnvironment::default();
@@ -1185,7 +1243,7 @@ fn test_git_fetch_undo() {
     let output = target_dir.run_jj(["undo"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Undid operation: 158b589e0e15 (2001-02-03 08:05:18) fetch from git remote(s) origin
+    Restored to operation: 8f47435a3990 (2001-02-03 08:05:07) add workspace 'default'
     [EOF]
     ");
     // The undo works as expected
@@ -1674,7 +1732,7 @@ fn test_git_fetch_preserve_commits_across_repos() {
         .success();
 
     // add commit to fork remote in another branch
-    add_commit_to_branch(&fork_repo, "feature");
+    add_commit_to_branch(&fork_repo, "feature", "message");
 
     // fetch remote bookmarks
     work_dir
@@ -1755,3 +1813,225 @@ fn test_git_fetch_preserve_commits_across_repos() {
     [EOF]
     ");
 }
+
+#[test]
+fn test_git_fetch_tracked() {
+    let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-bookmark = true");
+
+    // Set up a remote with multiple bookmarks
+    let remote_path = test_env.env_root().join("remote");
+    let remote_repo = git::init(remote_path.clone());
+    add_commit_to_branch(&remote_repo, "main", "message");
+    add_commit_to_branch(&remote_repo, "feature1", "message");
+    add_commit_to_branch(&remote_repo, "feature2", "message");
+
+    // Initialize jj repo
+    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
+    let work_dir = test_env.work_dir("repo");
+
+    // Add the remote to the jj repo
+    work_dir
+        .run_jj(["git", "remote", "add", "origin", "../remote"])
+        .success();
+
+    // Initially fetch only main and feature1
+    work_dir
+        .run_jj(["git", "fetch", "--branch", "main", "--branch", "feature1"])
+        .success();
+
+    // Both should be tracked
+    insta::assert_snapshot!(get_bookmark_output(&work_dir), @r###"
+    feature1: txqvqkwm fc8f3f42 message
+      @origin: txqvqkwm fc8f3f42 message
+    main: kmpysrkw 0130f303 message
+      @origin: kmpysrkw 0130f303 message
+    [EOF]
+    "###);
+
+    // Now untrack feature1
+    work_dir
+        .run_jj(["bookmark", "untrack", "feature1@origin"])
+        .success();
+
+    // Verify feature1 is untracked
+    insta::assert_snapshot!(get_bookmark_output(&work_dir), @r###"
+    feature1: txqvqkwm fc8f3f42 message
+    feature1@origin: txqvqkwm fc8f3f42 message
+    main: kmpysrkw 0130f303 message
+      @origin: kmpysrkw 0130f303 message
+    [EOF]
+    "###);
+
+    // Add new commits to all bookmarks on the remote
+    add_commit_to_branch(&remote_repo, "main", "message");
+    add_commit_to_branch(&remote_repo, "feature1", "message");
+    add_commit_to_branch(&remote_repo, "feature2", "message");
+
+    // Fetch with --tracked should only update main (which is still tracked)
+    work_dir.run_jj(["git", "fetch", "--tracked"]).success();
+
+    // Main should be updated to the new commit, but feature1 should remain
+    // unchanged
+    insta::assert_snapshot!(get_bookmark_output(&work_dir), @r"
+    feature1: txqvqkwm fc8f3f42 message
+    feature1@origin: txqvqkwm fc8f3f42 message
+    main: kmktnoqm 381bf13c (empty) message
+      @origin: kmktnoqm 381bf13c (empty) message
+    [EOF]
+    ");
+
+    // Now fetch all branches
+    work_dir
+        .run_jj(["git", "fetch", "--branch", "glob:*"])
+        .success();
+
+    // Now feature1@origin gets updated but feature1 stays at old commit
+    // (untracked), feature2 appears for the first time, and main stays at its
+    // already-updated commit
+    insta::assert_snapshot!(get_bookmark_output(&work_dir), @r"
+    feature1: txqvqkwm fc8f3f42 message
+    feature1@origin: ksswsvzv 0c0873bb (empty) message
+    feature2: ruyplonr 13e64e92 (empty) message
+      @origin: ruyplonr 13e64e92 (empty) message
+    main: kmktnoqm 381bf13c (empty) message
+      @origin: kmktnoqm 381bf13c (empty) message
+    [EOF]
+    ");
+}
+
+#[test]
+fn test_git_fetch_tracked_no_tracked_bookmarks() {
+    let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-bookmark = true");
+
+    // Set up a remote with bookmarks
+    let remote_path = test_env.env_root().join("remote");
+    let remote_repo = git::init(remote_path.clone());
+    add_commit_to_branch(&remote_repo, "main", "message");
+    add_commit_to_branch(&remote_repo, "feature", "message");
+
+    // Initialize jj repo
+    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
+    let work_dir = test_env.work_dir("repo");
+
+    // Add the remote to the jj repo
+    work_dir
+        .run_jj(["git", "remote", "add", "origin", "../remote"])
+        .success();
+
+    // Initially fetch bookmarks
+    work_dir.run_jj(["git", "fetch"]).success();
+
+    // Untrack all bookmarks
+    work_dir
+        .run_jj(["bookmark", "untrack", "glob:*@origin"])
+        .success();
+
+    // Fetch with --tracked should indicate nothing changed
+    let output = work_dir.run_jj(["git", "fetch", "--tracked"]);
+    insta::assert_snapshot!(output, @r#"
+    ------- stderr -------
+    Nothing changed.
+    [EOF]
+    "#);
+}
+
+#[test]
+fn test_git_fetch_tracked_multiple_remotes() {
+    let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-bookmark = true");
+
+    // Set up two remotes with different branches
+    let origin_path = test_env.env_root().join("origin");
+    let origin_repo = git::init(origin_path.clone());
+    add_commit_to_branch(&origin_repo, "main", "origin main commit");
+    add_commit_to_branch(&origin_repo, "feature1", "origin feature1 commit");
+    add_commit_to_branch(&origin_repo, "feature2", "origin feature2 commit");
+
+    let upstream_path = test_env.env_root().join("upstream");
+    let upstream_repo = git::init(upstream_path.clone());
+    add_commit_to_branch(&upstream_repo, "main", "upstream main commit");
+    add_commit_to_branch(&upstream_repo, "develop", "upstream develop commit");
+    add_commit_to_branch(&upstream_repo, "hotfix", "upstream hotfix commit");
+
+    // Initialize jj repo
+    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
+    let work_dir = test_env.work_dir("repo");
+
+    // Add both remotes
+    work_dir
+        .run_jj(["git", "remote", "add", "origin", "../origin"])
+        .success();
+    work_dir
+        .run_jj(["git", "remote", "add", "upstream", "../upstream"])
+        .success();
+
+    // Initial fetch from both remotes to set up tracking
+    work_dir.run_jj(["git", "fetch", "--all-remotes"]).success();
+
+    // Track different branches from different remotes
+    work_dir
+        .run_jj(["bookmark", "track", "feature1@origin"])
+        .success();
+    work_dir
+        .run_jj(["bookmark", "track", "develop@upstream"])
+        .success();
+
+    // Untrack some branches to test --tracked behavior
+    work_dir
+        .run_jj(["bookmark", "untrack", "feature2@origin"])
+        .success();
+    work_dir
+        .run_jj(["bookmark", "untrack", "hotfix@upstream"])
+        .success();
+
+    insta::assert_snapshot!(get_bookmark_output(&work_dir), @r"
+    develop: yzkwtzyq 4217fc8a upstream develop commit
+      @upstream: yzkwtzyq 4217fc8a upstream develop commit
+    feature1: ovvpyryn 8a4b3895 origin feature1 commit
+      @origin: ovvpyryn 8a4b3895 origin feature1 commit
+    feature2: ysxnuyrn 95a1c2bd origin feature2 commit
+    feature2@origin: ysxnuyrn 95a1c2bd origin feature2 commit
+    hotfix: pozyxktk e9e38ee9 upstream hotfix commit
+    hotfix@upstream: pozyxktk e9e38ee9 upstream hotfix commit
+    main (conflicted):
+      + orvppysl 25f66480 origin main commit
+      + nrlvptqt f241ccf9 upstream main commit
+      @origin (behind by 1 commits): orvppysl 25f66480 origin main commit
+      @upstream (behind by 1 commits): nrlvptqt f241ccf9 upstream main commit
+    [EOF]
+    ");
+
+    // Add new commits to tracked branches on both remotes
+    add_commit_to_branch(&origin_repo, "feature1", "new origin feature1 commit");
+    add_commit_to_branch(&upstream_repo, "develop", "new upstream develop commit");
+
+    // Add new commits to untracked branches
+    add_commit_to_branch(&origin_repo, "feature2", "new origin feature2 commit");
+    add_commit_to_branch(&upstream_repo, "hotfix", "new upstream hotfix commit");
+
+    // Fetch only tracked branches from all remotes
+    work_dir
+        .run_jj(["git", "fetch", "--tracked", "--all-remotes"])
+        .success();
+
+    // Only the tracked branches should be updated (feature1 and develop)
+    // Untracked branches (feature2, hotfix) should remain at old commits
+    insta::assert_snapshot!(get_bookmark_output(&work_dir), @r"
+    develop: kmsovkut 8b5845da (empty) new upstream develop commit
+      @upstream: kmsovkut 8b5845da (empty) new upstream develop commit
+    feature1: rmmunkwl d676351d (empty) new origin feature1 commit
+      @origin: rmmunkwl d676351d (empty) new origin feature1 commit
+    feature2: ysxnuyrn 95a1c2bd origin feature2 commit
+    feature2@origin: ysxnuyrn 95a1c2bd origin feature2 commit
+    hotfix: pozyxktk e9e38ee9 upstream hotfix commit
+    hotfix@upstream: pozyxktk e9e38ee9 upstream hotfix commit
+    main (conflicted):
+      + orvppysl 25f66480 origin main commit
+      + nrlvptqt f241ccf9 upstream main commit
+      @origin (behind by 1 commits): orvppysl 25f66480 origin main commit
+      @upstream (behind by 1 commits): nrlvptqt f241ccf9 upstream main commit
+    [EOF]
+    ");
+}
