@@ -48,6 +48,7 @@ fn test_log_with_no_template() {
     - builtin_config_list
     - builtin_config_list_detailed
     - builtin_draft_commit_description
+    - builtin_evolog_compact
     - builtin_log_comfortable
     - builtin_log_compact
     - builtin_log_compact_full_description
@@ -55,11 +56,13 @@ fn test_log_with_no_template() {
     - builtin_log_node
     - builtin_log_node_ascii
     - builtin_log_oneline
+    - builtin_log_redacted
     - builtin_op_log_comfortable
     - builtin_op_log_compact
     - builtin_op_log_node
     - builtin_op_log_node_ascii
     - builtin_op_log_oneline
+    - builtin_op_log_redacted
     - commit_summary_separator
     - default_commit_description
     - description_placeholder
@@ -1190,8 +1193,8 @@ fn test_graph_template_color() {
     ");
     let output = work_dir.run_jj(["--color=debug", "log", "-T", template]);
     insta::assert_snapshot!(output, @r"
-    [1m[38;5;2m<<log commit node working_copy::@>>[0m  [1m[38;5;2m<<log commit working_copy description::single line>>[0m
-    <<log commit node::○>>  [38;5;1m<<log commit description::first line>>[39m
+    [1m[38;5;2m<<log commit node working_copy mutable::@>>[0m  [1m[38;5;2m<<log commit working_copy description::single line>>[0m
+    <<log commit node mutable::○>>  [38;5;1m<<log commit description::first line>>[39m
     │  [38;5;1m<<log commit description::second line>>[39m
     │  [38;5;1m<<log commit description::third line>>[39m
     [1m[38;5;14m<<log commit node immutable::◆>>[0m
@@ -1683,3 +1686,26 @@ fn test_log_full_description_template() {
     [EOF]
     ");
 }
+
+#[test]
+fn test_log_anonymize() {
+    let test_env = TestEnvironment::default();
+    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
+    let work_dir = test_env.work_dir("repo");
+
+    work_dir
+        .run_jj([
+            "describe",
+            "-m",
+            "this is commit with a multiline description\n\n<full description>",
+        ])
+        .success();
+
+    let output = work_dir.run_jj(["log", "-Tbuiltin_log_redacted"]);
+    insta::assert_snapshot!(output, @r"
+    @  qpvuntsm user-78cd 2001-02-03 08:05:08 37b69cda
+    │  (empty) (redacted)
+    ◆  zzzzzzzz root() 00000000
+    [EOF]
+    ");
+}
