@@ -15,9 +15,9 @@
 #![allow(missing_docs)]
 
 use std::any::Any;
-use std::collections::BTreeMap;
 use std::fmt::Debug;
 use std::pin::Pin;
+use std::slice;
 use std::time::SystemTime;
 
 use async_trait::async_trait;
@@ -30,8 +30,8 @@ use crate::content_hash::ContentHash;
 use crate::hex_util;
 use crate::index::Index;
 use crate::merge::Merge;
-use crate::object_id::id_type;
 use crate::object_id::ObjectId as _;
+use crate::object_id::id_type;
 use crate::repo_path::RepoPath;
 use crate::repo_path::RepoPathBuf;
 use crate::repo_path::RepoPathComponent;
@@ -57,7 +57,7 @@ id_type!(pub CopyId { hex() });
 impl ChangeId {
     /// Parses the given "reverse" hex string into a `ChangeId`.
     pub fn try_from_reverse_hex(hex: impl AsRef<[u8]>) -> Option<Self> {
-        hex_util::decode_reverse_hex(hex.as_ref()).map(Self)
+        hex_util::decode_reverse_hex(hex).map(Self)
     }
 
     /// Returns the hex string representation of this ID, which uses `z-k`
@@ -179,14 +179,14 @@ impl Eq for MergedTreeId {}
 impl MergedTreeId {
     /// Create a resolved `MergedTreeId` from a single regular tree.
     pub fn resolved(tree_id: TreeId) -> Self {
-        MergedTreeId::Merge(Merge::resolved(tree_id))
+        Self::Merge(Merge::resolved(tree_id))
     }
 
     /// Return this id as `Merge<TreeId>`
     pub fn to_merge(&self) -> Merge<TreeId> {
         match self {
-            MergedTreeId::Legacy(tree_id) => Merge::resolved(tree_id.clone()),
-            MergedTreeId::Merge(tree_ids) => tree_ids.clone(),
+            Self::Legacy(tree_id) => Merge::resolved(tree_id.clone()),
+            Self::Merge(tree_ids) => tree_ids.clone(),
         }
     }
 }
@@ -357,11 +357,11 @@ pub enum TreeValue {
 impl TreeValue {
     pub fn hex(&self) -> String {
         match self {
-            TreeValue::File { id, .. } => id.hex(),
-            TreeValue::Symlink(id) => id.hex(),
-            TreeValue::Tree(id) => id.hex(),
-            TreeValue::GitSubmodule(id) => id.hex(),
-            TreeValue::Conflict(id) => id.hex(),
+            Self::File { id, .. } => id.hex(),
+            Self::Symlink(id) => id.hex(),
+            Self::Tree(id) => id.hex(),
+            Self::GitSubmodule(id) => id.hex(),
+            Self::Conflict(id) => id.hex(),
         }
     }
 }
@@ -374,7 +374,7 @@ pub struct TreeEntry<'a> {
 
 impl<'a> TreeEntry<'a> {
     pub fn new(name: &'a RepoPathComponent, value: &'a TreeValue) -> Self {
-        TreeEntry { name, value }
+        Self { name, value }
     }
 
     pub fn name(&self) -> &'a RepoPathComponent {
@@ -387,7 +387,7 @@ impl<'a> TreeEntry<'a> {
 }
 
 pub struct TreeEntriesNonRecursiveIterator<'a> {
-    iter: std::collections::btree_map::Iter<'a, RepoPathComponentBuf, TreeValue>,
+    iter: slice::Iter<'a, (RepoPathComponentBuf, TreeValue)>,
 }
 
 impl<'a> Iterator for TreeEntriesNonRecursiveIterator<'a> {
@@ -402,16 +402,21 @@ impl<'a> Iterator for TreeEntriesNonRecursiveIterator<'a> {
 
 #[derive(ContentHash, Default, PartialEq, Eq, Debug, Clone)]
 pub struct Tree {
-    entries: BTreeMap<RepoPathComponentBuf, TreeValue>,
+    entries: Vec<(RepoPathComponentBuf, TreeValue)>,
 }
 
 impl Tree {
+    pub fn from_sorted_entries(entries: Vec<(RepoPathComponentBuf, TreeValue)>) -> Self {
+        debug_assert!(entries.is_sorted_by(|(a, _), (b, _)| a < b));
+        Self { entries }
+    }
+
     pub fn is_empty(&self) -> bool {
         self.entries.is_empty()
     }
 
     pub fn names(&self) -> impl Iterator<Item = &RepoPathComponent> {
-        self.entries.keys().map(|name| name.as_ref())
+        self.entries.iter().map(|(name, _)| name.as_ref())
     }
 
     pub fn entries(&self) -> TreeEntriesNonRecursiveIterator<'_> {
@@ -420,33 +425,17 @@ impl Tree {
         }
     }
 
-    pub fn set(&mut self, name: RepoPathComponentBuf, value: TreeValue) {
-        self.entries.insert(name, value);
-    }
-
-    pub fn remove(&mut self, name: &RepoPathComponent) {
-        self.entries.remove(name);
-    }
-
-    pub fn set_or_remove(&mut self, name: &RepoPathComponent, value: Option<TreeValue>) {
-        match value {
-            None => {
-                self.entries.remove(name);
-            }
-            Some(value) => {
-                self.entries.insert(name.to_owned(), value);
-            }
-        }
-    }
-
     pub fn entry(&self, name: &RepoPathComponent) -> Option<TreeEntry<'_>> {
-        self.entries
-            .get_key_value(name)
-            .map(|(name, value)| TreeEntry { name, value })
+        let index = self
+            .entries
+            .binary_search_by_key(&name, |(name, _)| name)
+            .ok()?;
+        let (name, value) = &self.entries[index];
+        Some(TreeEntry { name, value })
     }
 
     pub fn value(&self, name: &RepoPathComponent) -> Option<&TreeValue> {
-        self.entries.get(name)
+        self.entry(name).map(|entry| entry.value)
     }
 }
 
