@@ -296,6 +296,13 @@ pub enum ConflictMarkerStyle {
     Git,
 }
 
+/// Options for conflict materialization.
+#[derive(Clone, Debug, Default)]
+pub struct ConflictMaterializeOptions {
+    pub marker_style: ConflictMarkerStyle,
+    pub marker_len: Option<usize>,
+}
+
 /// Characters which can be repeated to form a conflict marker line when
 /// materializing and parsing conflicts.
 #[derive(Clone, Copy, PartialEq, Eq)]
@@ -397,73 +404,35 @@ pub fn choose_materialized_conflict_marker_len<T: AsRef<[u8]>>(single_hunk: &Mer
 
 pub fn materialize_merge_result<T: AsRef<[u8]>>(
     single_hunk: &Merge<T>,
-    conflict_marker_style: ConflictMarkerStyle,
     output: &mut dyn Write,
+    options: &ConflictMaterializeOptions,
 ) -> io::Result<()> {
     let merge_result = files::merge_hunks(single_hunk);
     match &merge_result {
         MergeResult::Resolved(content) => output.write_all(content),
         MergeResult::Conflict(hunks) => {
-            let conflict_marker_len = choose_materialized_conflict_marker_len(single_hunk);
-            materialize_conflict_hunks(hunks, conflict_marker_style, conflict_marker_len, output)
-        }
-    }
-}
-
-pub fn materialize_merge_result_with_marker_len<T: AsRef<[u8]>>(
-    single_hunk: &Merge<T>,
-    conflict_marker_style: ConflictMarkerStyle,
-    conflict_marker_len: usize,
-    output: &mut dyn Write,
-) -> io::Result<()> {
-    let merge_result = files::merge_hunks(single_hunk);
-    match &merge_result {
-        MergeResult::Resolved(content) => output.write_all(content),
-        MergeResult::Conflict(hunks) => {
-            materialize_conflict_hunks(hunks, conflict_marker_style, conflict_marker_len, output)
+            let marker_len = options
+                .marker_len
+                .unwrap_or_else(|| choose_materialized_conflict_marker_len(single_hunk));
+            materialize_conflict_hunks(hunks, options.marker_style, marker_len, output)
         }
     }
 }
 
 pub fn materialize_merge_result_to_bytes<T: AsRef<[u8]>>(
     single_hunk: &Merge<T>,
-    conflict_marker_style: ConflictMarkerStyle,
+    options: &ConflictMaterializeOptions,
 ) -> BString {
     let merge_result = files::merge_hunks(single_hunk);
     match merge_result {
         MergeResult::Resolved(content) => content,
         MergeResult::Conflict(hunks) => {
-            let conflict_marker_len = choose_materialized_conflict_marker_len(single_hunk);
+            let marker_len = options
+                .marker_len
+                .unwrap_or_else(|| choose_materialized_conflict_marker_len(single_hunk));
             let mut output = Vec::new();
-            materialize_conflict_hunks(
-                &hunks,
-                conflict_marker_style,
-                conflict_marker_len,
-                &mut output,
-            )
-            .expect("writing to an in-memory buffer should never fail");
-            output.into()
-        }
-    }
-}
-
-pub fn materialize_merge_result_to_bytes_with_marker_len<T: AsRef<[u8]>>(
-    single_hunk: &Merge<T>,
-    conflict_marker_style: ConflictMarkerStyle,
-    conflict_marker_len: usize,
-) -> BString {
-    let merge_result = files::merge_hunks(single_hunk);
-    match merge_result {
-        MergeResult::Resolved(content) => content,
-        MergeResult::Conflict(hunks) => {
-            let mut output = Vec::new();
-            materialize_conflict_hunks(
-                &hunks,
-                conflict_marker_style,
-                conflict_marker_len,
-                &mut output,
-            )
-            .expect("writing to an in-memory buffer should never fail");
+            materialize_conflict_hunks(&hunks, options.marker_style, marker_len, &mut output)
+                .expect("writing to an in-memory buffer should never fail");
             output.into()
         }
     }
@@ -966,13 +935,11 @@ pub async fn update_from_content(
     // copy.
     let mut old_content = Vec::with_capacity(content.len());
     let merge_hunk = extract_as_single_hunk(&simplified_file_ids, store, path).await?;
-    materialize_merge_result_with_marker_len(
-        &merge_hunk,
-        conflict_marker_style,
-        conflict_marker_len,
-        &mut old_content,
-    )
-    .unwrap();
+    let materialize_options = ConflictMaterializeOptions {
+        marker_style: conflict_marker_style,
+        marker_len: Some(conflict_marker_len),
+    };
+    materialize_merge_result(&merge_hunk, &mut old_content, &materialize_options).unwrap();
     if content == old_content {
         return Ok(file_ids.clone());
     }
