@@ -30,6 +30,7 @@ use ref_cast::ref_cast_custom;
 
 use super::bit_set::AncestorsBitSet;
 use super::bit_set::PositionsBitSet;
+use super::changed_path::CompositeChangedPathIndex;
 use super::entry::CommitIndexEntry;
 use super::entry::GlobalCommitPosition;
 use super::entry::LocalCommitPosition;
@@ -50,6 +51,7 @@ use crate::object_id::HexPrefix;
 use crate::object_id::ObjectId as _;
 use crate::object_id::PrefixResolution;
 use crate::object_id::id_type;
+use crate::repo_path::RepoPathBuf;
 use crate::revset::ResolvedExpression;
 use crate::revset::Revset;
 use crate::revset::RevsetEvaluationError;
@@ -497,27 +499,38 @@ enum CompositeCommitIndexSegment {
 #[derive(Clone, Debug)]
 pub(super) struct CompositeIndex {
     commits: CompositeCommitIndexSegment,
-    // TODO: add changed-paths index
+    changed_paths: CompositeChangedPathIndex,
 }
 
 impl CompositeIndex {
-    pub(super) fn from_readonly(commits: Arc<ReadonlyCommitIndexSegment>) -> Self {
+    pub(super) fn from_readonly(
+        commits: Arc<ReadonlyCommitIndexSegment>,
+        changed_paths: CompositeChangedPathIndex,
+    ) -> Self {
         Self {
             commits: CompositeCommitIndexSegment::Readonly(commits),
+            changed_paths,
         }
     }
 
-    pub(super) fn from_mutable(commits: Box<MutableCommitIndexSegment>) -> Self {
+    pub(super) fn from_mutable(
+        commits: Box<MutableCommitIndexSegment>,
+        changed_paths: CompositeChangedPathIndex,
+    ) -> Self {
         Self {
             commits: CompositeCommitIndexSegment::Mutable(commits),
+            changed_paths,
         }
     }
 
-    pub(super) fn into_mutable(self) -> Option<Box<MutableCommitIndexSegment>> {
-        match self.commits {
-            CompositeCommitIndexSegment::Readonly(_) => None,
-            CompositeCommitIndexSegment::Mutable(segment) => Some(segment),
-        }
+    pub(super) fn into_mutable(
+        self,
+    ) -> Option<(Box<MutableCommitIndexSegment>, CompositeChangedPathIndex)> {
+        let commits = match self.commits {
+            CompositeCommitIndexSegment::Readonly(_) => return None,
+            CompositeCommitIndexSegment::Mutable(segment) => segment,
+        };
+        Some((commits, self.changed_paths))
     }
 
     pub(super) fn commits(&self) -> &CompositeCommitIndex {
@@ -541,13 +554,12 @@ impl CompositeIndex {
         }
     }
 
-    pub(super) fn evaluate_revset(
-        &self,
-        expression: &ResolvedExpression,
-        store: &Arc<Store>,
-    ) -> Result<Box<dyn Revset + '_>, RevsetEvaluationError> {
-        let revset_impl = revset_engine::evaluate(expression, store, self)?;
-        Ok(Box::new(revset_impl))
+    pub(super) fn changed_paths(&self) -> &CompositeChangedPathIndex {
+        &self.changed_paths
+    }
+
+    pub(super) fn changed_paths_mut(&mut self) -> &mut CompositeChangedPathIndex {
+        &mut self.changed_paths
     }
 }
 
@@ -593,12 +605,27 @@ impl Index for CompositeIndex {
         Ok(self.commits().heads(candidate_ids))
     }
 
+    fn changed_paths_in_commit(
+        &self,
+        commit_id: &CommitId,
+    ) -> Result<Option<Box<dyn Iterator<Item = RepoPathBuf> + '_>>, IndexError> {
+        let Some(paths) = self
+            .commits()
+            .commit_id_to_pos(commit_id)
+            .and_then(|pos| self.changed_paths().changed_paths(pos))
+        else {
+            return Ok(None);
+        };
+        Ok(Some(Box::new(paths.map(|path| path.to_owned()))))
+    }
+
     fn evaluate_revset(
         &self,
         expression: &ResolvedExpression,
         store: &Arc<Store>,
     ) -> Result<Box<dyn Revset + '_>, RevsetEvaluationError> {
-        Self::evaluate_revset(self, expression, store)
+        let revset_impl = revset_engine::evaluate(expression, store, self)?;
+        Ok(Box::new(revset_impl))
     }
 }
 
