@@ -21,41 +21,41 @@ use std::hash::Hasher;
 
 use smallvec::SmallVec;
 
-use super::composite::CompositeIndex;
-use super::composite::DynIndexSegment;
+use super::composite::CompositeCommitIndex;
+use super::composite::DynCommitIndexSegment;
 use crate::backend::ChangeId;
 use crate::backend::CommitId;
 use crate::object_id::ObjectId as _;
 
-/// Global index position.
+/// Global commit index position.
 #[derive(Debug, PartialEq, Eq, PartialOrd, Ord, Clone, Copy, Hash)]
-pub struct IndexPosition(pub(super) u32);
+pub(super) struct GlobalCommitPosition(pub(super) u32);
 
-impl IndexPosition {
-    pub const MIN: Self = IndexPosition(u32::MIN);
-    pub const MAX: Self = IndexPosition(u32::MAX);
+impl GlobalCommitPosition {
+    pub const MIN: Self = Self(u32::MIN);
+    pub const MAX: Self = Self(u32::MAX);
 }
 
-/// Local position within an index segment.
+/// Local commit position within an index segment.
 #[derive(Debug, PartialEq, Eq, PartialOrd, Ord, Clone, Copy, Hash)]
-pub(super) struct LocalPosition(pub(super) u32);
+pub(super) struct LocalCommitPosition(pub(super) u32);
 
 // SmallVec reuses two pointer-size fields as inline area, which meas we can
 // inline up to 16 bytes (on 64-bit platform) for free.
-pub(super) type SmallIndexPositionsVec = SmallVec<[IndexPosition; 4]>;
-pub(super) type SmallLocalPositionsVec = SmallVec<[LocalPosition; 4]>;
+pub(super) type SmallGlobalCommitPositionsVec = SmallVec<[GlobalCommitPosition; 4]>;
+pub(super) type SmallLocalCommitPositionsVec = SmallVec<[LocalCommitPosition; 4]>;
 
 #[derive(Clone)]
-pub struct IndexEntry<'a> {
-    source: &'a DynIndexSegment,
-    pos: IndexPosition,
+pub(super) struct CommitIndexEntry<'a> {
+    source: &'a DynCommitIndexSegment,
+    pos: GlobalCommitPosition,
     /// Position within the source segment
-    local_pos: LocalPosition,
+    local_pos: LocalCommitPosition,
 }
 
-impl Debug for IndexEntry<'_> {
+impl Debug for CommitIndexEntry<'_> {
     fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
-        f.debug_struct("IndexEntry")
+        f.debug_struct("CommitIndexEntry")
             .field("pos", &self.pos)
             .field("local_pos", &self.local_pos)
             .field("commit_id", &self.commit_id().hex())
@@ -63,34 +63,34 @@ impl Debug for IndexEntry<'_> {
     }
 }
 
-impl PartialEq for IndexEntry<'_> {
+impl PartialEq for CommitIndexEntry<'_> {
     fn eq(&self, other: &Self) -> bool {
         self.pos == other.pos
     }
 }
 
-impl Eq for IndexEntry<'_> {}
+impl Eq for CommitIndexEntry<'_> {}
 
-impl Hash for IndexEntry<'_> {
+impl Hash for CommitIndexEntry<'_> {
     fn hash<H: Hasher>(&self, state: &mut H) {
         self.pos.hash(state);
     }
 }
 
-impl<'a> IndexEntry<'a> {
+impl<'a> CommitIndexEntry<'a> {
     pub(super) fn new(
-        source: &'a DynIndexSegment,
-        pos: IndexPosition,
-        local_pos: LocalPosition,
+        source: &'a DynCommitIndexSegment,
+        pos: GlobalCommitPosition,
+        local_pos: LocalCommitPosition,
     ) -> Self {
-        IndexEntry {
+        Self {
             source,
             pos,
             local_pos,
         }
     }
 
-    pub fn position(&self) -> IndexPosition {
+    pub fn position(&self) -> GlobalCommitPosition {
         self.pos
     }
 
@@ -110,12 +110,12 @@ impl<'a> IndexEntry<'a> {
         self.source.num_parents(self.local_pos)
     }
 
-    pub fn parent_positions(&self) -> SmallIndexPositionsVec {
+    pub fn parent_positions(&self) -> SmallGlobalCommitPositionsVec {
         self.source.parent_positions(self.local_pos)
     }
 
-    pub fn parents(&self) -> impl ExactSizeIterator<Item = IndexEntry<'a>> + use<'a> {
-        let composite = CompositeIndex::new(self.source);
+    pub fn parents(&self) -> impl ExactSizeIterator<Item = CommitIndexEntry<'a>> + use<'a> {
+        let composite = CompositeCommitIndex::new(self.source);
         self.parent_positions()
             .into_iter()
             .map(move |pos| composite.entry_by_pos(pos))
