@@ -29,10 +29,13 @@ use std::sync::Arc;
 use blake2::Blake2b512;
 use digest::Digest as _;
 use itertools::Itertools as _;
+use pollster::FutureExt as _;
 use smallvec::SmallVec;
 use smallvec::smallvec;
 use tempfile::NamedTempFile;
 
+use super::changed_path::CompositeChangedPathIndex;
+use super::changed_path::collect_changed_paths;
 use super::composite::AsCompositeIndex;
 use super::composite::ChangeIdIndexImpl;
 use super::composite::CommitIndexSegment;
@@ -49,6 +52,7 @@ use super::readonly::DefaultReadonlyIndex;
 use super::readonly::FieldLengths;
 use super::readonly::OVERFLOW_FLAG;
 use super::readonly::ReadonlyCommitIndexSegment;
+use crate::backend::BackendResult;
 use crate::backend::ChangeId;
 use crate::backend::CommitId;
 use crate::commit::Commit;
@@ -64,6 +68,7 @@ use crate::index::ReadonlyIndex;
 use crate::object_id::HexPrefix;
 use crate::object_id::ObjectId;
 use crate::object_id::PrefixResolution;
+use crate::repo_path::RepoPathBuf;
 use crate::revset::ResolvedExpression;
 use crate::revset::Revset;
 use crate::revset::RevsetEvaluationError;
@@ -453,17 +458,24 @@ pub struct DefaultMutableIndex(CompositeIndex);
 impl DefaultMutableIndex {
     pub(super) fn full(lengths: FieldLengths) -> Self {
         let commits = Box::new(MutableCommitIndexSegment::full(lengths));
-        Self(CompositeIndex::from_mutable(commits))
+        // Changed-path index isn't enabled by default.
+        let mut changed_paths = CompositeChangedPathIndex::null();
+        changed_paths.make_mutable();
+        Self(CompositeIndex::from_mutable(commits, changed_paths))
     }
 
     pub(super) fn incremental(parent_index: &DefaultReadonlyIndex) -> Self {
         let commits = Box::new(MutableCommitIndexSegment::incremental(
             parent_index.readonly_commits().clone(),
         ));
-        Self(CompositeIndex::from_mutable(commits))
+        let mut changed_paths = parent_index.changed_paths().clone();
+        changed_paths.make_mutable();
+        Self(CompositeIndex::from_mutable(commits, changed_paths))
     }
 
-    pub(super) fn into_segment(self) -> Box<MutableCommitIndexSegment> {
+    pub(super) fn into_segment(
+        self,
+    ) -> (Box<MutableCommitIndexSegment>, CompositeChangedPathIndex) {
         self.0.into_mutable().expect("must have mutable")
     }
 
@@ -476,12 +488,21 @@ impl DefaultMutableIndex {
         self.0.commits().num_commits()
     }
 
-    pub(super) fn add_commit(&mut self, commit: &Commit) {
+    #[tracing::instrument(skip(self))]
+    pub(super) async fn add_commit(&mut self, commit: &Commit) -> BackendResult<()> {
+        let new_commit_pos = GlobalCommitPosition(self.num_commits());
         self.add_commit_data(
             commit.id().clone(),
             commit.change_id().clone(),
             commit.parent_ids(),
         );
+        if new_commit_pos == GlobalCommitPosition(self.num_commits()) {
+            return Ok(()); // commit already indexed
+        }
+        if self.0.changed_paths().next_mutable_commit_pos() == Some(new_commit_pos) {
+            self.add_commit_changed_paths(commit).await?;
+        }
+        Ok(())
     }
 
     pub(super) fn add_commit_data(
@@ -493,6 +514,31 @@ impl DefaultMutableIndex {
         self.mutable_commits()
             .add_commit_data(commit_id, change_id, parent_ids);
     }
+
+    // CompositeChangedPathIndex::add_commit() isn't implemented because we need
+    // a commit index to merge parent trees, which means we need to borrow self.
+    async fn add_commit_changed_paths(&mut self, commit: &Commit) -> BackendResult<()> {
+        let paths = collect_changed_paths(self, commit).await?;
+        self.0.changed_paths_mut().add_changed_paths(paths);
+        Ok(())
+    }
+
+    pub(super) fn merge_in(&mut self, other: &DefaultReadonlyIndex) {
+        let start_commit_pos = GlobalCommitPosition(self.num_commits());
+        self.mutable_commits().merge_in(other.readonly_commits());
+        if self.0.changed_paths().next_mutable_commit_pos() == Some(start_commit_pos) {
+            let other_commits = other.as_composite().commits();
+            for self_pos in (start_commit_pos.0..self.num_commits()).map(GlobalCommitPosition) {
+                let entry = self.0.commits().entry_by_pos(self_pos);
+                let other_pos = other_commits.commit_id_to_pos(&entry.commit_id()).unwrap();
+                let Some(paths) = other.changed_paths().changed_paths(other_pos) else {
+                    break; // no more indexed paths in other index
+                };
+                let paths = paths.map(|path| path.to_owned()).collect();
+                self.0.changed_paths_mut().add_changed_paths(paths);
+            }
+        }
+    }
 }
 
 impl AsCompositeIndex for DefaultMutableIndex {
@@ -535,6 +581,13 @@ impl Index for DefaultMutableIndex {
         self.0.heads(candidates)
     }
 
+    fn changed_paths_in_commit(
+        &self,
+        commit_id: &CommitId,
+    ) -> Result<Option<Box<dyn Iterator<Item = RepoPathBuf> + '_>>, IndexError> {
+        self.0.changed_paths_in_commit(commit_id)
+    }
+
     fn evaluate_revset(
         &self,
         expression: &ResolvedExpression,
@@ -565,8 +618,9 @@ impl MutableIndex for DefaultMutableIndex {
     }
 
     fn add_commit(&mut self, commit: &Commit) -> Result<(), IndexError> {
-        Self::add_commit(self, commit);
-        Ok(())
+        Self::add_commit(self, commit)
+            .block_on()
+            .map_err(|err| IndexError(err.into()))
     }
 
     fn merge_in(&mut self, other: &dyn ReadonlyIndex) {
@@ -574,7 +628,7 @@ impl MutableIndex for DefaultMutableIndex {
             .as_any()
             .downcast_ref::<DefaultReadonlyIndex>()
             .expect("index to merge in must be a DefaultReadonlyIndex");
-        self.mutable_commits().merge_in(other.readonly_commits());
+        Self::merge_in(self, other);
     }
 }
 
