@@ -23,6 +23,7 @@ use std::fs::File;
 use std::io;
 use std::io::Read;
 use std::iter;
+use std::ops::Range;
 use std::path::Path;
 use std::sync::Arc;
 
@@ -30,6 +31,7 @@ use itertools::Itertools as _;
 use smallvec::smallvec;
 use thiserror::Error;
 
+use super::changed_path::CompositeChangedPathIndex;
 use super::composite::AsCompositeIndex;
 use super::composite::ChangeIdIndexImpl;
 use super::composite::CommitIndexSegment;
@@ -55,6 +57,7 @@ use crate::index::ReadonlyIndex;
 use crate::object_id::HexPrefix;
 use crate::object_id::ObjectId;
 use crate::object_id::PrefixResolution;
+use crate::repo_path::RepoPathBuf;
 use crate::revset::ResolvedExpression;
 use crate::revset::Revset;
 use crate::revset::RevsetEvaluationError;
@@ -610,14 +613,21 @@ impl CommitIndexSegment for ReadonlyCommitIndexSegment {
 pub struct DefaultReadonlyIndex(CompositeIndex);
 
 impl DefaultReadonlyIndex {
-    pub(super) fn from_segment(commits: Arc<ReadonlyCommitIndexSegment>) -> Self {
-        Self(CompositeIndex::from_readonly(commits))
+    pub(super) fn from_segment(
+        commits: Arc<ReadonlyCommitIndexSegment>,
+        changed_paths: CompositeChangedPathIndex,
+    ) -> Self {
+        Self(CompositeIndex::from_readonly(commits, changed_paths))
     }
 
     pub(super) fn readonly_commits(&self) -> &Arc<ReadonlyCommitIndexSegment> {
         self.0.readonly_commits().expect("must have readonly")
     }
 
+    pub(super) fn changed_paths(&self) -> &CompositeChangedPathIndex {
+        self.0.changed_paths()
+    }
+
     /// Returns the number of all indexed commits.
     pub fn num_commits(&self) -> u32 {
         self.0.commits().num_commits()
@@ -647,6 +657,21 @@ impl DefaultReadonlyIndex {
             .collect_vec();
         commit_levels.reverse();
 
+        let changed_paths = self.changed_paths();
+        let changed_path_commits_range = changed_paths
+            .start_commit_pos()
+            .map(|GlobalCommitPosition(start)| start..(start + changed_paths.num_commits()));
+        let changed_path_levels = changed_paths
+            .readonly_segments()
+            .iter()
+            .map(|segment| ChangedPathIndexLevelStats {
+                num_commits: segment.num_local_commits(),
+                num_changed_paths: segment.num_changed_paths(),
+                num_paths: segment.num_paths(),
+                name: segment.id().hex(),
+            })
+            .collect_vec();
+
         IndexStats {
             num_commits,
             num_merges,
@@ -654,6 +679,8 @@ impl DefaultReadonlyIndex {
             num_heads,
             num_changes: change_ids.len().try_into().unwrap(),
             commit_levels,
+            changed_path_commits_range,
+            changed_path_levels,
         }
     }
 
@@ -718,6 +745,13 @@ impl Index for DefaultReadonlyIndex {
         self.0.heads(candidates)
     }
 
+    fn changed_paths_in_commit(
+        &self,
+        commit_id: &CommitId,
+    ) -> Result<Option<Box<dyn Iterator<Item = RepoPathBuf> + '_>>, IndexError> {
+        self.0.changed_paths_in_commit(commit_id)
+    }
+
     fn evaluate_revset(
         &self,
         expression: &ResolvedExpression,
@@ -775,6 +809,8 @@ pub struct IndexStats {
     pub num_heads: u32,
     pub num_changes: u32,
     pub commit_levels: Vec<CommitIndexLevelStats>,
+    pub changed_path_commits_range: Option<Range<u32>>,
+    pub changed_path_levels: Vec<ChangedPathIndexLevelStats>,
 }
 
 #[derive(Clone, Debug)]
@@ -783,6 +819,18 @@ pub struct CommitIndexLevelStats {
     pub name: String,
 }
 
+#[derive(Clone, Debug)]
+pub struct ChangedPathIndexLevelStats {
+    /// Number of commits.
+    pub num_commits: u32,
+    /// Sum of number of per-commit changed paths.
+    pub num_changed_paths: u32,
+    /// Number of unique paths.
+    pub num_paths: u32,
+    /// Index file name.
+    pub name: String,
+}
+
 /// Binary search result in a sorted lookup table.
 #[derive(Clone, Copy, Debug)]
 struct PositionLookupResult {
