@@ -26,10 +26,17 @@ use std::slice;
 use std::sync::Arc;
 
 use itertools::Itertools as _;
+use pollster::FutureExt as _;
+use prost::Message as _;
 use tempfile::NamedTempFile;
 use thiserror::Error;
 
+use super::changed_path::ChangedPathIndexSegmentId;
+use super::changed_path::CompositeChangedPathIndex;
+use super::changed_path::collect_changed_paths;
+use super::composite::AsCompositeIndex as _;
 use super::composite::CommitIndexSegmentId;
+use super::entry::GlobalCommitPosition;
 use super::mutable::DefaultMutableIndex;
 use super::readonly::DefaultReadonlyIndex;
 use super::readonly::FieldLengths;
@@ -43,7 +50,7 @@ use crate::dag_walk;
 use crate::file_util;
 use crate::file_util::IoResultExt as _;
 use crate::file_util::PathError;
-use crate::file_util::persist_content_addressed_temp_file;
+use crate::file_util::persist_temp_file;
 use crate::index::Index as _;
 use crate::index::IndexReadError;
 use crate::index::IndexStore;
@@ -73,16 +80,16 @@ impl From<DefaultIndexStoreInitError> for BackendInitError {
 
 #[derive(Debug, Error)]
 pub enum DefaultIndexStoreError {
-    #[error("Failed to associate commit index file with an operation {op_id}")]
+    #[error("Failed to associate index files with an operation {op_id}")]
     AssociateIndex {
         op_id: OperationId,
         source: PathError,
     },
-    #[error("Failed to load associated commit index file name")]
+    #[error("Failed to load associated index file names")]
     LoadAssociation(#[source] PathError),
     #[error(transparent)]
     LoadIndex(ReadonlyIndexLoadError),
-    #[error("Failed to write commit index file")]
+    #[error("Failed to write index file")]
     SaveIndex(#[source] PathError),
     #[error("Failed to index commits at operation {op_id}")]
     IndexCommits {
@@ -121,10 +128,12 @@ impl DefaultIndexStore {
         // Create base directories in case the store was initialized by old jj.
         self.ensure_base_dirs()?;
         // Remove all operation links to trigger rebuilding.
-        file_util::remove_dir_contents(&self.operations_dir())?;
+        file_util::remove_dir_contents(&self.op_links_dir())?;
+        file_util::remove_dir_contents(&self.legacy_operations_dir())?;
         // Remove index segments to save disk space. If raced, new segment file
         // will be created by the other process.
         file_util::remove_dir_contents(&self.commit_segments_dir())?;
+        file_util::remove_dir_contents(&self.changed_path_segments_dir())?;
         // jj <= 0.14 created segment files in the top directory
         for entry in self.dir.read_dir().context(&self.dir)? {
             let entry = entry.context(&self.dir)?;
@@ -139,13 +148,24 @@ impl DefaultIndexStore {
     }
 
     fn ensure_base_dirs(&self) -> Result<(), PathError> {
-        for dir in [self.operations_dir(), self.commit_segments_dir()] {
+        for dir in [
+            self.op_links_dir(),
+            self.legacy_operations_dir(),
+            self.commit_segments_dir(),
+            self.changed_path_segments_dir(),
+        ] {
             file_util::create_or_reuse_dir(&dir).context(&dir)?;
         }
         Ok(())
     }
 
-    fn operations_dir(&self) -> PathBuf {
+    /// Directory for mapping from operations to segments. (jj >= 0.33)
+    fn op_links_dir(&self) -> PathBuf {
+        self.dir.join("op_links")
+    }
+
+    /// Directory for mapping from operations to commit segments. (jj < 0.33)
+    fn legacy_operations_dir(&self) -> PathBuf {
         self.dir.join("operations")
     }
 
@@ -154,23 +174,72 @@ impl DefaultIndexStore {
         self.dir.join("segments")
     }
 
+    /// Directory for changed-path segment files.
+    fn changed_path_segments_dir(&self) -> PathBuf {
+        self.dir.join("changed_paths")
+    }
+
     fn load_index_at_operation(
         &self,
         op_id: &OperationId,
         lengths: FieldLengths,
     ) -> Result<DefaultReadonlyIndex, DefaultIndexStoreError> {
-        let op_id_file = self.operations_dir().join(op_id.hex());
-        let index_file_id_hex = fs::read(&op_id_file)
-            .context(&op_id_file)
-            .map_err(DefaultIndexStoreError::LoadAssociation)?;
-        let index_file_id = CommitIndexSegmentId::try_from_hex(&index_file_id_hex)
-            .ok_or_else(|| io::Error::new(io::ErrorKind::InvalidData, "file name is not valid hex"))
-            .context(&op_id_file)
-            .map_err(DefaultIndexStoreError::LoadAssociation)?;
-        let commits =
-            ReadonlyCommitIndexSegment::load(&self.commit_segments_dir(), index_file_id, lengths)
-                .map_err(DefaultIndexStoreError::LoadIndex)?;
-        Ok(DefaultReadonlyIndex::from_segment(commits))
+        let commit_segment_id;
+        let changed_path_start_commit_pos;
+        let changed_path_segment_ids;
+        let op_link_file = self.op_links_dir().join(op_id.hex());
+        match fs::read(&op_link_file).context(&op_link_file) {
+            Ok(data) => {
+                let proto = crate::protos::default_index::SegmentControl::decode(&*data)
+                    .map_err(|err| io::Error::new(io::ErrorKind::InvalidData, err))
+                    .context(&op_link_file)
+                    .map_err(DefaultIndexStoreError::LoadAssociation)?;
+                commit_segment_id = CommitIndexSegmentId::new(proto.commit_segment_id);
+                changed_path_start_commit_pos = proto
+                    .changed_path_start_commit_pos
+                    .map(GlobalCommitPosition);
+                changed_path_segment_ids = proto
+                    .changed_path_segment_ids
+                    .into_iter()
+                    .map(ChangedPathIndexSegmentId::new)
+                    .collect_vec();
+            }
+            // TODO: drop support for legacy operation link file in jj 0.39 or so
+            Err(PathError { source: error, .. }) if error.kind() == io::ErrorKind::NotFound => {
+                let op_id_file = self.legacy_operations_dir().join(op_id.hex());
+                let data = fs::read(&op_id_file)
+                    .context(&op_id_file)
+                    .map_err(DefaultIndexStoreError::LoadAssociation)?;
+                commit_segment_id = CommitIndexSegmentId::try_from_hex(&data)
+                    .ok_or_else(|| {
+                        io::Error::new(io::ErrorKind::InvalidData, "file name is not valid hex")
+                    })
+                    .context(&op_id_file)
+                    .map_err(DefaultIndexStoreError::LoadAssociation)?;
+                changed_path_start_commit_pos = None;
+                changed_path_segment_ids = vec![];
+            }
+            Err(err) => return Err(DefaultIndexStoreError::LoadAssociation(err)),
+        };
+
+        let commits = ReadonlyCommitIndexSegment::load(
+            &self.commit_segments_dir(),
+            commit_segment_id,
+            lengths,
+        )
+        .map_err(DefaultIndexStoreError::LoadIndex)?;
+        // TODO: lazy load or mmap?
+        let changed_paths = if let Some(start_commit_pos) = changed_path_start_commit_pos {
+            CompositeChangedPathIndex::load(
+                &self.changed_path_segments_dir(),
+                start_commit_pos,
+                &changed_path_segment_ids,
+            )
+            .map_err(DefaultIndexStoreError::LoadIndex)?
+        } else {
+            CompositeChangedPathIndex::null()
+        };
+        Ok(DefaultReadonlyIndex::from_segment(commits, changed_paths))
     }
 
     /// Rebuilds index for the given `operation`.
@@ -178,13 +247,14 @@ impl DefaultIndexStore {
     /// The index to be built will be calculated from one of the ancestor
     /// operations if exists. Use `reinit()` to rebuild index from scratch.
     #[tracing::instrument(skip(self, store))]
-    pub fn build_index_at_operation(
+    pub async fn build_index_at_operation(
         &self,
         operation: &Operation,
         store: &Arc<Store>,
     ) -> Result<DefaultReadonlyIndex, DefaultIndexStoreError> {
         tracing::info!("scanning operations to index");
-        let operations_dir = self.operations_dir();
+        let op_links_dir = self.op_links_dir();
+        let legacy_operations_dir = self.legacy_operations_dir();
         let field_lengths = FieldLengths {
             commit_id: store.commit_id_length(),
             change_id: store.change_id_length(),
@@ -194,7 +264,9 @@ impl DefaultIndexStore {
         let mut parent_op = None;
         for op in op_walk::walk_ancestors(slice::from_ref(operation)) {
             let op = op?;
-            if operations_dir.join(op.id().hex()).is_file() {
+            if op_links_dir.join(op.id().hex()).is_file()
+                || legacy_operations_dir.join(op.id().hex()).is_file()
+            {
                 parent_op = Some(op);
                 break;
             } else {
@@ -305,8 +377,13 @@ impl DefaultIndexStore {
             },
             |_| panic!("graph has cycle"),
         )?;
-        for (CommitByCommitterTimestamp(commit), _) in commits.iter().rev() {
-            mutable_index.add_commit(commit);
+        for (CommitByCommitterTimestamp(commit), op_id) in commits.iter().rev() {
+            mutable_index.add_commit(commit).await.map_err(|source| {
+                DefaultIndexStoreError::IndexCommits {
+                    op_id: op_id.clone(),
+                    source,
+                }
+            })?;
         }
 
         let index = self.save_mutable_index(mutable_index, operation.id())?;
@@ -315,17 +392,121 @@ impl DefaultIndexStore {
         Ok(index)
     }
 
+    /// Builds changed-path index for the specified operation.
+    ///
+    /// At most `max_commits` number of commits will be scanned from the latest
+    /// unindexed commit.
+    #[tracing::instrument(skip(self, store))]
+    pub async fn build_changed_path_index_at_operation(
+        &self,
+        op_id: &OperationId,
+        store: &Arc<Store>,
+        max_commits: u32,
+        // TODO: add progress callback?
+    ) -> Result<DefaultReadonlyIndex, DefaultIndexStoreError> {
+        // Create directories in case the store was initialized by jj < 0.33.
+        self.ensure_base_dirs()
+            .map_err(DefaultIndexStoreError::SaveIndex)?;
+        let field_lengths = FieldLengths {
+            commit_id: store.commit_id_length(),
+            change_id: store.change_id_length(),
+        };
+        let index = self.load_index_at_operation(op_id, field_lengths)?;
+        let old_changed_paths = index.changed_paths();
+
+        // Distribute max_commits to contiguous pre/post ranges:
+        //   ..|pre|old_changed_paths|post|
+        //   (where pre.len() + post.len() <= max_commits)
+        let pre_start;
+        let pre_end;
+        let post_start;
+        let post_end;
+        if let Some(GlobalCommitPosition(pos)) = old_changed_paths.start_commit_pos() {
+            post_start = pos + old_changed_paths.num_commits();
+            assert!(post_start <= index.num_commits());
+            post_end = u32::saturating_add(post_start, max_commits).min(index.num_commits());
+            pre_start = u32::saturating_sub(pos, max_commits - (post_end - post_start));
+            pre_end = pos;
+        } else {
+            pre_start = u32::saturating_sub(index.num_commits(), max_commits);
+            pre_end = index.num_commits();
+            post_start = pre_end;
+            post_end = pre_end;
+        }
+
+        let to_index_err = |source| DefaultIndexStoreError::IndexCommits {
+            op_id: op_id.clone(),
+            source,
+        };
+        let index_commit = async |changed_paths: &mut CompositeChangedPathIndex,
+                                  pos: GlobalCommitPosition| {
+            assert_eq!(changed_paths.next_mutable_commit_pos(), Some(pos));
+            let commit_id = index.as_composite().commits().entry_by_pos(pos).commit_id();
+            let commit = store.get_commit_async(&commit_id).await?;
+            let paths = collect_changed_paths(&index, &commit).await?;
+            changed_paths.add_changed_paths(paths);
+            Ok(())
+        };
+
+        // Index pre range
+        let mut new_changed_paths =
+            CompositeChangedPathIndex::empty(GlobalCommitPosition(pre_start));
+        new_changed_paths.make_mutable();
+        tracing::info!(?pre_start, ?pre_end, "indexing changed paths in commits");
+        for pos in (pre_start..pre_end).map(GlobalCommitPosition) {
+            index_commit(&mut new_changed_paths, pos)
+                .await
+                .map_err(to_index_err)?;
+        }
+        new_changed_paths
+            .save_in(&self.changed_path_segments_dir())
+            .map_err(DefaultIndexStoreError::SaveIndex)?;
+
+        // Copy previously-indexed segments
+        new_changed_paths.append_segments(old_changed_paths);
+
+        // Index post range, which is usually empty
+        new_changed_paths.make_mutable();
+        tracing::info!(?post_start, ?post_end, "indexing changed paths in commits");
+        for pos in (post_start..post_end).map(GlobalCommitPosition) {
+            index_commit(&mut new_changed_paths, pos)
+                .await
+                .map_err(to_index_err)?;
+        }
+        new_changed_paths.maybe_squash_with_ancestors();
+        new_changed_paths
+            .save_in(&self.changed_path_segments_dir())
+            .map_err(DefaultIndexStoreError::SaveIndex)?;
+
+        // Update the operation link to point to the new segments
+        let commits = index.readonly_commits().clone();
+        let index = DefaultReadonlyIndex::from_segment(commits, new_changed_paths);
+        self.associate_index_with_operation(&index, op_id)
+            .map_err(|source| DefaultIndexStoreError::AssociateIndex {
+                op_id: op_id.to_owned(),
+                source,
+            })?;
+        Ok(index)
+    }
+
     fn save_mutable_index(
         &self,
         index: DefaultMutableIndex,
         op_id: &OperationId,
     ) -> Result<DefaultReadonlyIndex, DefaultIndexStoreError> {
-        let commits = index.into_segment();
+        // Create directories in case the store was initialized by jj < 0.33.
+        self.ensure_base_dirs()
+            .map_err(DefaultIndexStoreError::SaveIndex)?;
+        let (commits, mut changed_paths) = index.into_segment();
         let commits = commits
             .maybe_squash_with_ancestors()
             .save_in(&self.commit_segments_dir())
             .map_err(DefaultIndexStoreError::SaveIndex)?;
-        let index = DefaultReadonlyIndex::from_segment(commits);
+        changed_paths.maybe_squash_with_ancestors();
+        changed_paths
+            .save_in(&self.changed_path_segments_dir())
+            .map_err(DefaultIndexStoreError::SaveIndex)?;
+        let index = DefaultReadonlyIndex::from_segment(commits, changed_paths);
         self.associate_index_with_operation(&index, op_id)
             .map_err(|source| DefaultIndexStoreError::AssociateIndex {
                 op_id: op_id.to_owned(),
@@ -340,13 +521,35 @@ impl DefaultIndexStore {
         index: &DefaultReadonlyIndex,
         op_id: &OperationId,
     ) -> Result<(), PathError> {
-        let dir = self.operations_dir();
+        let proto = crate::protos::default_index::SegmentControl {
+            commit_segment_id: index.readonly_commits().id().to_bytes(),
+            changed_path_start_commit_pos: index
+                .changed_paths()
+                .start_commit_pos()
+                .map(|GlobalCommitPosition(start)| start),
+            changed_path_segment_ids: index
+                .changed_paths()
+                .readonly_segments()
+                .iter()
+                .map(|segment| segment.id().to_bytes())
+                .collect(),
+        };
+        let dir = self.op_links_dir();
+        let mut temp_file = NamedTempFile::new_in(&dir).context(&dir)?;
+        let file = temp_file.as_file_mut();
+        file.write_all(&proto.encode_to_vec())
+            .context(temp_file.path())?;
+        let path = dir.join(op_id.hex());
+        persist_temp_file(temp_file, &path).context(&path)?;
+
+        // TODO: drop support for legacy operation link file in jj 0.39 or so
+        let dir = self.legacy_operations_dir();
         let mut temp_file = NamedTempFile::new_in(&dir).context(&dir)?;
         let file = temp_file.as_file_mut();
         file.write_all(index.readonly_commits().id().hex().as_bytes())
             .context(temp_file.path())?;
         let path = dir.join(op_id.hex());
-        persist_content_addressed_temp_file(temp_file, &path).context(&path)?;
+        persist_temp_file(temp_file, &path).context(&path)?;
         Ok(())
     }
 }
@@ -370,10 +573,10 @@ impl IndexStore for DefaultIndexStore {
             change_id: store.change_id_length(),
         };
         let index = match self.load_index_at_operation(op.id(), field_lengths) {
-            Err(DefaultIndexStoreError::LoadAssociation(PathError { error, .. }))
+            Err(DefaultIndexStoreError::LoadAssociation(PathError { source: error, .. }))
                 if error.kind() == io::ErrorKind::NotFound =>
             {
-                self.build_index_at_operation(op, store)
+                self.build_index_at_operation(op, store).block_on()
             }
             Err(DefaultIndexStoreError::LoadIndex(err)) if err.is_corrupt_or_not_found() => {
                 // If the index was corrupt (maybe it was written in a different format),
@@ -394,7 +597,7 @@ impl IndexStore for DefaultIndexStore {
                     }
                 }
                 self.reinit().map_err(|err| IndexReadError(err.into()))?;
-                self.build_index_at_operation(op, store)
+                self.build_index_at_operation(op, store).block_on()
             }
             result => result,
         }
