@@ -21,20 +21,20 @@ use crate::submodule_store::SubmoduleStore;
 
 #[derive(Debug)]
 pub struct DefaultSubmoduleStore {
-    #[allow(dead_code)]
+    #[expect(dead_code)]
     path: PathBuf,
 }
 
 impl DefaultSubmoduleStore {
     /// Load an existing SubmoduleStore
     pub fn load(store_path: &Path) -> Self {
-        DefaultSubmoduleStore {
+        Self {
             path: store_path.to_path_buf(),
         }
     }
 
     pub fn init(store_path: &Path) -> Self {
-        DefaultSubmoduleStore {
+        Self {
             path: store_path.to_path_buf(),
         }
     }
