@@ -41,8 +41,7 @@ pub use self::platform::try_symlink;
 #[error("Cannot access {path}")]
 pub struct PathError {
     pub path: PathBuf,
-    #[source]
-    pub error: io::Error,
+    pub source: io::Error,
 }
 
 pub trait IoResultExt<T> {
@@ -53,7 +52,7 @@ impl<T> IoResultExt<T> for io::Result<T> {
     fn context(self, path: impl AsRef<Path>) -> Result<T, PathError> {
         self.map_err(|error| PathError {
             path: path.as_ref().to_path_buf(),
-            error,
+            source: error,
         })
     }
 }
@@ -191,12 +190,33 @@ fn to_slash_separated(path: &Path) -> OsString {
     buf
 }
 
-/// Like `NamedTempFile::persist()`, but doesn't try to overwrite the existing
+/// Persists the temporary file after synchronizing the content.
+///
+/// After system crash, the persisted file should have a valid content if
+/// existed. However, the persisted file name (or directory entry) could be
+/// lost. It's up to caller to synchronize the directory entries.
+///
+/// See also <https://lwn.net/Articles/457667/> for the behavior on Linux.
+pub fn persist_temp_file<P: AsRef<Path>>(
+    temp_file: NamedTempFile,
+    new_path: P,
+) -> io::Result<File> {
+    // Ensure persisted file content is flushed to disk.
+    temp_file.as_file().sync_data()?;
+    temp_file
+        .persist(new_path)
+        .map_err(|PersistError { error, file: _ }| error)
+}
+
+/// Like [`persist_temp_file()`], but doesn't try to overwrite the existing
 /// target on Windows.
 pub fn persist_content_addressed_temp_file<P: AsRef<Path>>(
     temp_file: NamedTempFile,
     new_path: P,
 ) -> io::Result<File> {
+    // Ensure new file content is flushed to disk, so the old file content
+    // wouldn't be lost if existed at the same location.
+    temp_file.as_file().sync_data()?;
     if cfg!(windows) {
         // On Windows, overwriting file can fail if the file is opened without
         // FILE_SHARE_DELETE for example. We don't need to take a risk if the
