@@ -28,6 +28,7 @@ use std::sync::Arc;
 use bstr::BStr;
 use bstr::BString;
 use futures::StreamExt as _;
+use gix::refspec::Instruction;
 use itertools::Itertools as _;
 use pollster::FutureExt as _;
 use thiserror::Error;
@@ -189,6 +190,25 @@ impl RefSpec {
     }
 }
 
+/// Representation of a negative Git refspec
+#[derive(Debug)]
+#[repr(transparent)]
+pub(crate) struct NegativeRefSpec {
+    source: String,
+}
+
+impl NegativeRefSpec {
+    fn new(source: impl Into<String>) -> Self {
+        Self {
+            source: source.into(),
+        }
+    }
+
+    pub(crate) fn to_git_format(&self) -> String {
+        format!("^{}", self.source)
+    }
+}
+
 /// Helper struct that matches a refspec with its expected location in the
 /// remote it's being pushed to
 pub(crate) struct RefToPush<'a> {
@@ -1055,7 +1075,7 @@ fn diff_refs_to_export(
         .filter(|&((kind, symbol), _)| {
             // There are two situations where remote bookmarks get out of sync:
             // 1. `jj bookmark forget --include-remotes`
-            // 2. `jj op undo`/`restore` in colocated repo
+            // 2. `jj op revert`/`restore` in colocated repo
             kind == GitRefKind::Bookmark && git_ref_filter(kind, symbol)
         });
     for ((_kind, symbol), target) in known_git_refs {
@@ -1301,7 +1321,7 @@ pub fn reset_head(mut_repo: &mut MutableRepo, wc_commit: &Commit) -> Result<(),
             "BISECT_LOG",
         ];
         const STATE_DIR_NAMES: &[&str] = &["rebase-merge", "rebase-apply", "sequencer"];
-        let handle_err = |err: PathError| match err.error.kind() {
+        let handle_err = |err: PathError| match err.source.kind() {
             std::io::ErrorKind::NotFound => Ok(()),
             _ => Err(GitResetHeadError::from_git(err)),
         };
@@ -1772,7 +1792,9 @@ fn remove_remote_git_config_sections(
         })
         .map(|section| {
             if section.value_names().any(|name| {
-                !name.eq_ignore_ascii_case(b"url") && !name.eq_ignore_ascii_case(b"fetch")
+                !name.eq_ignore_ascii_case(b"url")
+                    && !name.eq_ignore_ascii_case(b"fetch")
+                    && !name.eq_ignore_ascii_case(b"tagOpt")
             }) {
                 return Err(GitRemoteManagementError::NonstandardConfiguration(
                     remote_name.to_owned(),
@@ -1810,6 +1832,7 @@ pub fn add_remote(
     store: &Store,
     remote_name: &RemoteName,
     url: &str,
+    fetch_tags: gix::remote::fetch::Tags,
 ) -> Result<(), GitRemoteManagementError> {
     let git_repo = get_git_repo(store)?;
 
@@ -1824,6 +1847,7 @@ pub fn add_remote(
     let mut remote = git_repo
         .remote_at(url)
         .map_err(GitRemoteManagementError::from_git)?
+        .with_fetch_tags(fetch_tags)
         .with_refspecs(
             [default_fetch_refspec(remote_name).as_bytes()],
             gix::remote::Direction::Fetch,
@@ -2099,16 +2123,36 @@ pub enum GitFetchError {
     Subprocess(#[from] GitSubprocessError),
 }
 
+#[derive(Error, Debug)]
+pub enum GitDefaultRefspecError {
+    #[error("No git remote named '{}'", .0.as_symbol())]
+    NoSuchRemote(RemoteNameBuf),
+    #[error("Invalid configuration for remote `{}`", .0.as_symbol())]
+    InvalidRemoteConfiguration(RemoteNameBuf, #[source] Box<gix::remote::find::Error>),
+}
+
 struct FetchedBranches {
     remote: RemoteNameBuf,
     branches: Vec<StringPattern>,
 }
 
-fn expand_fetch_refspecs(
+/// Represents the refspecs to fetch from a remote
+#[derive(Debug)]
+pub struct ExpandedFetchRefSpecs {
+    // NB: branch names need not necessarily map 1:1 with refspecs below:
+    // for example, we can have negative refspecs in which case there will not
+    // be an expected_branch_name entry here
+    expected_branch_names: Vec<StringPattern>,
+    refspecs: Vec<RefSpec>,
+    negative_refspecs: Vec<NegativeRefSpec>,
+}
+
+/// Expand a list of branch string patterns to refspecs to fetch
+pub fn expand_fetch_refspecs(
     remote: &RemoteName,
-    branch_names: &[StringPattern],
-) -> Result<Vec<RefSpec>, GitFetchError> {
-    branch_names
+    branch_names: Vec<StringPattern>,
+) -> Result<ExpandedFetchRefSpecs, GitFetchError> {
+    let refspecs = branch_names
         .iter()
         .map(|pattern| {
             pattern
@@ -2126,7 +2170,163 @@ fn expand_fetch_refspecs(
                 })
                 .ok_or_else(|| GitFetchError::InvalidBranchPattern(pattern.clone()))
         })
-        .collect()
+        .try_collect()?;
+
+    Ok(ExpandedFetchRefSpecs {
+        expected_branch_names: branch_names,
+        refspecs,
+        negative_refspecs: Vec::new(),
+    })
+}
+
+/// A list of fetch refspecs configured within a remote that were ignored during
+/// a expansion. Callers should consider displaying these in the UI as
+/// appropriate.
+#[derive(Debug)]
+#[must_use = "warnings should be surfaced in the UI"]
+pub struct IgnoredRefspecs(pub Vec<IgnoredRefspec>);
+
+/// A fetch refspec configured within a remote that was ignored during
+/// expansion.
+#[derive(Debug)]
+pub struct IgnoredRefspec {
+    /// The ignored refspec
+    pub refspec: BString,
+    /// The reason why it was ignored
+    pub reason: &'static str,
+}
+
+/// Expand the remote's configured fetch refspecs
+pub fn expand_default_fetch_refspecs(
+    remote: &RemoteName,
+    git_repo: &gix::Repository,
+) -> Result<(IgnoredRefspecs, ExpandedFetchRefSpecs), GitDefaultRefspecError> {
+    let remote_name = remote.as_str();
+    let remote = git_repo
+        .try_find_remote(remote_name)
+        .ok_or_else(|| GitDefaultRefspecError::NoSuchRemote(remote.to_owned()))?
+        .map_err(|e| {
+            GitDefaultRefspecError::InvalidRemoteConfiguration(remote.to_owned(), Box::new(e))
+        })?;
+
+    let remote_refspecs = remote.refspecs(gix::remote::Direction::Fetch);
+
+    let mut ignored_refspecs = Vec::with_capacity(remote_refspecs.len());
+    let mut expected_branch_names = Vec::with_capacity(remote_refspecs.len());
+    let mut negative_refspecs = Vec::new();
+
+    let refspecs = remote_refspecs
+        .iter()
+        .filter_map(|refspec| {
+            let forced = refspec.allow_non_fast_forward();
+            let refspec = refspec.to_ref();
+
+            let mut ensure_utf8 = |s| match str::from_utf8(s) {
+                Ok(s) => Some(s),
+                Err(_) => {
+                    ignored_refspecs.push(IgnoredRefspec {
+                        refspec: refspec.to_bstring(),
+                        reason: "invalid UTF-8",
+                    });
+                    None
+                }
+            };
+
+            let (src, dst) = match refspec.instruction() {
+                // Already filtered out above
+                Instruction::Push(_) => unreachable!(),
+                Instruction::Fetch(fetch) => match fetch {
+                    gix::refspec::instruction::Fetch::Only { src: _ } => {
+                        ignored_refspecs.push(IgnoredRefspec {
+                            refspec: refspec.to_bstring(),
+                            reason: "fetch-only refspecs are not supported",
+                        });
+                        return None;
+                    }
+
+                    gix::refspec::instruction::Fetch::AndUpdate {
+                        src,
+                        dst,
+                        allow_non_fast_forward: _, // Already captured above
+                    } => (ensure_utf8(src)?, ensure_utf8(dst)?),
+
+                    gix::refspec::instruction::Fetch::Exclude { src } => {
+                        let src = ensure_utf8(src)?;
+                        negative_refspecs.push(NegativeRefSpec::new(src));
+                        return None;
+                    }
+                },
+            };
+
+            if !forced {
+                ignored_refspecs.push(IgnoredRefspec {
+                    refspec: refspec.to_bstring(),
+                    reason: "non-forced refspecs are not supported",
+                });
+                return None;
+            }
+
+            let Some(src_branch) = src.strip_prefix("refs/heads/") else {
+                ignored_refspecs.push(IgnoredRefspec {
+                    refspec: refspec.to_bstring(),
+                    reason: "only refs/heads/ is supported for refspec sources",
+                });
+                return None;
+            };
+
+            let dst = {
+                let Some(dst_without_prefix) = dst.strip_prefix("refs/remotes/") else {
+                    ignored_refspecs.push(IgnoredRefspec {
+                        refspec: refspec.to_bstring(),
+                        reason: "only refs/remotes/ is supported for fetch destinations",
+                    });
+                    return None;
+                };
+
+                let Some(dst_branch) = dst_without_prefix
+                    .strip_prefix(remote_name)
+                    .and_then(|d| d.strip_prefix("/"))
+                else {
+                    ignored_refspecs.push(IgnoredRefspec {
+                        refspec: refspec.to_bstring(),
+                        reason: "remote renaming not supported",
+                    });
+                    return None;
+                };
+
+                if src_branch == dst_branch {
+                    dst.to_owned()
+                } else {
+                    ignored_refspecs.push(IgnoredRefspec {
+                        refspec: refspec.to_bstring(),
+                        reason: "renaming is not supported",
+                    });
+                    return None;
+                }
+            };
+
+            // At this point src_branch and dst_branch match
+            let Ok(branch) = StringPattern::glob(src_branch) else {
+                ignored_refspecs.push(IgnoredRefspec {
+                    refspec: refspec.to_bstring(),
+                    reason: "invalid pattern",
+                });
+                return None;
+            };
+            expected_branch_names.push(branch);
+
+            Some(RefSpec::forced(src, dst))
+        })
+        .collect();
+
+    Ok((
+        IgnoredRefspecs(ignored_refspecs),
+        ExpandedFetchRefSpecs {
+            expected_branch_names,
+            refspecs,
+            negative_refspecs,
+        },
+    ))
 }
 
 /// Helper struct to execute multiple `git fetch` operations
@@ -2165,9 +2365,14 @@ impl<'a> GitFetch<'a> {
     pub fn fetch(
         &mut self,
         remote_name: &RemoteName,
-        branch_names: &[StringPattern],
+        ExpandedFetchRefSpecs {
+            expected_branch_names,
+            refspecs: mut remaining_refspecs,
+            negative_refspecs,
+        }: ExpandedFetchRefSpecs,
         mut callbacks: RemoteCallbacks<'_>,
         depth: Option<NonZeroU32>,
+        fetch_tags_override: Option<FetchTagsOverride>,
     ) -> Result<(), GitFetchError> {
         validate_remote_name(remote_name)?;
 
@@ -2179,9 +2384,7 @@ impl<'a> GitFetch<'a> {
         {
             return Err(GitFetchError::NoSuchRemote(remote_name.to_owned()));
         }
-        // At this point, we are only updating Git's remote tracking branches, not the
-        // local branches.
-        let mut remaining_refspecs: Vec<_> = expand_fetch_refspecs(remote_name, branch_names)?;
+
         if remaining_refspecs.is_empty() {
             // Don't fall back to the base refspecs.
             return Ok(());
@@ -2195,10 +2398,14 @@ impl<'a> GitFetch<'a> {
         //
         // even more unfortunately, git errors out one refspec at a time,
         // meaning that the below cycle runs in O(#failed refspecs)
-        while let Some(failing_refspec) =
-            self.git_ctx
-                .spawn_fetch(remote_name, &remaining_refspecs, &mut callbacks, depth)?
-        {
+        while let Some(failing_refspec) = self.git_ctx.spawn_fetch(
+            remote_name,
+            &remaining_refspecs,
+            &negative_refspecs,
+            &mut callbacks,
+            depth,
+            fetch_tags_override,
+        )? {
             tracing::debug!(failing_refspec, "failed to fetch ref");
             remaining_refspecs.retain(|r| r.source.as_ref() != Some(&failing_refspec));
 
@@ -2216,7 +2423,7 @@ impl<'a> GitFetch<'a> {
 
         self.fetched.push(FetchedBranches {
             remote: remote_name.to_owned(),
-            branches: branch_names.to_vec(),
+            branches: expected_branch_names,
         });
         Ok(())
     }
@@ -2414,3 +2621,15 @@ pub struct Progress {
     pub bytes_downloaded: Option<u64>,
     pub overall: f32,
 }
+
+/// Allows temporarily overriding the behavior of a single `git fetch`
+/// operation as to whether tags are fetched
+#[derive(Copy, Clone, Debug)]
+pub enum FetchTagsOverride {
+    /// For this one fetch attempt, fetch all tags regardless of what the
+    /// remote's `tagOpt` is configured to
+    AllTags,
+    /// For this one fetch attempt, fetch no tags regardless of what the
+    /// remote's `tagOpt` is configured to
+    NoTags,
+}
