@@ -41,6 +41,7 @@ use gix::bstr::BString;
 use gix::objs::CommitRefIter;
 use gix::objs::WriteTo as _;
 use itertools::Itertools as _;
+use once_cell::sync::OnceCell as OnceLock;
 use pollster::FutureExt as _;
 use prost::Message as _;
 use smallvec::SmallVec;
@@ -48,7 +49,6 @@ use thiserror::Error;
 use tokio::io::AsyncRead;
 use tokio::io::AsyncReadExt as _;
 
-use crate::backend::make_root_commit;
 use crate::backend::Backend;
 use crate::backend::BackendError;
 use crate::backend::BackendInitError;
@@ -74,6 +74,7 @@ use crate::backend::Timestamp;
 use crate::backend::Tree;
 use crate::backend::TreeId;
 use crate::backend::TreeValue;
+use crate::backend::make_root_commit;
 use crate::config::ConfigGetError;
 use crate::file_util;
 use crate::file_util::BadPathEncoding;
@@ -121,7 +122,7 @@ pub enum GitBackendInitError {
 
 impl From<Box<GitBackendInitError>> for BackendInitError {
     fn from(err: Box<GitBackendInitError>) -> Self {
-        BackendInitError(err)
+        Self(err)
     }
 }
 
@@ -139,7 +140,7 @@ pub enum GitBackendLoadError {
 
 impl From<Box<GitBackendLoadError>> for BackendLoadError {
     fn from(err: Box<GitBackendLoadError>) -> Self {
-        BackendLoadError(err)
+        Self(err)
     }
 }
 
@@ -154,7 +155,7 @@ pub enum GitBackendError {
 
 impl From<GitBackendError> for BackendError {
     fn from(err: GitBackendError) -> Self {
-        BackendError::Other(err.into())
+        Self::Other(err.into())
     }
 }
 
@@ -176,6 +177,7 @@ pub struct GitBackend {
     root_commit_id: CommitId,
     root_change_id: ChangeId,
     empty_tree_id: TreeId,
+    shallow_root_ids: OnceLock<Vec<CommitId>>,
     extra_metadata_store: TableStore,
     cached_extra_metadata: Mutex<Option<Arc<ReadonlyTable>>>,
     git_executable: PathBuf,
@@ -196,12 +198,13 @@ impl GitBackend {
         let root_commit_id = CommitId::from_bytes(&[0; HASH_LENGTH]);
         let root_change_id = ChangeId::from_bytes(&[0; CHANGE_ID_LENGTH]);
         let empty_tree_id = TreeId::from_hex("4b825dc642cb6eb9a060e54bf8d69288fbee4904");
-        GitBackend {
+        Self {
             base_repo,
             repo,
             root_commit_id,
             root_change_id,
             empty_tree_id,
+            shallow_root_ids: OnceLock::new(),
             extra_metadata_store,
             cached_extra_metadata: Mutex::new(None),
             git_executable: git_settings.executable_path,
@@ -305,7 +308,7 @@ impl GitBackend {
             .context(&target_path)
             .map_err(GitBackendInitError::Path)?;
         let extra_metadata_store = TableStore::init(extra_path, HASH_LENGTH);
-        Ok(GitBackend::new(repo, extra_metadata_store, git_settings))
+        Ok(Self::new(repo, extra_metadata_store, git_settings))
     }
 
     pub fn load(
@@ -333,7 +336,7 @@ impl GitBackend {
         let git_settings = settings
             .git_settings()
             .map_err(GitBackendLoadError::Config)?;
-        Ok(GitBackend::new(repo, extra_metadata_store, git_settings))
+        Ok(Self::new(repo, extra_metadata_store, git_settings))
     }
 
     fn lock_git_repo(&self) -> MutexGuard<'_, gix::Repository> {
@@ -355,6 +358,25 @@ impl GitBackend {
         self.base_repo.work_dir()
     }
 
+    fn shallow_root_ids(&self, git_repo: &gix::Repository) -> BackendResult<&[CommitId]> {
+        // The list of shallow roots is cached by gix, but it's still expensive
+        // to stat file on every read_object() call. Refreshing shallow roots is
+        // also bad for consistency reasons.
+        self.shallow_root_ids
+            .get_or_try_init(|| {
+                let maybe_oids = git_repo
+                    .shallow_commits()
+                    .map_err(|err| BackendError::Other(err.into()))?;
+                let commit_ids = maybe_oids.map_or(vec![], |oids| {
+                    oids.iter()
+                        .map(|oid| CommitId::from_bytes(oid.as_bytes()))
+                        .collect()
+                });
+                Ok(commit_ids)
+            })
+            .map(AsRef::as_ref)
+    }
+
     fn cached_extra_metadata_table(&self) -> BackendResult<Arc<ReadonlyTable>> {
         let mut locked_head = self.cached_extra_metadata.lock().unwrap();
         match locked_head.as_ref() {
@@ -430,6 +452,7 @@ impl GitBackend {
             &mut mut_table,
             &table_lock,
             &head_ids,
+            self.shallow_root_ids(&locked_repo)?,
         )?;
         self.save_extra_metadata_table(mut_table, &table_lock)
     }
@@ -524,7 +547,7 @@ fn gix_open_opts_from_settings(settings: &UserSettings) -> gix::open::Options {
 /// Parses the `jj:trees` header value.
 fn root_tree_from_git_extra_header(value: &BStr) -> Result<MergedTreeId, ()> {
     let mut tree_ids = SmallVec::new();
-    for hex in str::from_utf8(value.as_ref()).or(Err(()))?.split(' ') {
+    for hex in value.split(|b| *b == b' ') {
         let tree_id = TreeId::try_from_hex(hex).ok_or(())?;
         if tree_id.as_bytes().len() != HASH_LENGTH {
             return Err(());
@@ -910,11 +933,8 @@ fn import_extra_metadata_entries_from_heads(
     mut_table: &mut MutableTable,
     _table_lock: &FileLock,
     head_ids: &HashSet<&CommitId>,
+    shallow_roots: &[CommitId],
 ) -> BackendResult<()> {
-    let shallow_commits = git_repo
-        .shallow_commits()
-        .map_err(|e| BackendError::Other(Box::new(e)))?;
-
     let mut work_ids = head_ids
         .iter()
         .filter(|&id| mut_table.get_value(id.as_bytes()).is_none())
@@ -924,9 +944,7 @@ fn import_extra_metadata_entries_from_heads(
         let git_object = git_repo
             .find_object(validate_git_object_id(&id)?)
             .map_err(|err| map_not_found_err(err, &id))?;
-        let is_shallow = shallow_commits
-            .as_ref()
-            .is_some_and(|shallow| shallow.contains(&git_object.id));
+        let is_shallow = shallow_roots.contains(&id);
         // TODO(#1624): Should we read the root tree here and check if it has a
         // `.jjconflict-...` entries? That could happen if the user used `git` to e.g.
         // change the description of a commit with tree-level conflicts.
@@ -1065,57 +1083,64 @@ impl Backend for GitBackend {
             .map_err(|err| map_not_found_err(err, id))?
             .try_into_tree()
             .map_err(|err| to_read_object_err(err, id))?;
-        let mut tree = Tree::default();
-        for entry in git_tree.iter() {
-            let entry = entry.map_err(|err| to_read_object_err(err, id))?;
-            let name =
-                str::from_utf8(entry.filename()).map_err(|err| to_invalid_utf8_err(err, id))?;
-            let (name, value) = match entry.mode().kind() {
-                gix::object::tree::EntryKind::Tree => {
-                    let id = TreeId::from_bytes(entry.oid().as_bytes());
-                    (name, TreeValue::Tree(id))
-                }
-                gix::object::tree::EntryKind::Blob => {
-                    let id = FileId::from_bytes(entry.oid().as_bytes());
-                    if let Some(basename) = name.strip_suffix(CONFLICT_SUFFIX) {
-                        (
-                            basename,
-                            TreeValue::Conflict(ConflictId::from_bytes(entry.oid().as_bytes())),
-                        )
-                    } else {
+        let mut entries: Vec<_> = git_tree
+            .iter()
+            .map(|entry| -> BackendResult<_> {
+                let entry = entry.map_err(|err| to_read_object_err(err, id))?;
+                let name =
+                    str::from_utf8(entry.filename()).map_err(|err| to_invalid_utf8_err(err, id))?;
+                let (name, value) = match entry.mode().kind() {
+                    gix::object::tree::EntryKind::Tree => {
+                        let id = TreeId::from_bytes(entry.oid().as_bytes());
+                        (name, TreeValue::Tree(id))
+                    }
+                    gix::object::tree::EntryKind::Blob => {
+                        let id = FileId::from_bytes(entry.oid().as_bytes());
+                        if let Some(basename) = name.strip_suffix(CONFLICT_SUFFIX) {
+                            (
+                                basename,
+                                TreeValue::Conflict(ConflictId::from_bytes(entry.oid().as_bytes())),
+                            )
+                        } else {
+                            (
+                                name,
+                                TreeValue::File {
+                                    id,
+                                    executable: false,
+                                    copy_id: CopyId::placeholder(),
+                                },
+                            )
+                        }
+                    }
+                    gix::object::tree::EntryKind::BlobExecutable => {
+                        let id = FileId::from_bytes(entry.oid().as_bytes());
                         (
                             name,
                             TreeValue::File {
                                 id,
-                                executable: false,
+                                executable: true,
                                 copy_id: CopyId::placeholder(),
                             },
                         )
                     }
-                }
-                gix::object::tree::EntryKind::BlobExecutable => {
-                    let id = FileId::from_bytes(entry.oid().as_bytes());
-                    (
-                        name,
-                        TreeValue::File {
-                            id,
-                            executable: true,
-                            copy_id: CopyId::placeholder(),
-                        },
-                    )
-                }
-                gix::object::tree::EntryKind::Link => {
-                    let id = SymlinkId::from_bytes(entry.oid().as_bytes());
-                    (name, TreeValue::Symlink(id))
-                }
-                gix::object::tree::EntryKind::Commit => {
-                    let id = CommitId::from_bytes(entry.oid().as_bytes());
-                    (name, TreeValue::GitSubmodule(id))
-                }
-            };
-            tree.set(RepoPathComponentBuf::new(name).unwrap(), value);
+                    gix::object::tree::EntryKind::Link => {
+                        let id = SymlinkId::from_bytes(entry.oid().as_bytes());
+                        (name, TreeValue::Symlink(id))
+                    }
+                    gix::object::tree::EntryKind::Commit => {
+                        let id = CommitId::from_bytes(entry.oid().as_bytes());
+                        (name, TreeValue::GitSubmodule(id))
+                    }
+                };
+                Ok((RepoPathComponentBuf::new(name).unwrap(), value))
+            })
+            .try_collect()?;
+        // While Git tree entries are sorted, the rule is slightly different.
+        // Directory names are sorted as if they had trailing "/".
+        if !entries.is_sorted_by_key(|(name, _)| name) {
+            entries.sort_unstable_by(|(a, _), (b, _)| a.cmp(b));
         }
-        Ok(tree)
+        Ok(Tree::from_sorted_entries(entries))
     }
 
     async fn write_tree(&self, _path: &RepoPath, contents: &Tree) -> BackendResult<TreeId> {
@@ -1219,11 +1244,7 @@ impl Backend for GitBackend {
             let git_object = locked_repo
                 .find_object(git_commit_id)
                 .map_err(|err| map_not_found_err(err, id))?;
-            let is_shallow = locked_repo
-                .shallow_commits()
-                .ok()
-                .flatten()
-                .is_some_and(|shallow| shallow.contains(&git_object.id));
+            let is_shallow = self.shallow_root_ids(&locked_repo)?.contains(id);
             commit_from_git_without_root_parent(id, &git_object, false, is_shallow)?
         };
         if commit.parents.is_empty() {
@@ -1858,10 +1879,12 @@ mod tests {
 
         // read_commit() without import_head_commits() works as of now. This might be
         // changed later.
-        assert!(backend
-            .read_commit(&CommitId::from_bytes(git_commit_id.as_bytes()))
-            .block_on()
-            .is_ok());
+        assert!(
+            backend
+                .read_commit(&CommitId::from_bytes(git_commit_id.as_bytes()))
+                .block_on()
+                .is_ok()
+        );
         assert!(
             backend
                 .cached_extra_metadata_table()
@@ -2142,11 +2165,13 @@ mod tests {
             ))
             .unwrap();
         let git_tree = git_repo.find_tree(git_commit.tree_id().unwrap()).unwrap();
-        assert!(git_tree
-            .iter()
-            .map(Result::unwrap)
-            .filter(|entry| entry.filename() != b"README")
-            .all(|entry| entry.mode().value() == 0o040000));
+        assert!(
+            git_tree
+                .iter()
+                .map(Result::unwrap)
+                .filter(|entry| entry.filename() != b"README")
+                .all(|entry| entry.mode().value() == 0o040000)
+        );
         let mut iter = git_tree.iter().map(Result::unwrap);
         let entry = iter.next().unwrap();
         assert_eq!(entry.filename(), b".jjconflict-base-0");
@@ -2279,12 +2304,14 @@ mod tests {
         backend
             .import_head_commits([&commit_id, &commit_id])
             .unwrap();
-        assert!(git_repo
-            .references()
-            .unwrap()
-            .prefixed("refs/jj/keep/")
-            .unwrap()
-            .any(|git_ref| git_ref.unwrap().id().detach() == git_commit_id));
+        assert!(
+            git_repo
+                .references()
+                .unwrap()
+                .prefixed("refs/jj/keep/")
+                .unwrap()
+                .any(|git_ref| git_ref.unwrap().id().detach() == git_commit_id)
+        );
     }
 
     #[test]
