@@ -28,7 +28,9 @@ use bstr::ByteSlice as _;
 use itertools::Itertools as _;
 use thiserror::Error;
 
+use crate::git::FetchTagsOverride;
 use crate::git::GitPushStats;
+use crate::git::NegativeRefSpec;
 use crate::git::Progress;
 use crate::git::RefSpec;
 use crate::git::RefToPush;
@@ -159,8 +161,10 @@ impl<'a> GitSubprocessContext<'a> {
         &self,
         remote_name: &RemoteName,
         refspecs: &[RefSpec],
+        negative_refspecs: &[NegativeRefSpec],
         callbacks: &mut RemoteCallbacks<'_>,
         depth: Option<NonZeroU32>,
+        fetch_tags_override: Option<FetchTagsOverride>,
     ) -> Result<Option<String>, GitSubprocessError> {
         if refspecs.is_empty() {
             return Ok(None);
@@ -176,8 +180,22 @@ impl<'a> GitSubprocessContext<'a> {
         if let Some(d) = depth {
             command.arg(format!("--depth={d}"));
         }
+        match fetch_tags_override {
+            Some(FetchTagsOverride::AllTags) => {
+                command.arg("--tags");
+            }
+            Some(FetchTagsOverride::NoTags) => {
+                command.arg("--no-tags");
+            }
+            None => {}
+        }
         command.arg("--").arg(remote_name.as_str());
-        command.args(refspecs.iter().map(|x| x.to_git_format()));
+        command.args(
+            refspecs
+                .iter()
+                .map(|x| x.to_git_format())
+                .chain(negative_refspecs.iter().map(|x| x.to_git_format())),
+        );
 
         let output = wait_with_progress(self.spawn_cmd(command)?, callbacks)?;
 
