@@ -59,6 +59,18 @@ impl<N> GraphEdge<N> {
             edge_type: self.edge_type,
         }
     }
+
+    pub fn is_missing(&self) -> bool {
+        self.edge_type == GraphEdgeType::Missing
+    }
+
+    pub fn is_direct(&self) -> bool {
+        self.edge_type == GraphEdgeType::Direct
+    }
+
+    pub fn is_indirect(&self) -> bool {
+        self.edge_type == GraphEdgeType::Indirect
+    }
 }
 
 #[derive(Debug, PartialEq, Eq, Clone, Copy, Hash)]
@@ -71,7 +83,7 @@ pub enum GraphEdgeType {
 fn reachable_targets<N>(edges: &[GraphEdge<N>]) -> impl DoubleEndedIterator<Item = &N> {
     edges
         .iter()
-        .filter(|edge| edge.edge_type != GraphEdgeType::Missing)
+        .filter(|edge| !edge.is_missing())
         .map(|edge| &edge.target)
 }
 
@@ -155,7 +167,7 @@ where
     /// Wraps the given iterator to group topological branches. The input
     /// iterator must be topologically ordered.
     pub fn new(input_iter: I, as_id: F) -> Self {
-        TopoGroupedGraphIterator {
+        Self {
             input_iter,
             as_id,
             nodes: HashMap::new(),
