@@ -45,7 +45,7 @@ macro_rules! id_type {
         $vis:vis $name:ident { $hex_method:ident() }
     ) => {
         $(#[$attr])*
-        #[derive(ContentHash, PartialEq, Eq, PartialOrd, Ord, Clone, Hash)]
+        #[derive($crate::content_hash::ContentHash, PartialEq, Eq, PartialOrd, Ord, Clone, Hash)]
         $vis struct $name(Vec<u8>);
         $crate::object_id::impl_id_type!($name, $hex_method);
     };
@@ -53,6 +53,7 @@ macro_rules! id_type {
 
 macro_rules! impl_id_type {
     ($name:ident, $hex_method:ident) => {
+        #[allow(dead_code)]
         impl $name {
             pub fn new(value: Vec<u8>) -> Self {
                 Self(value)
@@ -71,7 +72,7 @@ macro_rules! impl_id_type {
             }
 
             /// Parses the given hex string into an ObjectId.
-            pub fn try_from_hex(hex: &str) -> Option<Self> {
+            pub fn try_from_hex(hex: impl AsRef<[u8]>) -> Option<Self> {
                 $crate::hex_util::decode_hex(hex).map(Self)
             }
         }
@@ -142,9 +143,9 @@ pub struct HexPrefix {
 impl HexPrefix {
     /// Returns a new `HexPrefix` or `None` if `prefix` cannot be decoded from
     /// hex to bytes.
-    pub fn try_from_hex(prefix: &str) -> Option<HexPrefix> {
+    pub fn try_from_hex(prefix: impl AsRef<[u8]>) -> Option<Self> {
         let (min_prefix_bytes, has_odd_byte) = hex_util::decode_hex_prefix(prefix)?;
-        Some(HexPrefix {
+        Some(Self {
             min_prefix_bytes,
             has_odd_byte,
         })
@@ -152,16 +153,16 @@ impl HexPrefix {
 
     /// Returns a new `HexPrefix` or `None` if `prefix` cannot be decoded from
     /// "reverse" hex to bytes.
-    pub fn try_from_reverse_hex(prefix: &str) -> Option<HexPrefix> {
+    pub fn try_from_reverse_hex(prefix: impl AsRef<[u8]>) -> Option<Self> {
         let (min_prefix_bytes, has_odd_byte) = hex_util::decode_reverse_hex_prefix(prefix)?;
-        Some(HexPrefix {
+        Some(Self {
             min_prefix_bytes,
             has_odd_byte,
         })
     }
 
     pub fn from_bytes(bytes: &[u8]) -> Self {
-        HexPrefix {
+        Self {
             min_prefix_bytes: bytes.to_owned(),
             has_odd_byte: false,
         }
@@ -244,23 +245,21 @@ pub enum PrefixResolution<T> {
 impl<T> PrefixResolution<T> {
     pub fn map<U>(self, f: impl FnOnce(T) -> U) -> PrefixResolution<U> {
         match self {
-            PrefixResolution::NoMatch => PrefixResolution::NoMatch,
-            PrefixResolution::SingleMatch(x) => PrefixResolution::SingleMatch(f(x)),
-            PrefixResolution::AmbiguousMatch => PrefixResolution::AmbiguousMatch,
+            Self::NoMatch => PrefixResolution::NoMatch,
+            Self::SingleMatch(x) => PrefixResolution::SingleMatch(f(x)),
+            Self::AmbiguousMatch => PrefixResolution::AmbiguousMatch,
         }
     }
 }
 
 impl<T: Clone> PrefixResolution<T> {
-    pub fn plus(&self, other: &PrefixResolution<T>) -> PrefixResolution<T> {
+    pub fn plus(&self, other: &Self) -> Self {
         match (self, other) {
-            (PrefixResolution::NoMatch, other) => other.clone(),
-            (local, PrefixResolution::NoMatch) => local.clone(),
-            (PrefixResolution::AmbiguousMatch, _) => PrefixResolution::AmbiguousMatch,
-            (_, PrefixResolution::AmbiguousMatch) => PrefixResolution::AmbiguousMatch,
-            (PrefixResolution::SingleMatch(_), PrefixResolution::SingleMatch(_)) => {
-                PrefixResolution::AmbiguousMatch
-            }
+            (Self::NoMatch, other) => other.clone(),
+            (local, Self::NoMatch) => local.clone(),
+            (Self::AmbiguousMatch, _) => Self::AmbiguousMatch,
+            (_, Self::AmbiguousMatch) => Self::AmbiguousMatch,
+            (Self::SingleMatch(_), Self::SingleMatch(_)) => Self::AmbiguousMatch,
         }
     }
 }
