@@ -20,10 +20,10 @@ use std::collections::HashMap;
 use std::collections::HashSet;
 use std::fmt::Debug;
 use std::iter;
+use std::sync::LazyLock;
 use std::time::SystemTime;
 
 use itertools::Itertools as _;
-use once_cell::sync::Lazy;
 use thiserror::Error;
 
 use crate::backend::CommitId;
@@ -31,10 +31,10 @@ use crate::backend::MillisSinceEpoch;
 use crate::backend::Timestamp;
 use crate::content_hash::ContentHash;
 use crate::merge::Merge;
-use crate::object_id::id_type;
 use crate::object_id::HexPrefix;
 use crate::object_id::ObjectId as _;
 use crate::object_id::PrefixResolution;
+use crate::object_id::id_type;
 use crate::ref_name::GitRefNameBuf;
 use crate::ref_name::RefName;
 use crate::ref_name::RefNameBuf;
@@ -68,7 +68,7 @@ impl RefTarget {
     ///
     /// This will typically be used in place of `None` returned by map lookup.
     pub fn absent_ref() -> &'static Self {
-        static TARGET: Lazy<RefTarget> = Lazy::new(RefTarget::absent);
+        static TARGET: LazyLock<RefTarget> = LazyLock::new(RefTarget::absent);
         &TARGET
     }
 
@@ -91,7 +91,7 @@ impl RefTarget {
     }
 
     pub fn from_merge(merge: Merge<Option<CommitId>>) -> Self {
-        RefTarget { merge }
+        Self { merge }
     }
 
     /// Returns the underlying value if this target is non-conflicting.
@@ -143,7 +143,7 @@ pub struct RemoteRef {
 impl RemoteRef {
     /// Creates remote ref pointing to no commit.
     pub fn absent() -> Self {
-        RemoteRef {
+        Self {
             target: RefTarget::absent(),
             state: RemoteRefState::New,
         }
@@ -153,7 +153,7 @@ impl RemoteRef {
     ///
     /// This will typically be used in place of `None` returned by map lookup.
     pub fn absent_ref() -> &'static Self {
-        static TARGET: Lazy<RemoteRef> = Lazy::new(RemoteRef::absent);
+        static TARGET: LazyLock<RemoteRef> = LazyLock::new(RemoteRef::absent);
         &TARGET
     }
 
@@ -269,7 +269,7 @@ impl View {
     /// The caller should add at least one commit ID to `head_ids`. The other
     /// fields may be empty.
     pub fn empty() -> Self {
-        View {
+        Self {
             head_ids: HashSet::new(),
             local_bookmarks: BTreeMap::new(),
             tags: BTreeMap::new(),
@@ -282,7 +282,7 @@ impl View {
 
     /// Creates new (mostly empty) view containing the given commit as the head.
     pub fn make_root(root_commit_id: CommitId) -> Self {
-        View {
+        Self {
             head_ids: HashSet::from([root_commit_id]),
             local_bookmarks: BTreeMap::new(),
             tags: BTreeMap::new(),
@@ -399,7 +399,7 @@ pub struct Operation {
 }
 
 impl Operation {
-    pub fn make_root(root_view_id: ViewId) -> Operation {
+    pub fn make_root(root_view_id: ViewId) -> Self {
         let timestamp = Timestamp {
             timestamp: MillisSinceEpoch(0),
             tz_offset: 0,
@@ -415,7 +415,7 @@ impl Operation {
             is_snapshot: false,
             tags: HashMap::new(),
         };
-        Operation {
+        Self {
             view_id: root_view_id,
             parents: vec![],
             metadata,
@@ -584,7 +584,7 @@ mod tests {
                 "bookmark1".as_ref(),
                 BookmarkTarget {
                     local_target: &local_bookmark1_target,
-                    remote_refs: vec![],
+                    remote_refs: vec![]
                 },
             )],
         );
