@@ -20,6 +20,7 @@ use std::fmt::Error;
 use std::fmt::Formatter;
 use std::hash::Hash;
 use std::hash::Hasher;
+use std::iter;
 use std::sync::Arc;
 
 use crate::backend::CommitId;
@@ -80,7 +81,7 @@ impl Operation {
         id: OperationId,
         data: impl Into<Arc<op_store::Operation>>,
     ) -> Self {
-        Operation {
+        Self {
             op_store,
             id,
             data: data.into(),
@@ -103,11 +104,11 @@ impl Operation {
         &self.data.parents
     }
 
-    pub fn parents(&self) -> impl ExactSizeIterator<Item = OpStoreResult<Operation>> + use<'_> {
+    pub fn parents(&self) -> impl ExactSizeIterator<Item = OpStoreResult<Self>> + use<'_> {
         let op_store = &self.op_store;
         self.data.parents.iter().map(|parent_id| {
             let data = op_store.read_operation(parent_id)?;
-            Ok(Operation::new(op_store.clone(), parent_id.clone(), data))
+            Ok(Self::new(op_store.clone(), parent_id.clone(), data))
         })
     }
 
@@ -133,6 +134,21 @@ impl Operation {
         Some(map.get(commit_id)?)
     }
 
+    /// Iterates all commit ids referenced by this operation ignoring the view.
+    ///
+    /// Use this in addition to [`View::all_referenced_commit_ids()`] to build
+    /// commit index from scratch. The predecessor commit ids are also included,
+    /// which ensures that the old commits to be returned by
+    /// [`Self::predecessors_for_commit()`] are still reachable.
+    ///
+    /// The iteration order is unspecified.
+    pub fn all_referenced_commit_ids(&self) -> impl Iterator<Item = &CommitId> {
+        self.data.commit_predecessors.iter().flat_map(|map| {
+            map.iter()
+                .flat_map(|(new_id, old_ids)| iter::once(new_id).chain(old_ids))
+        })
+    }
+
     pub fn store_operation(&self) -> &op_store::Operation {
         &self.data
     }
