@@ -37,7 +37,6 @@ use tempfile::NamedTempFile;
 use tokio::io::AsyncRead;
 use tokio::io::AsyncReadExt as _;
 
-use crate::backend::make_root_commit;
 use crate::backend::Backend;
 use crate::backend::BackendError;
 use crate::backend::BackendResult;
@@ -61,6 +60,7 @@ use crate::backend::Timestamp;
 use crate::backend::Tree;
 use crate::backend::TreeId;
 use crate::backend::TreeValue;
+use crate::backend::make_root_commit;
 use crate::content_hash::blake2b_hash;
 use crate::file_util::persist_content_addressed_temp_file;
 use crate::index::Index;
@@ -127,7 +127,7 @@ impl SimpleBackend {
         let empty_tree_id = TreeId::from_hex(
             "482ae5a29fbe856c7272f2071b8b0f0359ee2d89ff392b8a900643fbd0836eccd067b8bf41909e206c90d45d6e7d8b6686b93ecaee5fe1a9060d87b672101310",
         );
-        SimpleBackend {
+        Self {
             path: store_path.to_path_buf(),
             root_commit_id,
             root_change_id,
@@ -456,12 +456,16 @@ fn tree_to_proto(tree: &Tree) -> crate::protos::simple_store::Tree {
 }
 
 fn tree_from_proto(proto: crate::protos::simple_store::Tree) -> Tree {
-    let mut tree = Tree::default();
-    for proto_entry in proto.entries {
-        let value = tree_value_from_proto(proto_entry.value.unwrap());
-        tree.set(RepoPathComponentBuf::new(proto_entry.name).unwrap(), value);
-    }
-    tree
+    // Serialized data should be sorted
+    let entries = proto
+        .entries
+        .into_iter()
+        .map(|proto_entry| {
+            let value = tree_value_from_proto(proto_entry.value.unwrap());
+            (RepoPathComponentBuf::new(proto_entry.name).unwrap(), value)
+        })
+        .collect();
+    Tree::from_sorted_entries(entries)
 }
 
 fn tree_value_to_proto(value: &TreeValue) -> crate::protos::simple_store::TreeValue {
