@@ -15,10 +15,9 @@
 #![allow(missing_docs)]
 
 use std::collections::BTreeMap;
-use std::collections::HashMap;
 use std::collections::HashSet;
 
-use itertools::Itertools;
+use itertools::Itertools as _;
 use thiserror::Error;
 
 use crate::backend::CommitId;
@@ -27,10 +26,16 @@ use crate::op_store::BookmarkTarget;
 use crate::op_store::RefTarget;
 use crate::op_store::RefTargetOptionExt as _;
 use crate::op_store::RemoteRef;
-use crate::op_store::WorkspaceId;
+use crate::ref_name::GitRefName;
+use crate::ref_name::GitRefNameBuf;
+use crate::ref_name::RefName;
+use crate::ref_name::RefNameBuf;
+use crate::ref_name::RemoteName;
+use crate::ref_name::RemoteRefSymbol;
+use crate::ref_name::WorkspaceName;
+use crate::ref_name::WorkspaceNameBuf;
 use crate::refs;
 use crate::refs::LocalAndRemoteRef;
-use crate::refs::RemoteRefSymbol;
 use crate::str_util::StringPattern;
 
 /// A wrapper around [`op_store::View`] that defines additional methods.
@@ -41,27 +46,27 @@ pub struct View {
 
 impl View {
     pub fn new(op_store_view: op_store::View) -> Self {
-        View {
+        Self {
             data: op_store_view,
         }
     }
 
-    pub fn wc_commit_ids(&self) -> &HashMap<WorkspaceId, CommitId> {
+    pub fn wc_commit_ids(&self) -> &BTreeMap<WorkspaceNameBuf, CommitId> {
         &self.data.wc_commit_ids
     }
 
-    pub fn get_wc_commit_id(&self, workspace_id: &WorkspaceId) -> Option<&CommitId> {
-        self.data.wc_commit_ids.get(workspace_id)
+    pub fn get_wc_commit_id(&self, name: &WorkspaceName) -> Option<&CommitId> {
+        self.data.wc_commit_ids.get(name)
     }
 
-    pub fn workspaces_for_wc_commit_id(&self, commit_id: &CommitId) -> Vec<WorkspaceId> {
-        let mut workspaces_ids = vec![];
-        for (workspace_id, wc_commit_id) in &self.data.wc_commit_ids {
+    pub fn workspaces_for_wc_commit_id(&self, commit_id: &CommitId) -> Vec<WorkspaceNameBuf> {
+        let mut workspace_names = vec![];
+        for (name, wc_commit_id) in &self.data.wc_commit_ids {
             if wc_commit_id == commit_id {
-                workspaces_ids.push(workspace_id.clone());
+                workspace_names.push(name.clone());
             }
         }
-        workspaces_ids
+        workspace_names
     }
 
     pub fn is_wc_commit_id(&self, commit_id: &CommitId) -> bool {
@@ -73,15 +78,15 @@ impl View {
     }
 
     /// Iterates pair of local and remote bookmarks by bookmark name.
-    pub fn bookmarks(&self) -> impl Iterator<Item = (&str, BookmarkTarget<'_>)> {
+    pub fn bookmarks(&self) -> impl Iterator<Item = (&RefName, BookmarkTarget<'_>)> {
         op_store::merge_join_bookmark_views(&self.data.local_bookmarks, &self.data.remote_views)
     }
 
-    pub fn tags(&self) -> &BTreeMap<String, RefTarget> {
+    pub fn tags(&self) -> &BTreeMap<RefNameBuf, RefTarget> {
         &self.data.tags
     }
 
-    pub fn git_refs(&self) -> &BTreeMap<String, RefTarget> {
+    pub fn git_refs(&self) -> &BTreeMap<GitRefNameBuf, RefTarget> {
         &self.data.git_refs
     }
 
@@ -89,34 +94,30 @@ impl View {
         &self.data.git_head
     }
 
-    pub fn set_wc_commit(&mut self, workspace_id: WorkspaceId, commit_id: CommitId) {
-        self.data.wc_commit_ids.insert(workspace_id, commit_id);
+    pub fn set_wc_commit(&mut self, name: WorkspaceNameBuf, commit_id: CommitId) {
+        self.data.wc_commit_ids.insert(name, commit_id);
     }
 
-    pub fn remove_wc_commit(&mut self, workspace_id: &WorkspaceId) {
-        self.data.wc_commit_ids.remove(workspace_id);
+    pub fn remove_wc_commit(&mut self, name: &WorkspaceName) {
+        self.data.wc_commit_ids.remove(name);
     }
 
     pub fn rename_workspace(
         &mut self,
-        old_workspace_id: &WorkspaceId,
-        new_workspace_id: WorkspaceId,
+        old_name: &WorkspaceName,
+        new_name: WorkspaceNameBuf,
     ) -> Result<(), RenameWorkspaceError> {
-        if self.data.wc_commit_ids.contains_key(&new_workspace_id) {
+        if self.data.wc_commit_ids.contains_key(&new_name) {
             return Err(RenameWorkspaceError::WorkspaceAlreadyExists {
-                workspace_id: new_workspace_id.as_str().to_owned(),
+                name: new_name.clone(),
             });
         }
-        let wc_commit_id = self
-            .data
-            .wc_commit_ids
-            .remove(old_workspace_id)
-            .ok_or_else(|| RenameWorkspaceError::WorkspaceDoesNotExist {
-                workspace_id: old_workspace_id.as_str().to_owned(),
-            })?;
-        self.data
-            .wc_commit_ids
-            .insert(new_workspace_id, wc_commit_id);
+        let wc_commit_id = self.data.wc_commit_ids.remove(old_name).ok_or_else(|| {
+            RenameWorkspaceError::WorkspaceDoesNotExist {
+                name: old_name.to_owned(),
+            }
+        })?;
+        self.data.wc_commit_ids.insert(new_name, wc_commit_id);
         Ok(())
     }
 
@@ -129,7 +130,7 @@ impl View {
     }
 
     /// Iterates local bookmark `(name, target)`s in lexicographical order.
-    pub fn local_bookmarks(&self) -> impl Iterator<Item = (&str, &RefTarget)> {
+    pub fn local_bookmarks(&self) -> impl Iterator<Item = (&RefName, &RefTarget)> {
         self.data
             .local_bookmarks
             .iter()
@@ -138,33 +139,33 @@ impl View {
 
     /// Iterates local bookmarks `(name, target)` in lexicographical order where
     /// the target adds `commit_id`.
-    pub fn local_bookmarks_for_commit<'a: 'b, 'b>(
+    pub fn local_bookmarks_for_commit<'a, 'b>(
         &'a self,
         commit_id: &'b CommitId,
-    ) -> impl Iterator<Item = (&'a str, &'a RefTarget)> + 'b {
+    ) -> impl Iterator<Item = (&'a RefName, &'a RefTarget)> + use<'a, 'b> {
         self.local_bookmarks()
             .filter(|(_, target)| target.added_ids().contains(commit_id))
     }
 
     /// Iterates local bookmark `(name, target)`s matching the given pattern.
     /// Entries are sorted by `name`.
-    pub fn local_bookmarks_matching<'a: 'b, 'b>(
+    pub fn local_bookmarks_matching<'a, 'b>(
         &'a self,
         pattern: &'b StringPattern,
-    ) -> impl Iterator<Item = (&'a str, &'a RefTarget)> + 'b {
+    ) -> impl Iterator<Item = (&'a RefName, &'a RefTarget)> + use<'a, 'b> {
         pattern
-            .filter_btree_map(&self.data.local_bookmarks)
+            .filter_btree_map_as_deref(&self.data.local_bookmarks)
             .map(|(name, target)| (name.as_ref(), target))
     }
 
-    pub fn get_local_bookmark(&self, name: &str) -> &RefTarget {
+    pub fn get_local_bookmark(&self, name: &RefName) -> &RefTarget {
         self.data.local_bookmarks.get(name).flatten()
     }
 
     /// Sets local bookmark to point to the given target. If the target is
     /// absent, and if no associated remote bookmarks exist, the bookmark
     /// will be removed.
-    pub fn set_local_bookmark_target(&mut self, name: &str, target: RefTarget) {
+    pub fn set_local_bookmark_target(&mut self, name: &RefName, target: RefTarget) {
         if target.is_present() {
             self.data.local_bookmarks.insert(name.to_owned(), target);
         } else {
@@ -176,12 +177,14 @@ impl View {
     /// lexicographical order.
     pub fn all_remote_bookmarks(&self) -> impl Iterator<Item = (RemoteRefSymbol<'_>, &RemoteRef)> {
         op_store::flatten_remote_bookmarks(&self.data.remote_views)
-            .map(|((name, remote), remote_ref)| (RemoteRefSymbol { name, remote }, remote_ref))
     }
 
     /// Iterates over `(name, remote_ref)`s for all remote bookmarks of the
     /// specified remote in lexicographical order.
-    pub fn remote_bookmarks(&self, remote_name: &str) -> impl Iterator<Item = (&str, &RemoteRef)> {
+    pub fn remote_bookmarks(
+        &self,
+        remote_name: &RemoteName,
+    ) -> impl Iterator<Item = (&RefName, &RemoteRef)> + use<'_> {
         let maybe_remote_view = self.data.remote_views.get(remote_name);
         maybe_remote_view
             .map(|remote_view| {
@@ -198,21 +201,18 @@ impl View {
     /// specified remote that match the given pattern.
     ///
     /// Entries are sorted by `symbol`, which is `(name, remote)`.
-    pub fn remote_bookmarks_matching<'a: 'b, 'b>(
+    pub fn remote_bookmarks_matching<'a, 'b>(
         &'a self,
         bookmark_pattern: &'b StringPattern,
         remote_pattern: &'b StringPattern,
-    ) -> impl Iterator<Item = (RemoteRefSymbol<'a>, &'a RemoteRef)> + 'b {
+    ) -> impl Iterator<Item = (RemoteRefSymbol<'a>, &'a RemoteRef)> + use<'a, 'b> {
         // Use kmerge instead of flat_map for consistency with all_remote_bookmarks().
         remote_pattern
-            .filter_btree_map(&self.data.remote_views)
+            .filter_btree_map_as_deref(&self.data.remote_views)
             .map(|(remote, remote_view)| {
                 bookmark_pattern
-                    .filter_btree_map(&remote_view.bookmarks)
-                    .map(|(name, remote_ref)| {
-                        let symbol = RemoteRefSymbol { name, remote };
-                        (symbol, remote_ref)
-                    })
+                    .filter_btree_map_as_deref(&remote_view.bookmarks)
+                    .map(|(name, remote_ref)| (name.to_remote_symbol(remote), remote_ref))
             })
             .kmerge_by(|(symbol1, _), (symbol2, _)| symbol1 < symbol2)
     }
@@ -249,10 +249,10 @@ impl View {
     /// Note that this does *not* take into account whether the local bookmark
     /// tracks the remote bookmark or not. Missing values are represented as
     /// RefTarget::absent_ref() or RemoteRef::absent_ref().
-    pub fn local_remote_bookmarks<'a>(
-        &'a self,
-        remote_name: &str,
-    ) -> impl Iterator<Item = (&'a str, LocalAndRemoteRef<'a>)> + 'a {
+    pub fn local_remote_bookmarks(
+        &self,
+        remote_name: &RemoteName,
+    ) -> impl Iterator<Item = (&RefName, LocalAndRemoteRef<'_>)> + use<'_> {
         refs::iter_named_local_remote_refs(
             self.local_bookmarks(),
             self.remote_bookmarks(remote_name),
@@ -275,18 +275,20 @@ impl View {
     /// Note that this does *not* take into account whether the local bookmark
     /// tracks the remote bookmark or not. Missing values are represented as
     /// RefTarget::absent_ref() or RemoteRef::absent_ref().
-    pub fn local_remote_bookmarks_matching<'a: 'b, 'b>(
+    pub fn local_remote_bookmarks_matching<'a, 'b>(
         &'a self,
         bookmark_pattern: &'b StringPattern,
-        remote_name: &str,
-    ) -> impl Iterator<Item = (&'a str, LocalAndRemoteRef<'a>)> + 'b {
+        remote_name: &RemoteName,
+    ) -> impl Iterator<Item = (&'a RefName, LocalAndRemoteRef<'a>)> + use<'a, 'b> {
         // Change remote_name to StringPattern if needed, but merge-join adapter won't
         // be usable.
         let maybe_remote_view = self.data.remote_views.get(remote_name);
         refs::iter_named_local_remote_refs(
-            bookmark_pattern.filter_btree_map(&self.data.local_bookmarks),
+            bookmark_pattern.filter_btree_map_as_deref(&self.data.local_bookmarks),
             maybe_remote_view
-                .map(|remote_view| bookmark_pattern.filter_btree_map(&remote_view.bookmarks))
+                .map(|remote_view| {
+                    bookmark_pattern.filter_btree_map_as_deref(&remote_view.bookmarks)
+                })
                 .into_iter()
                 .flatten(),
         )
@@ -299,34 +301,34 @@ impl View {
         })
     }
 
-    pub fn remove_remote(&mut self, remote_name: &str) {
+    pub fn remove_remote(&mut self, remote_name: &RemoteName) {
         self.data.remote_views.remove(remote_name);
     }
 
-    pub fn rename_remote(&mut self, old: &str, new: &str) {
+    pub fn rename_remote(&mut self, old: &RemoteName, new: &RemoteName) {
         if let Some(remote_view) = self.data.remote_views.remove(old) {
             self.data.remote_views.insert(new.to_owned(), remote_view);
         }
     }
 
-    pub fn get_tag(&self, name: &str) -> &RefTarget {
+    pub fn get_tag(&self, name: &RefName) -> &RefTarget {
         self.data.tags.get(name).flatten()
     }
 
     /// Iterates tags `(name, target)`s matching the given pattern. Entries
     /// are sorted by `name`.
-    pub fn tags_matching<'a: 'b, 'b>(
+    pub fn tags_matching<'a, 'b>(
         &'a self,
         pattern: &'b StringPattern,
-    ) -> impl Iterator<Item = (&'a str, &'a RefTarget)> + 'b {
+    ) -> impl Iterator<Item = (&'a RefName, &'a RefTarget)> + use<'a, 'b> {
         pattern
-            .filter_btree_map(&self.data.tags)
+            .filter_btree_map_as_deref(&self.data.tags)
             .map(|(name, target)| (name.as_ref(), target))
     }
 
     /// Sets tag to point to the given target. If the target is absent, the tag
     /// will be removed.
-    pub fn set_tag_target(&mut self, name: &str, target: RefTarget) {
+    pub fn set_tag_target(&mut self, name: &RefName, target: RefTarget) {
         if target.is_present() {
             self.data.tags.insert(name.to_owned(), target);
         } else {
@@ -334,13 +336,13 @@ impl View {
         }
     }
 
-    pub fn get_git_ref(&self, name: &str) -> &RefTarget {
+    pub fn get_git_ref(&self, name: &GitRefName) -> &RefTarget {
         self.data.git_refs.get(name).flatten()
     }
 
     /// Sets the last imported Git ref to point to the given target. If the
     /// target is absent, the reference will be removed.
-    pub fn set_git_ref_target(&mut self, name: &str, target: RefTarget) {
+    pub fn set_git_ref_target(&mut self, name: &GitRefName, target: RefTarget) {
         if target.is_present() {
             self.data.git_refs.insert(name.to_owned(), target);
         } else {
@@ -357,9 +359,9 @@ impl View {
     /// Iterates all commit ids referenced by this view.
     ///
     /// This can include hidden commits referenced by remote bookmarks, previous
-    /// positions of conflicted bookmarks, etc. The ancestors and predecessors
-    /// of the returned commits should be considered reachable from the
-    /// view. Use this to build commit index from scratch.
+    /// positions of conflicted bookmarks, etc. The ancestors of the returned
+    /// commits should be considered reachable from the view. Use this to build
+    /// commit index from scratch.
     ///
     /// The iteration order is unspecified, and may include duplicated entries.
     pub fn all_referenced_commit_ids(&self) -> impl Iterator<Item = &CommitId> {
@@ -412,9 +414,9 @@ impl View {
 /// Error from attempts to rename a workspace
 #[derive(Debug, Error)]
 pub enum RenameWorkspaceError {
-    #[error("Workspace {workspace_id} not found")]
-    WorkspaceDoesNotExist { workspace_id: String },
+    #[error("Workspace {} not found", name.as_symbol())]
+    WorkspaceDoesNotExist { name: WorkspaceNameBuf },
 
-    #[error("Workspace {workspace_id} already exists")]
-    WorkspaceAlreadyExists { workspace_id: String },
+    #[error("Workspace {} already exists", name.as_symbol())]
+    WorkspaceAlreadyExists { name: WorkspaceNameBuf },
 }
