@@ -28,7 +28,6 @@ use tracing::instrument;
 use crate::backend::BackendError;
 use crate::backend::MergedTreeId;
 use crate::commit::Commit;
-use crate::conflicts::ConflictMarkerStyle;
 use crate::dag_walk;
 use crate::fsmonitor::FsmonitorSettings;
 use crate::gitignore::GitIgnoreError;
@@ -123,11 +122,7 @@ pub trait LockedWorkingCopy {
     ) -> Result<(MergedTreeId, SnapshotStats), SnapshotError>;
 
     /// Check out the specified commit in the working copy.
-    fn check_out(
-        &mut self,
-        commit: &Commit,
-        options: &CheckoutOptions,
-    ) -> Result<CheckoutStats, CheckoutError>;
+    fn check_out(&mut self, commit: &Commit) -> Result<CheckoutStats, CheckoutError>;
 
     /// Update the workspace name.
     fn rename_workspace(&mut self, new_workspace_name: WorkspaceNameBuf);
@@ -151,7 +146,6 @@ pub trait LockedWorkingCopy {
     fn set_sparse_patterns(
         &mut self,
         new_sparse_patterns: Vec<RepoPathBuf>,
-        options: &CheckoutOptions,
     ) -> Result<CheckoutStats, CheckoutError>;
 
     /// Finish the modifications to the working copy by writing the updated
@@ -187,6 +181,9 @@ pub enum SnapshotError {
     /// Checking path with ignore patterns failed.
     #[error(transparent)]
     GitIgnoreError(#[from] GitIgnoreError),
+    /// Failed to load the working copy state.
+    #[error(transparent)]
+    WorkingCopyStateError(#[from] WorkingCopyStateError),
     /// Some other error happened while snapshotting the working copy.
     #[error("{message}")]
     Other {
@@ -223,8 +220,6 @@ pub struct SnapshotOptions<'a> {
     /// (depending on implementation)
     /// return `SnapshotError::NewFileTooLarge`.
     pub max_new_file_size: u64,
-    /// Expected conflict marker style for checking for changed files.
-    pub conflict_marker_style: ConflictMarkerStyle,
 }
 
 impl SnapshotOptions<'_> {
@@ -236,7 +231,6 @@ impl SnapshotOptions<'_> {
             progress: None,
             start_tracking_matcher: &EverythingMatcher,
             max_new_file_size: u64::MAX,
-            conflict_marker_style: ConflictMarkerStyle::default(),
         }
     }
 }
@@ -265,22 +259,6 @@ pub enum UntrackedReason {
     FileNotAutoTracked,
 }
 
-/// Options used when checking out a tree in the working copy.
-#[derive(Clone)]
-pub struct CheckoutOptions {
-    /// Conflict marker style to use when materializing files
-    pub conflict_marker_style: ConflictMarkerStyle,
-}
-
-impl CheckoutOptions {
-    /// Create an instance for use in tests.
-    pub fn empty_for_test() -> Self {
-        Self {
-            conflict_marker_style: ConflictMarkerStyle::default(),
-        }
-    }
-}
-
 /// Stats about a checkout operation on a working copy. All "files" mentioned
 /// below may also be symlinks or materialized conflicts.
 #[derive(Debug, PartialEq, Eq, Clone, Default)]
@@ -326,6 +304,9 @@ pub enum CheckoutError {
     /// Reading or writing from the commit backend failed.
     #[error("Internal backend error")]
     InternalBackendError(#[from] BackendError),
+    /// Failed to load the working copy state.
+    #[error(transparent)]
+    WorkingCopyStateError(#[from] WorkingCopyStateError),
     /// Some other error happened while checking out the working copy.
     #[error("{message}")]
     Other {
@@ -350,7 +331,10 @@ pub enum ResetError {
     /// Reading or writing from the commit backend failed.
     #[error("Internal error")]
     InternalBackendError(#[from] BackendError),
-    /// Some other error happened while checking out the working copy.
+    /// Failed to load the working copy state.
+    #[error(transparent)]
+    WorkingCopyStateError(#[from] WorkingCopyStateError),
+    /// Some other error happened while resetting the working copy.
     #[error("{message}")]
     Other {
         /// Error message.
