@@ -55,7 +55,6 @@ use crate::signing::Signer;
 use crate::simple_backend::SimpleBackend;
 use crate::transaction::TransactionCommitError;
 use crate::working_copy::CheckoutError;
-use crate::working_copy::CheckoutOptions;
 use crate::working_copy::CheckoutStats;
 use crate::working_copy::LockedWorkingCopy;
 use crate::working_copy::WorkingCopy;
@@ -119,7 +118,7 @@ fn create_jj_dir(workspace_root: &Path) -> Result<PathBuf, WorkspaceInitError> {
     let jj_dir = workspace_root.join(".jj");
     match std::fs::create_dir(&jj_dir).context(&jj_dir) {
         Ok(()) => Ok(jj_dir),
-        Err(ref e) if e.error.kind() == io::ErrorKind::AlreadyExists => {
+        Err(ref e) if e.source.kind() == io::ErrorKind::AlreadyExists => {
             Err(WorkspaceInitError::DestinationExists(jj_dir))
         }
         Err(e) => Err(e.into()),
@@ -430,14 +429,8 @@ impl Workspace {
         operation_id: OperationId,
         old_tree_id: Option<&MergedTreeId>,
         commit: &Commit,
-        options: &CheckoutOptions,
     ) -> Result<CheckoutStats, CheckoutError> {
-        let mut locked_ws =
-            self.start_working_copy_mutation()
-                .map_err(|err| CheckoutError::Other {
-                    message: "Failed to start editing the working copy state".to_string(),
-                    err: err.into(),
-                })?;
+        let mut locked_ws = self.start_working_copy_mutation()?;
         // Check if the current working-copy commit has changed on disk compared to what
         // the caller expected. It's safe to check out another commit
         // regardless, but it's probably not what  the caller wanted, so we let
@@ -447,7 +440,7 @@ impl Workspace {
                 return Err(CheckoutError::ConcurrentCheckout);
             }
         }
-        let stats = locked_ws.locked_wc().check_out(commit, options)?;
+        let stats = locked_ws.locked_wc().check_out(commit)?;
         locked_ws
             .finish(operation_id)
             .map_err(|err| CheckoutError::Other {
@@ -492,7 +485,7 @@ pub fn get_working_copy_factory<'a>(
     } else {
         Err(StoreLoadError::UnsupportedType {
             store: "working copy",
-            store_type: working_copy_type.to_string(),
+            store_type: working_copy_type.clone(),
         })
     }
 }
