@@ -18,17 +18,31 @@ use jj_lib::backend::CommitId;
 use jj_lib::backend::MillisSinceEpoch;
 use jj_lib::backend::Signature;
 use jj_lib::backend::Timestamp;
+use jj_lib::config::ConfigLayer;
+use jj_lib::config::ConfigSource;
 use jj_lib::id_prefix::IdPrefixContext;
 use jj_lib::object_id::HexPrefix;
 use jj_lib::object_id::ObjectId as _;
 use jj_lib::object_id::PrefixResolution::AmbiguousMatch;
 use jj_lib::object_id::PrefixResolution::NoMatch;
 use jj_lib::object_id::PrefixResolution::SingleMatch;
+use jj_lib::op_store::RefTarget;
 use jj_lib::repo::Repo as _;
 use jj_lib::revset::RevsetExpression;
+use jj_lib::settings::UserSettings;
 use testutils::TestRepo;
 use testutils::TestRepoBackend;
 
+fn stable_settings() -> UserSettings {
+    let mut config = testutils::base_user_config();
+    let mut layer = ConfigLayer::empty(ConfigSource::User);
+    layer
+        .set_value("debug.commit-timestamp", "2001-02-03T04:05:06+07:00")
+        .unwrap();
+    config.add_layer(layer);
+    UserSettings::from_config(config).unwrap()
+}
+
 #[test]
 fn test_id_prefix() {
     let test_repo = TestRepo::init_with_backend(TestRepoBackend::Git);
@@ -67,32 +81,32 @@ fn test_id_prefix() {
         .sorted()
         .join("\n");
     insta::assert_snapshot!(commit_prefixes, @r"
-    11a 5
-    214 24
-    2a6 2
-    33e 14
-    3be 16
-    3ea 18
-    593 20
-    5d3 1
-    5f6 13
-    676 3
-    7b6 25
-    7da 9
-    81c 10
-    87e 12
-    997 21
-    9f7 22
-    a0e 4
-    a55 19
-    ac4 23
-    c18 17
-    ce9 0
-    d42 6
-    d9d 8
-    eec 15
-    efe 7
-    fa3 11
+    0c8 9
+    18f 7
+    19a 10
+    37a 13
+    3b4 21
+    3c0 1
+    4ee 16
+    51f 4
+    56e 14
+    711 17
+    761 3
+    7b1 11
+    7c6 24
+    7f4 8
+    846 23
+    8d7 25
+    960 15
+    a30 12
+    b51 19
+    b97 22
+    b9d 5
+    bb4 2
+    c3a 18
+    c47 0
+    d3c 6
+    d54 20
     ");
     let change_prefixes = commits
         .iter()
@@ -129,38 +143,38 @@ fn test_id_prefix() {
     fee 25
     ");
 
-    let prefix = |x| HexPrefix::new(x).unwrap();
+    let prefix = |x| HexPrefix::try_from_hex(x).unwrap();
 
     // Without a disambiguation revset
     // ---------------------------------------------------------------------------------------------
     let context = IdPrefixContext::default();
     let index = context.populate(repo.as_ref()).unwrap();
     assert_eq!(
-        index.shortest_commit_prefix_len(repo.as_ref(), commits[2].id()),
+        index.shortest_commit_prefix_len(repo.as_ref(), commits[7].id()),
         2
     );
     assert_eq!(
-        index.shortest_commit_prefix_len(repo.as_ref(), commits[5].id()),
+        index.shortest_commit_prefix_len(repo.as_ref(), commits[16].id()),
         1
     );
     assert_eq!(
-        index.resolve_commit_prefix(repo.as_ref(), &prefix("2")),
+        index.resolve_commit_prefix(repo.as_ref(), &prefix("1")),
         AmbiguousMatch
     );
     assert_eq!(
-        index.resolve_commit_prefix(repo.as_ref(), &prefix("2a")),
-        SingleMatch(commits[2].id().clone())
+        index.resolve_commit_prefix(repo.as_ref(), &prefix("18")),
+        SingleMatch(commits[7].id().clone())
     );
     assert_eq!(
-        index.resolve_commit_prefix(repo.as_ref(), &prefix("20")),
+        index.resolve_commit_prefix(repo.as_ref(), &prefix("10")),
         NoMatch
     );
     assert_eq!(
-        index.resolve_commit_prefix(repo.as_ref(), &prefix("2a0")),
+        index.resolve_commit_prefix(repo.as_ref(), &prefix("180")),
         NoMatch
     );
     assert_eq!(
-        index.shortest_change_prefix_len(repo.as_ref(), commits[0].change_id()),
+        index.shortest_change_prefix_len(repo.as_ref(), commits[2].change_id()),
         2
     );
     assert_eq!(
@@ -168,50 +182,50 @@ fn test_id_prefix() {
         1
     );
     assert_eq!(
-        index.resolve_change_prefix(repo.as_ref(), &prefix("7")),
+        index.resolve_change_prefix(repo.as_ref(), &prefix("4")),
         AmbiguousMatch
     );
     assert_eq!(
-        index.resolve_change_prefix(repo.as_ref(), &prefix("78")),
-        SingleMatch(vec![commits[0].id().clone()])
+        index.resolve_change_prefix(repo.as_ref(), &prefix("43")),
+        SingleMatch(vec![commits[2].id().clone()])
     );
     assert_eq!(
-        index.resolve_change_prefix(repo.as_ref(), &prefix("70")),
+        index.resolve_change_prefix(repo.as_ref(), &prefix("40")),
         NoMatch
     );
     assert_eq!(
-        index.resolve_change_prefix(repo.as_ref(), &prefix("780")),
+        index.resolve_change_prefix(repo.as_ref(), &prefix("430")),
         NoMatch
     );
 
     // Disambiguate within a revset
     // ---------------------------------------------------------------------------------------------
     let expression =
-        RevsetExpression::commits(vec![commits[0].id().clone(), commits[2].id().clone()]);
+        RevsetExpression::commits(vec![commits[7].id().clone(), commits[2].id().clone()]);
     let context = context.disambiguate_within(expression);
     let index = context.populate(repo.as_ref()).unwrap();
     // The prefix is now shorter
     assert_eq!(
-        index.shortest_commit_prefix_len(repo.as_ref(), commits[2].id()),
+        index.shortest_commit_prefix_len(repo.as_ref(), commits[7].id()),
         1
     );
     // Shorter prefix within the set can be used
     assert_eq!(
-        index.resolve_commit_prefix(repo.as_ref(), &prefix("2")),
-        SingleMatch(commits[2].id().clone())
+        index.resolve_commit_prefix(repo.as_ref(), &prefix("1")),
+        SingleMatch(commits[7].id().clone())
     );
     // Can still resolve commits outside the set
     assert_eq!(
-        index.resolve_commit_prefix(repo.as_ref(), &prefix("21")),
-        SingleMatch(commits[24].id().clone())
+        index.resolve_commit_prefix(repo.as_ref(), &prefix("19")),
+        SingleMatch(commits[10].id().clone())
     );
     assert_eq!(
-        index.shortest_change_prefix_len(repo.as_ref(), commits[0].change_id()),
+        index.shortest_change_prefix_len(repo.as_ref(), commits[2].change_id()),
         1
     );
     assert_eq!(
-        index.resolve_change_prefix(repo.as_ref(), &prefix("7")),
-        SingleMatch(vec![commits[0].id().clone()])
+        index.resolve_change_prefix(repo.as_ref(), &prefix("4")),
+        SingleMatch(vec![commits[2].id().clone()])
     );
 
     // Single commit in revset. Length 0 is unambiguous, but we pretend 1 digit is
@@ -311,12 +325,12 @@ fn test_id_prefix_divergent() {
         .map(|(i, commit)| format!("{} {}", &commit.id().hex()[..4], i))
         .join("\n");
     insta::assert_snapshot!(commit_prefixes, @r"
-    eafa 0
-    d48d 1
-    2fbb 2
+    e2b9 0
+    f8d1 1
+    c596 2
     ");
 
-    let prefix = |x| HexPrefix::new(x).unwrap();
+    let prefix = |x| HexPrefix::try_from_hex(x).unwrap();
 
     // Without a disambiguation revset
     // --------------------------------
@@ -424,16 +438,16 @@ fn test_id_prefix_hidden() {
         .sorted()
         .join("\n");
     insta::assert_snapshot!(commit_prefixes, @r"
-    15e 9
-    397 6
-    53c 7
-    62e 2
-    648 8
-    7c7 3
-    853 4
-    c0a 5
-    ce9 0
-    f10 1
+    3ae 6
+    64c 5
+    84e 2
+    906 8
+    912 7
+    9d1 3
+    a6b 1
+    c47 0
+    d9b 4
+    f5f 9
     ");
     let change_prefixes = commits
         .iter()
@@ -459,7 +473,7 @@ fn test_id_prefix_hidden() {
     tx.repo_mut().rebase_descendants().unwrap();
     let repo = tx.commit("test").unwrap();
 
-    let prefix = |x: &str| HexPrefix::new(x).unwrap();
+    let prefix = |x: &str| HexPrefix::try_from_hex(x).unwrap();
 
     // Without a disambiguation revset
     // --------------------------------
@@ -532,3 +546,90 @@ fn test_id_prefix_hidden() {
         NoMatch
     );
 }
+
+#[test]
+fn test_id_prefix_shadowed_by_ref() {
+    let settings = stable_settings();
+    let test_repo = TestRepo::init_with_settings(&settings);
+    let repo = &test_repo.repo;
+    let root_commit_id = repo.store().root_commit_id();
+
+    let mut tx = repo.start_transaction();
+    let commit = tx
+        .repo_mut()
+        .new_commit(
+            vec![root_commit_id.clone()],
+            repo.store().empty_merged_tree_id(),
+        )
+        .write()
+        .unwrap();
+
+    let commit_id_sym = commit.id().to_string();
+    let change_id_sym = commit.change_id().to_string();
+    insta::assert_snapshot!(commit_id_sym, @"ccde67f2a6ece4661cf4");
+    insta::assert_snapshot!(change_id_sym, @"sryyqqkqmuumyrlruupspprvnulvovzm");
+
+    let context = IdPrefixContext::default();
+    let index = context.populate(tx.repo()).unwrap();
+
+    assert_eq!(index.shortest_commit_prefix_len(tx.repo(), commit.id()), 1);
+    assert_eq!(
+        index.shortest_change_prefix_len(tx.repo(), commit.change_id()),
+        1
+    );
+
+    // Longer symbol doesn't count
+    let dummy_target = RefTarget::normal(root_commit_id.clone());
+    tx.repo_mut()
+        .set_tag_target(commit_id_sym[..2].as_ref(), dummy_target.clone());
+    tx.repo_mut()
+        .set_tag_target(change_id_sym[..2].as_ref(), dummy_target.clone());
+    assert_eq!(index.shortest_commit_prefix_len(tx.repo(), commit.id()), 1);
+    assert_eq!(
+        index.shortest_change_prefix_len(tx.repo(), commit.change_id()),
+        1
+    );
+
+    // 1-char conflict with bookmark, 2-char with tag
+    tx.repo_mut()
+        .set_local_bookmark_target(commit_id_sym[..1].as_ref(), dummy_target.clone());
+    tx.repo_mut()
+        .set_local_bookmark_target(change_id_sym[..1].as_ref(), dummy_target.clone());
+    assert_eq!(index.shortest_commit_prefix_len(tx.repo(), commit.id()), 3);
+    assert_eq!(
+        index.shortest_change_prefix_len(tx.repo(), commit.change_id()),
+        3
+    );
+
+    // Many-char conflicts
+    for n in 3..commit_id_sym.len() {
+        tx.repo_mut()
+            .set_tag_target(commit_id_sym[..n].as_ref(), dummy_target.clone());
+    }
+    for n in 3..change_id_sym.len() {
+        tx.repo_mut()
+            .set_tag_target(change_id_sym[..n].as_ref(), dummy_target.clone());
+    }
+    assert_eq!(
+        index.shortest_commit_prefix_len(tx.repo(), commit.id()),
+        commit_id_sym.len()
+    );
+    assert_eq!(
+        index.shortest_change_prefix_len(tx.repo(), commit.change_id()),
+        change_id_sym.len()
+    );
+
+    // Full-char conflicts
+    tx.repo_mut()
+        .set_tag_target(commit_id_sym.as_ref(), dummy_target.clone());
+    tx.repo_mut()
+        .set_tag_target(change_id_sym.as_ref(), dummy_target.clone());
+    assert_eq!(
+        index.shortest_commit_prefix_len(tx.repo(), commit.id()),
+        commit_id_sym.len()
+    );
+    assert_eq!(
+        index.shortest_change_prefix_len(tx.repo(), commit.change_id()),
+        change_id_sym.len()
+    );
+}
