@@ -17,6 +17,7 @@ use std::fs;
 use std::sync::Arc;
 
 use assert_matches::assert_matches;
+use itertools::Itertools as _;
 use jj_lib::backend::ChangeId;
 use jj_lib::backend::CommitId;
 use jj_lib::commit::Commit;
@@ -36,12 +37,18 @@ use jj_lib::ref_name::RemoteRefSymbol;
 use jj_lib::repo::MutableRepo;
 use jj_lib::repo::ReadonlyRepo;
 use jj_lib::repo::Repo as _;
+use jj_lib::repo_path::RepoPathBuf;
 use jj_lib::revset::GENERATION_RANGE_FULL;
 use jj_lib::revset::PARENTS_RANGE_FULL;
 use jj_lib::revset::ResolvedExpression;
 use maplit::hashset;
+use pollster::FutureExt as _;
+use test_case::test_case;
 use testutils::TestRepo;
 use testutils::commit_transactions;
+use testutils::create_tree;
+use testutils::repo_path;
+use testutils::repo_path_buf;
 use testutils::test_backend::TestBackend;
 use testutils::write_random_commit;
 use testutils::write_random_commit_with_parents;
@@ -57,6 +64,23 @@ where
     }
 }
 
+fn enable_changed_path_index(repo: &ReadonlyRepo) -> Arc<ReadonlyRepo> {
+    let default_index_store: &DefaultIndexStore =
+        repo.index_store().as_any().downcast_ref().unwrap();
+    default_index_store
+        .build_changed_path_index_at_operation(repo.op_id(), repo.store(), 0)
+        .block_on()
+        .unwrap();
+    repo.reload_at(repo.operation()).unwrap()
+}
+
+fn collect_changed_paths(repo: &ReadonlyRepo, commit_id: &CommitId) -> Option<Vec<RepoPathBuf>> {
+    repo.index()
+        .changed_paths_in_commit(commit_id)
+        .unwrap()
+        .map(|paths| paths.collect())
+}
+
 #[test]
 fn test_index_commits_empty_repo() {
     let test_repo = TestRepo::init();
@@ -695,9 +719,11 @@ fn test_reindex_from_merged_operation() {
     let index = as_readonly_index(&repo);
     assert_eq!(index.num_commits(), 4);
 
-    let index_operations_dir = test_repo.repo_path().join("index").join("operations");
+    let op_links_dir = test_repo.repo_path().join("index").join("op_links");
+    let legacy_operations_dir = test_repo.repo_path().join("index").join("operations");
     for &op_id in &op_ids_to_delete {
-        fs::remove_file(index_operations_dir.join(op_id.hex())).unwrap();
+        fs::remove_file(op_links_dir.join(op_id.hex())).unwrap();
+        fs::remove_file(legacy_operations_dir.join(op_id.hex())).unwrap();
     }
 
     // When re-indexing, one of the merge parent operations will be selected as
@@ -737,6 +763,7 @@ fn test_reindex_missing_commit() {
     default_index_store.reinit().unwrap();
     let err = default_index_store
         .build_index_at_operation(repo.operation(), repo.store())
+        .block_on()
         .unwrap_err();
     assert_matches!(err, DefaultIndexStoreError::IndexCommits { op_id, .. } if op_id == *bad_op_id);
 }
@@ -755,6 +782,383 @@ fn test_index_store_type() {
     );
 }
 
+#[test]
+fn test_read_legacy_operation_link_file() {
+    let test_repo = TestRepo::init();
+    let repo = &test_repo.repo;
+
+    // Delete new operation link files and directory
+    let op_links_dir = test_repo.repo_path().join("index").join("op_links");
+    fs::remove_dir_all(&op_links_dir).unwrap();
+
+    // Reload repo and index
+    let repo = repo.reload_at(repo.operation()).unwrap();
+    let _ = repo.readonly_index();
+    // Existing index should still be readable, so new operation link file won't
+    // be created
+    assert!(!op_links_dir.join(repo.op_id().hex()).exists());
+
+    // New operation link file and directory can be created
+    let mut tx = repo.start_transaction();
+    write_random_commit(tx.repo_mut());
+    let repo = tx.commit("test").unwrap();
+    assert!(op_links_dir.join(repo.op_id().hex()).exists());
+}
+
+#[test]
+fn test_changed_path_segments() {
+    let test_repo = TestRepo::init();
+    let repo = &test_repo.repo;
+    let root_commit_id = repo.store().root_commit_id();
+
+    // Changed-path index should be disabled by default
+    let segments_dir = test_repo.repo_path().join("index").join("changed_paths");
+    let count_segment_files = || {
+        let entries = segments_dir.read_dir().unwrap();
+        entries.process_results(|entries| entries.count()).unwrap()
+    };
+    assert_eq!(count_segment_files(), 0);
+    let stats = as_readonly_index(repo).stats();
+    assert_eq!(stats.changed_path_commits_range, None);
+    assert_eq!(stats.changed_path_levels.len(), 0);
+
+    let repo = enable_changed_path_index(repo);
+    let stats = as_readonly_index(&repo).stats();
+    assert_eq!(stats.changed_path_commits_range, Some(1..1));
+    assert_eq!(stats.changed_path_levels.len(), 0);
+
+    let tree1 = create_tree(&repo, &[(repo_path("a"), "")]);
+    let tree2 = create_tree(&repo, &[(repo_path("a"), ""), (repo_path("b"), "")]);
+
+    // Add new commit with changed-path index enabled
+    let mut tx = repo.start_transaction();
+    let commit1 = tx
+        .repo_mut()
+        .new_commit(vec![root_commit_id.clone()], tree1.id())
+        .write()
+        .unwrap();
+    let repo = tx.commit("test").unwrap();
+    let stats = as_readonly_index(&repo).stats();
+    assert_eq!(count_segment_files(), 1);
+    assert_eq!(stats.changed_path_commits_range, Some(1..2));
+    assert_eq!(stats.changed_path_levels.len(), 1);
+    assert_eq!(stats.changed_path_levels[0].num_commits, 1);
+    assert_eq!(stats.changed_path_levels[0].num_changed_paths, 1);
+    assert_eq!(stats.changed_path_levels[0].num_paths, 1);
+    assert_eq!(collect_changed_paths(&repo, root_commit_id), None);
+    assert_eq!(
+        collect_changed_paths(&repo, commit1.id()),
+        Some(vec![repo_path_buf("a")])
+    );
+
+    // Add one more commit, segment files should be squashed
+    let mut tx = repo.start_transaction();
+    let commit2 = tx
+        .repo_mut()
+        .new_commit(vec![root_commit_id.clone()], tree2.id())
+        .write()
+        .unwrap();
+    let repo = tx.commit("test").unwrap();
+    let stats = as_readonly_index(&repo).stats();
+    assert_eq!(count_segment_files(), 2);
+    assert_eq!(stats.changed_path_commits_range, Some(1..3));
+    assert_eq!(stats.changed_path_levels.len(), 1);
+    assert_eq!(stats.changed_path_levels[0].num_commits, 2);
+    assert_eq!(stats.changed_path_levels[0].num_changed_paths, 3);
+    assert_eq!(stats.changed_path_levels[0].num_paths, 2);
+    assert_eq!(collect_changed_paths(&repo, root_commit_id), None);
+    assert_eq!(
+        collect_changed_paths(&repo, commit1.id()),
+        Some(vec![repo_path_buf("a")])
+    );
+    assert_eq!(
+        collect_changed_paths(&repo, commit2.id()),
+        Some(vec![repo_path_buf("a"), repo_path_buf("b")])
+    );
+}
+
+#[test]
+fn test_build_changed_path_segments() {
+    let test_repo = TestRepo::init();
+    let repo = test_repo.repo;
+    let root_commit_id = repo.store().root_commit_id();
+    let default_index_store: &DefaultIndexStore =
+        repo.index_store().as_any().downcast_ref().unwrap();
+
+    let mut tx = repo.start_transaction();
+    for i in 1..10 {
+        let tree = create_tree(&repo, &[(repo_path(&i.to_string()), "")]);
+        tx.repo_mut()
+            .new_commit(vec![root_commit_id.clone()], tree.id())
+            .write()
+            .unwrap();
+    }
+    let repo = tx.commit("test").unwrap();
+
+    // Index the last 4 commits
+    default_index_store
+        .build_changed_path_index_at_operation(repo.op_id(), repo.store(), 4)
+        .block_on()
+        .unwrap();
+    let repo = repo.reload_at(repo.operation()).unwrap();
+    let stats = as_readonly_index(&repo).stats();
+    assert_eq!(stats.changed_path_commits_range, Some(6..10));
+    assert_eq!(stats.changed_path_levels.len(), 1);
+    assert_eq!(stats.changed_path_levels[0].num_commits, 4);
+    assert_eq!(stats.changed_path_levels[0].num_changed_paths, 4);
+    assert_eq!(stats.changed_path_levels[0].num_paths, 4);
+
+    // Index remainders
+    default_index_store
+        .build_changed_path_index_at_operation(repo.op_id(), repo.store(), u32::MAX)
+        .block_on()
+        .unwrap();
+    let repo = repo.reload_at(repo.operation()).unwrap();
+    let stats = as_readonly_index(&repo).stats();
+    assert_eq!(stats.changed_path_commits_range, Some(0..10));
+    assert_eq!(stats.changed_path_levels.len(), 2);
+    assert_eq!(stats.changed_path_levels[0].num_commits, 6);
+    assert_eq!(stats.changed_path_levels[0].num_changed_paths, 5);
+    assert_eq!(stats.changed_path_levels[0].num_paths, 5);
+    assert_eq!(stats.changed_path_levels[1].num_commits, 4);
+    assert_eq!(stats.changed_path_levels[1].num_changed_paths, 4);
+    assert_eq!(stats.changed_path_levels[1].num_paths, 4);
+}
+
+#[test]
+fn test_build_changed_path_segments_partially_enabled() {
+    let test_repo = TestRepo::init();
+    let repo = test_repo.repo;
+    let root_commit_id = repo.store().root_commit_id();
+    let default_index_store: &DefaultIndexStore =
+        repo.index_store().as_any().downcast_ref().unwrap();
+
+    // Partially enable index by merging two operations
+    let tx1 = {
+        let mut tx = repo.start_transaction();
+        for i in 1..5 {
+            let tree = create_tree(&repo, &[(repo_path(&i.to_string()), "")]);
+            tx.repo_mut()
+                .new_commit(vec![root_commit_id.clone()], tree.id())
+                .write()
+                .unwrap();
+        }
+        let repo = tx.commit("test").unwrap();
+        let repo = enable_changed_path_index(&repo);
+        let mut tx = repo.start_transaction();
+        let tree = create_tree(&repo, &[(repo_path("5"), "")]);
+        tx.repo_mut()
+            .new_commit(vec![root_commit_id.clone()], tree.id())
+            .write()
+            .unwrap();
+        tx
+    };
+    let mut tx2 = repo.start_transaction();
+    for i in 6..10 {
+        let tree = create_tree(&repo, &[(repo_path(&i.to_string()), "")]);
+        tx2.repo_mut()
+            .new_commit(vec![root_commit_id.clone()], tree.id())
+            .write()
+            .unwrap();
+    }
+    let repo = commit_transactions(vec![tx1, tx2]);
+    let stats = as_readonly_index(&repo).stats();
+    assert_eq!(stats.num_commits, 10);
+    assert_eq!(stats.changed_path_commits_range, Some(5..6));
+    assert_eq!(stats.changed_path_levels.len(), 1);
+    assert_eq!(stats.changed_path_levels[0].num_commits, 1);
+    assert_eq!(stats.changed_path_levels[0].num_changed_paths, 1);
+    assert_eq!(stats.changed_path_levels[0].num_paths, 1);
+
+    // Index later commits from the mid point
+    default_index_store
+        .build_changed_path_index_at_operation(repo.op_id(), repo.store(), 2)
+        .block_on()
+        .unwrap();
+    let repo = repo.reload_at(repo.operation()).unwrap();
+    let stats = as_readonly_index(&repo).stats();
+    assert_eq!(stats.changed_path_commits_range, Some(5..8));
+    assert_eq!(stats.changed_path_levels.len(), 1);
+    assert_eq!(stats.changed_path_levels[0].num_commits, 3);
+    assert_eq!(stats.changed_path_levels[0].num_changed_paths, 3);
+    assert_eq!(stats.changed_path_levels[0].num_paths, 3);
+
+    // Index later and earlier commits from the mid point
+    default_index_store
+        .build_changed_path_index_at_operation(repo.op_id(), repo.store(), 3)
+        .block_on()
+        .unwrap();
+    let repo = repo.reload_at(repo.operation()).unwrap();
+    let stats = as_readonly_index(&repo).stats();
+    assert_eq!(stats.changed_path_commits_range, Some(4..10));
+    assert_eq!(stats.changed_path_levels.len(), 1);
+    assert_eq!(stats.changed_path_levels[0].num_commits, 6);
+    assert_eq!(stats.changed_path_levels[0].num_changed_paths, 6);
+    assert_eq!(stats.changed_path_levels[0].num_paths, 6);
+}
+
+#[test]
+fn test_merge_changed_path_segments_both_enabled() {
+    let test_repo = TestRepo::init();
+    let repo = enable_changed_path_index(&test_repo.repo);
+    let root_commit_id = repo.store().root_commit_id();
+
+    let tree1 = create_tree(&repo, &[(repo_path("a"), "")]);
+    let tree2 = create_tree(&repo, &[(repo_path("a"), ""), (repo_path("b"), "")]);
+    let tree3 = create_tree(&repo, &[(repo_path("c"), ""), (repo_path("d"), "")]);
+
+    // Add index segment that will be squashed
+    let mut tx = repo.start_transaction();
+    tx.repo_mut()
+        .new_commit(vec![root_commit_id.clone()], tree1.id())
+        .write()
+        .unwrap();
+    let repo = tx.commit("test").unwrap();
+
+    // Merge concurrent index segments without the common base segment
+    let mut tx1 = repo.start_transaction();
+    tx1.repo_mut()
+        .new_commit(vec![root_commit_id.clone()], tree2.id())
+        .write()
+        .unwrap();
+    let mut tx2 = repo.start_transaction();
+    tx2.repo_mut()
+        .new_commit(vec![root_commit_id.clone()], tree3.id())
+        .write()
+        .unwrap();
+    let repo = commit_transactions(vec![tx1, tx2]);
+    let stats = as_readonly_index(&repo).stats();
+    assert_eq!(stats.num_commits, 4);
+    assert_eq!(stats.changed_path_commits_range, Some(1..4));
+    assert_eq!(stats.changed_path_levels.len(), 1);
+    assert_eq!(stats.changed_path_levels[0].num_commits, 3);
+    assert_eq!(stats.changed_path_levels[0].num_changed_paths, 5);
+    assert_eq!(stats.changed_path_levels[0].num_paths, 4);
+}
+
+#[test]
+fn test_merge_changed_path_segments_enabled_and_disabled() {
+    let test_repo = TestRepo::init();
+    let repo = test_repo.repo;
+    let root_commit_id = repo.store().root_commit_id();
+
+    let tree1 = create_tree(&repo, &[(repo_path("a"), "")]);
+    let tree2 = create_tree(&repo, &[(repo_path("b"), "")]);
+    let tree3 = create_tree(&repo, &[(repo_path("c"), "")]);
+
+    // Enable changed-path index only in tx1
+    let tx1 = {
+        let mut tx = repo.start_transaction();
+        tx.repo_mut()
+            .new_commit(vec![root_commit_id.clone()], tree1.id())
+            .write()
+            .unwrap();
+        let repo = tx.commit("test").unwrap();
+        let repo = enable_changed_path_index(&repo);
+        let mut tx = repo.start_transaction();
+        tx.repo_mut()
+            .new_commit(vec![root_commit_id.clone()], tree2.id())
+            .write()
+            .unwrap();
+        tx
+    };
+    let mut tx2 = repo.start_transaction();
+    tx2.repo_mut()
+        .new_commit(vec![root_commit_id.clone()], tree3.id())
+        .write()
+        .unwrap();
+    let repo = commit_transactions(vec![tx1, tx2]);
+    let stats = as_readonly_index(&repo).stats();
+    assert_eq!(stats.num_commits, 4);
+    assert_eq!(stats.changed_path_commits_range, Some(2..3));
+    assert_eq!(stats.changed_path_levels.len(), 1);
+    assert_eq!(stats.changed_path_levels[0].num_commits, 1);
+    assert_eq!(stats.changed_path_levels[0].num_changed_paths, 1);
+    assert_eq!(stats.changed_path_levels[0].num_paths, 1);
+
+    // Changed paths in new commit can no longer be indexed
+    let mut tx = repo.start_transaction();
+    write_random_commit(tx.repo_mut());
+    let repo = tx.commit("test").unwrap();
+    let stats = as_readonly_index(&repo).stats();
+    assert_eq!(stats.num_commits, 5);
+    assert_eq!(stats.changed_path_commits_range, Some(2..3));
+}
+
+#[test_case(false; "without changed-path index")]
+#[test_case(true; "with changed-path index")]
+fn test_commit_is_empty(indexed: bool) {
+    let test_repo = TestRepo::init();
+    let repo = if indexed {
+        enable_changed_path_index(&test_repo.repo)
+    } else {
+        test_repo.repo
+    };
+    let root_commit_id = repo.store().root_commit_id();
+    let root_tree_id = repo.store().empty_merged_tree_id();
+
+    let tree2 = create_tree(&repo, &[(repo_path("a"), "")]);
+    let tree3 = create_tree(&repo, &[(repo_path("b"), "")]);
+    let tree4 = create_tree(&repo, &[(repo_path("a"), ""), (repo_path("b"), "")]);
+
+    let mut tx = repo.start_transaction();
+    let commit1 = tx
+        .repo_mut()
+        .new_commit(vec![root_commit_id.clone()], root_tree_id.clone())
+        .write()
+        .unwrap();
+    let commit2 = tx
+        .repo_mut()
+        .new_commit(vec![root_commit_id.clone()], tree2.id())
+        .write()
+        .unwrap();
+    let commit3 = tx
+        .repo_mut()
+        .new_commit(vec![root_commit_id.clone()], tree3.id())
+        .write()
+        .unwrap();
+    let commit4 = tx
+        .repo_mut()
+        .new_commit(
+            vec![
+                commit1.id().clone(),
+                commit2.id().clone(),
+                commit3.id().clone(),
+            ],
+            tree4.id(),
+        )
+        .write()
+        .unwrap();
+    let repo = tx.commit("test").unwrap();
+
+    // Sanity check
+    let stats = as_readonly_index(&repo).stats();
+    if indexed {
+        assert_eq!(stats.changed_path_commits_range, Some(1..5));
+    } else {
+        assert_eq!(stats.changed_path_commits_range, None);
+    }
+
+    assert!(commit1.is_empty(repo.as_ref()).unwrap());
+    assert!(!commit2.is_empty(repo.as_ref()).unwrap());
+    assert!(!commit3.is_empty(repo.as_ref()).unwrap());
+    assert!(commit4.is_empty(repo.as_ref()).unwrap());
+
+    assert_eq!(
+        commit1.parent_tree(repo.as_ref()).unwrap().id(),
+        root_tree_id
+    );
+    assert_eq!(
+        commit2.parent_tree(repo.as_ref()).unwrap().id(),
+        root_tree_id
+    );
+    assert_eq!(
+        commit3.parent_tree(repo.as_ref()).unwrap().id(),
+        root_tree_id
+    );
+    assert_eq!(commit4.parent_tree(repo.as_ref()).unwrap().id(), tree4.id());
+}
+
 #[test]
 fn test_change_id_index() {
     let test_repo = TestRepo::init();
