@@ -14,8 +14,6 @@
 
 #[cfg(unix)]
 use std::os::unix::fs::PermissionsExt as _;
-#[cfg(unix)]
-use std::os::unix::net::UnixListener;
 use std::path::Path;
 use std::path::PathBuf;
 use std::sync::Arc;
@@ -45,7 +43,6 @@ use jj_lib::repo_path::RepoPathBuf;
 use jj_lib::secret_backend::SecretBackend;
 use jj_lib::tree_builder::TreeBuilder;
 use jj_lib::working_copy::CheckoutError;
-use jj_lib::working_copy::CheckoutOptions;
 use jj_lib::working_copy::CheckoutStats;
 use jj_lib::working_copy::SnapshotOptions;
 use jj_lib::working_copy::UntrackedReason;
@@ -294,20 +291,10 @@ fn test_checkout_file_transitions(backend: TestRepoBackend) {
     let right_commit = commit_with_tree(&store, right_tree_id.clone());
 
     let ws = &mut test_workspace.workspace;
-    ws.check_out(
-        repo.op_id().clone(),
-        None,
-        &left_commit,
-        &CheckoutOptions::empty_for_test(),
-    )
-    .unwrap();
-    ws.check_out(
-        repo.op_id().clone(),
-        None,
-        &right_commit,
-        &CheckoutOptions::empty_for_test(),
-    )
-    .unwrap();
+    ws.check_out(repo.op_id().clone(), None, &left_commit)
+        .unwrap();
+    ws.check_out(repo.op_id().clone(), None, &right_commit)
+        .unwrap();
 
     // Check that the working copy is clean.
     let new_tree = test_workspace.snapshot().unwrap();
@@ -401,13 +388,7 @@ fn test_checkout_no_op() {
     let commit2 = commit_with_tree(repo.store(), tree.id());
 
     let ws = &mut test_workspace.workspace;
-    ws.check_out(
-        repo.op_id().clone(),
-        None,
-        &commit1,
-        &CheckoutOptions::empty_for_test(),
-    )
-    .unwrap();
+    ws.check_out(repo.op_id().clone(), None, &commit1).unwrap();
 
     // Test the setup: the file should exist on in the tree state.
     let wc: &LocalWorkingCopy = ws.working_copy().as_any().downcast_ref().unwrap();
@@ -415,14 +396,7 @@ fn test_checkout_no_op() {
 
     // Update to commit2 (same tree as commit1)
     let new_op_id = OperationId::from_bytes(b"whatever");
-    let stats = ws
-        .check_out(
-            new_op_id.clone(),
-            None,
-            &commit2,
-            &CheckoutOptions::empty_for_test(),
-        )
-        .unwrap();
+    let stats = ws.check_out(new_op_id.clone(), None, &commit2).unwrap();
     assert_eq!(stats, CheckoutStats::default());
 
     // The tree state is unchanged but the recorded operation id is updated.
@@ -446,20 +420,9 @@ fn test_conflict_subdirectory() {
     let merged_commit = commit_with_tree(repo.store(), merged_tree.id());
     let repo = &test_workspace.repo;
     let ws = &mut test_workspace.workspace;
-    ws.check_out(
-        repo.op_id().clone(),
-        None,
-        &commit1,
-        &CheckoutOptions::empty_for_test(),
-    )
-    .unwrap();
-    ws.check_out(
-        repo.op_id().clone(),
-        None,
-        &merged_commit,
-        &CheckoutOptions::empty_for_test(),
-    )
-    .unwrap();
+    ws.check_out(repo.op_id().clone(), None, &commit1).unwrap();
+    ws.check_out(repo.op_id().clone(), None, &merged_commit)
+        .unwrap();
 }
 
 #[test]
@@ -509,13 +472,7 @@ fn test_acl() {
     let commit1 = repo.store().get_commit(commit1.id()).unwrap();
     let commit2 = repo.store().get_commit(commit2.id()).unwrap();
 
-    ws.check_out(
-        repo.op_id().clone(),
-        None,
-        &commit1,
-        &CheckoutOptions::empty_for_test(),
-    )
-    .unwrap();
+    ws.check_out(repo.op_id().clone(), None, &commit1).unwrap();
     assert!(
         !secret_modified_path
             .to_fs_path_unchecked(&workspace_root)
@@ -541,13 +498,7 @@ fn test_acl() {
             .to_fs_path_unchecked(&workspace_root)
             .is_file()
     );
-    ws.check_out(
-        repo.op_id().clone(),
-        None,
-        &commit2,
-        &CheckoutOptions::empty_for_test(),
-    )
-    .unwrap();
+    ws.check_out(repo.op_id().clone(), None, &commit2).unwrap();
     assert!(
         !secret_modified_path
             .to_fs_path_unchecked(&workspace_root)
@@ -585,13 +536,7 @@ fn test_tree_builder_file_directory_transition() {
     let mut check_out_tree = |tree_id: &TreeId| {
         let tree = repo.store().get_tree(RepoPathBuf::root(), tree_id).unwrap();
         let commit = commit_with_tree(repo.store(), MergedTreeId::Legacy(tree.id().clone()));
-        ws.check_out(
-            repo.op_id().clone(),
-            None,
-            &commit,
-            &CheckoutOptions::empty_for_test(),
-        )
-        .unwrap();
+        ws.check_out(repo.op_id().clone(), None, &commit).unwrap();
     };
 
     let parent_path = repo_path("foo/bar");
@@ -690,14 +635,7 @@ fn test_conflicting_changes_on_disk() {
     )
     .unwrap();
 
-    let stats = ws
-        .check_out(
-            repo.op_id().clone(),
-            None,
-            &commit,
-            &CheckoutOptions::empty_for_test(),
-        )
-        .unwrap();
+    let stats = ws.check_out(repo.op_id().clone(), None, &commit).unwrap();
     assert_eq!(
         stats,
         CheckoutStats {
@@ -747,13 +685,7 @@ fn test_reset() {
 
     let ws = &mut test_workspace.workspace;
     let commit = commit_with_tree(repo.store(), tree_with_file.id());
-    ws.check_out(
-        repo.op_id().clone(),
-        None,
-        &commit,
-        &CheckoutOptions::empty_for_test(),
-    )
-    .unwrap();
+    ws.check_out(repo.op_id().clone(), None, &commit).unwrap();
 
     // Test the setup: the file should exist on disk and in the tree state.
     assert!(ignored_path.to_fs_path_unchecked(&workspace_root).is_file());
@@ -804,13 +736,7 @@ fn test_checkout_discard() {
     let commit2 = commit_with_tree(repo.store(), tree2.id());
 
     let ws = &mut test_workspace.workspace;
-    ws.check_out(
-        repo.op_id().clone(),
-        None,
-        &commit1,
-        &CheckoutOptions::empty_for_test(),
-    )
-    .unwrap();
+    ws.check_out(repo.op_id().clone(), None, &commit1).unwrap();
     let wc: &LocalWorkingCopy = ws.working_copy().as_any().downcast_ref().unwrap();
     let state_path = wc.state_path().to_path_buf();
 
@@ -821,10 +747,7 @@ fn test_checkout_discard() {
 
     // Start a checkout
     let mut locked_ws = ws.start_working_copy_mutation().unwrap();
-    locked_ws
-        .locked_wc()
-        .check_out(&commit2, &CheckoutOptions::empty_for_test())
-        .unwrap();
+    locked_ws.locked_wc().check_out(&commit2).unwrap();
     // The change should be reflected in the working copy but not saved
     assert!(!file1_path.to_fs_path_unchecked(&workspace_root).is_file());
     assert!(file2_path.to_fs_path_unchecked(&workspace_root).is_file());
@@ -856,7 +779,6 @@ fn test_snapshot_file_directory_transition() {
     let mut test_workspace = TestWorkspace::init();
     let repo = test_workspace.repo.clone();
     let workspace_root = test_workspace.workspace.workspace_root().to_owned();
-    let checkout_options = CheckoutOptions::empty_for_test();
     let to_ws_path = |path: &RepoPath| path.to_fs_path(&workspace_root).unwrap();
 
     // file <-> directory transition at root and sub directories
@@ -871,8 +793,7 @@ fn test_snapshot_file_directory_transition() {
     let commit2 = commit_with_tree(repo.store(), tree2.id());
 
     let ws = &mut test_workspace.workspace;
-    ws.check_out(repo.op_id().clone(), None, &commit1, &checkout_options)
-        .unwrap();
+    ws.check_out(repo.op_id().clone(), None, &commit1).unwrap();
 
     // file -> directory
     std::fs::remove_file(to_ws_path(file1p_path)).unwrap();
@@ -885,8 +806,7 @@ fn test_snapshot_file_directory_transition() {
     assert_eq!(new_tree.id(), tree2.id());
 
     let ws = &mut test_workspace.workspace;
-    ws.check_out(repo.op_id().clone(), None, &commit2, &checkout_options)
-        .unwrap();
+    ws.check_out(repo.op_id().clone(), None, &commit2).unwrap();
 
     // directory -> file
     std::fs::remove_file(to_ws_path(file1_path)).unwrap();
@@ -926,14 +846,7 @@ fn test_materialize_snapshot_conflicted_files() {
         .unwrap();
     let commit = commit_with_tree(repo.store(), merged_tree.id());
 
-    let stats = ws
-        .check_out(
-            repo.op_id().clone(),
-            None,
-            &commit,
-            &CheckoutOptions::empty_for_test(),
-        )
-        .unwrap();
+    let stats = ws.check_out(repo.op_id().clone(), None, &commit).unwrap();
     assert_eq!(
         stats,
         CheckoutStats {
@@ -1064,11 +977,11 @@ fn test_snapshot_special_file() {
     let file2_path = repo_path("file2");
     let file2_disk_path = file2_path.to_fs_path_unchecked(&workspace_root);
     std::fs::write(file2_disk_path, "contents".as_bytes()).unwrap();
-    let socket_disk_path = workspace_root.join("socket");
-    UnixListener::bind(&socket_disk_path).unwrap();
-    // Test the setup
-    assert!(socket_disk_path.exists());
-    assert!(!socket_disk_path.is_file());
+
+    let fifo_disk_path = workspace_root.join("fifo");
+    nix::unistd::mkfifo(&fifo_disk_path, nix::sys::stat::Mode::S_IRWXU).unwrap();
+    assert!(fifo_disk_path.exists());
+    assert!(!fifo_disk_path.is_file());
 
     // Snapshot the working copy with the socket file
     let mut locked_ws = ws.start_working_copy_mutation().unwrap();
@@ -1091,7 +1004,7 @@ fn test_snapshot_special_file() {
 
     // Replace a regular file by a socket and snapshot the working copy again
     std::fs::remove_file(&file1_disk_path).unwrap();
-    UnixListener::bind(&file1_disk_path).unwrap();
+    nix::unistd::mkfifo(&file1_disk_path, nix::sys::stat::Mode::S_IRWXU).unwrap();
     let tree = test_workspace.snapshot().unwrap();
     // Only the regular file should be in the tree
     assert_eq!(
@@ -1180,13 +1093,7 @@ fn test_gitignores_in_ignored_dir() {
     let tree1 = create_tree(&test_workspace.repo, &[(gitignore_path, "ignored\n")]);
     let commit1 = commit_with_tree(test_workspace.repo.store(), tree1.id());
     let ws = &mut test_workspace.workspace;
-    ws.check_out(
-        op_id.clone(),
-        None,
-        &commit1,
-        &CheckoutOptions::empty_for_test(),
-    )
-    .unwrap();
+    ws.check_out(op_id.clone(), None, &commit1).unwrap();
 
     testutils::write_working_copy_file(&workspace_root, nested_gitignore_path, "!file\n");
     testutils::write_working_copy_file(&workspace_root, ignored_path, "contents");
@@ -1237,15 +1144,7 @@ fn test_gitignores_checkout_never_overwrites_ignored() {
     // "contents". The exiting contents ("garbage") shouldn't be replaced in the
     // working copy.
     let ws = &mut test_workspace.workspace;
-    assert!(
-        ws.check_out(
-            repo.op_id().clone(),
-            None,
-            &commit,
-            &CheckoutOptions::empty_for_test()
-        )
-        .is_ok()
-    );
+    assert!(ws.check_out(repo.op_id().clone(), None, &commit,).is_ok());
 
     // Check that the old contents are in the working copy
     let path = workspace_root.join("modified");
@@ -1294,13 +1193,7 @@ fn test_gitignores_ignored_directory_already_tracked() {
 
     // Check out the tree with the files in `ignored/`
     let ws = &mut test_workspace.workspace;
-    ws.check_out(
-        repo.op_id().clone(),
-        None,
-        &commit,
-        &CheckoutOptions::empty_for_test(),
-    )
-    .unwrap();
+    ws.check_out(repo.op_id().clone(), None, &commit).unwrap();
 
     // Make some changes inside the ignored directory and check that they are
     // detected when we snapshot. The files that are still there should not be
@@ -1422,13 +1315,7 @@ fn test_git_submodule(gitignore_content: &str) {
     let commit2 = commit_with_tree(repo.store(), tree_id2.clone());
 
     let ws = &mut test_workspace.workspace;
-    ws.check_out(
-        repo.op_id().clone(),
-        None,
-        &commit1,
-        &CheckoutOptions::empty_for_test(),
-    )
-    .unwrap();
+    ws.check_out(repo.op_id().clone(), None, &commit1).unwrap();
 
     std::fs::create_dir(submodule_path.to_fs_path_unchecked(&workspace_root)).unwrap();
 
@@ -1455,13 +1342,7 @@ fn test_git_submodule(gitignore_content: &str) {
     // Check out new commit updating the submodule, which shouldn't fail because
     // of existing submodule files
     let ws = &mut test_workspace.workspace;
-    ws.check_out(
-        repo.op_id().clone(),
-        None,
-        &commit2,
-        &CheckoutOptions::empty_for_test(),
-    )
-    .unwrap();
+    ws.check_out(repo.op_id().clone(), None, &commit2).unwrap();
 
     // Check that the files in the submodule are not deleted
     let file_in_submodule_path = added_submodule_path.to_fs_path_unchecked(&workspace_root);
@@ -1480,12 +1361,7 @@ fn test_git_submodule(gitignore_content: &str) {
     // Check out the empty tree, which shouldn't fail
     let ws = &mut test_workspace.workspace;
     let stats = ws
-        .check_out(
-            repo.op_id().clone(),
-            None,
-            &store.root_commit(),
-            &CheckoutOptions::empty_for_test(),
-        )
+        .check_out(repo.op_id().clone(), None, &store.root_commit())
         .unwrap();
     assert_eq!(stats.skipped_files, 1);
 }
@@ -1503,13 +1379,7 @@ fn test_check_out_existing_file_cannot_be_removed() {
     let commit2 = commit_with_tree(repo.store(), tree2.id());
 
     let ws = &mut test_workspace.workspace;
-    ws.check_out(
-        repo.op_id().clone(),
-        None,
-        &commit1,
-        &CheckoutOptions::empty_for_test(),
-    )
-    .unwrap();
+    ws.check_out(repo.op_id().clone(), None, &commit1).unwrap();
 
     // Make the parent directory readonly.
     let writable_dir_perm = workspace_root.symlink_metadata().unwrap().permissions();
@@ -1517,12 +1387,7 @@ fn test_check_out_existing_file_cannot_be_removed() {
     readonly_dir_perm.set_readonly(true);
 
     std::fs::set_permissions(&workspace_root, readonly_dir_perm).unwrap();
-    let result = ws.check_out(
-        repo.op_id().clone(),
-        None,
-        &commit2,
-        &CheckoutOptions::empty_for_test(),
-    );
+    let result = ws.check_out(repo.op_id().clone(), None, &commit2);
     std::fs::set_permissions(&workspace_root, writable_dir_perm).unwrap();
 
     // TODO: find a way to trigger the error on Windows
@@ -1547,26 +1412,13 @@ fn test_check_out_existing_file_replaced_with_directory() {
     let commit2 = commit_with_tree(repo.store(), tree2.id());
 
     let ws = &mut test_workspace.workspace;
-    ws.check_out(
-        repo.op_id().clone(),
-        None,
-        &commit1,
-        &CheckoutOptions::empty_for_test(),
-    )
-    .unwrap();
+    ws.check_out(repo.op_id().clone(), None, &commit1).unwrap();
 
     std::fs::remove_file(file_path.to_fs_path_unchecked(&workspace_root)).unwrap();
     std::fs::create_dir(file_path.to_fs_path_unchecked(&workspace_root)).unwrap();
 
     // Checkout doesn't fail, but the file should be skipped.
-    let stats = ws
-        .check_out(
-            repo.op_id().clone(),
-            None,
-            &commit2,
-            &CheckoutOptions::empty_for_test(),
-        )
-        .unwrap();
+    let stats = ws.check_out(repo.op_id().clone(), None, &commit2).unwrap();
     assert_eq!(stats.skipped_files, 1);
     assert!(file_path.to_fs_path_unchecked(&workspace_root).is_dir());
 }
@@ -1592,14 +1444,7 @@ fn test_check_out_existing_directory_symlink() {
 
     // Checkout doesn't fail, but the file should be skipped.
     let ws = &mut test_workspace.workspace;
-    let stats = ws
-        .check_out(
-            repo.op_id().clone(),
-            None,
-            &commit,
-            &CheckoutOptions::empty_for_test(),
-        )
-        .unwrap();
+    let stats = ws.check_out(repo.op_id().clone(), None, &commit).unwrap();
     assert_eq!(stats.skipped_files, 1);
 
     // Therefore, "../escaped" shouldn't be created.
@@ -1628,14 +1473,7 @@ fn test_check_out_existing_directory_symlink_icase_fs() {
 
     // Checkout doesn't fail, but the file should be skipped on icase fs.
     let ws = &mut test_workspace.workspace;
-    let stats = ws
-        .check_out(
-            repo.op_id().clone(),
-            None,
-            &commit,
-            &CheckoutOptions::empty_for_test(),
-        )
-        .unwrap();
+    let stats = ws.check_out(repo.op_id().clone(), None, &commit).unwrap();
     if is_icase_fs {
         assert_eq!(stats.skipped_files, 1);
     } else {
@@ -1678,14 +1516,7 @@ fn test_check_out_existing_file_symlink_icase_fs(victim_exists: bool) {
 
     // Checkout doesn't fail, but the file should be skipped on icase fs.
     let ws = &mut test_workspace.workspace;
-    let stats = ws
-        .check_out(
-            repo.op_id().clone(),
-            None,
-            &commit,
-            &CheckoutOptions::empty_for_test(),
-        )
-        .unwrap();
+    let stats = ws.check_out(repo.op_id().clone(), None, &commit).unwrap();
     if is_icase_fs {
         assert_eq!(stats.skipped_files, 1);
     } else {
@@ -1719,13 +1550,7 @@ fn test_check_out_file_removal_over_existing_directory_symlink() {
 
     // Check out "parent/escaped".
     let ws = &mut test_workspace.workspace;
-    ws.check_out(
-        repo.op_id().clone(),
-        None,
-        &commit1,
-        &CheckoutOptions::empty_for_test(),
-    )
-    .unwrap();
+    ws.check_out(repo.op_id().clone(), None, &commit1).unwrap();
 
     // Pretend that "parent" was a symlink, which might be created by
     // e.g. checking out "PARENT" on case-insensitive fs. The file
@@ -1738,14 +1563,7 @@ fn test_check_out_file_removal_over_existing_directory_symlink() {
     assert!(file_path.to_fs_path_unchecked(&workspace_root).exists());
 
     // Check out empty tree, which tries to remove "parent/escaped".
-    let stats = ws
-        .check_out(
-            repo.op_id().clone(),
-            None,
-            &commit2,
-            &CheckoutOptions::empty_for_test(),
-        )
-        .unwrap();
+    let stats = ws.check_out(repo.op_id().clone(), None, &commit2).unwrap();
     assert_eq!(stats.skipped_files, 1);
 
     // "../escaped" shouldn't be removed.
@@ -1765,12 +1583,7 @@ fn test_check_out_malformed_file_path(file_path_str: &str) {
 
     // Checkout should fail
     let ws = &mut test_workspace.workspace;
-    let result = ws.check_out(
-        repo.op_id().clone(),
-        None,
-        &commit,
-        &CheckoutOptions::empty_for_test(),
-    );
+    let result = ws.check_out(repo.op_id().clone(), None, &commit);
     assert_matches!(result, Err(CheckoutError::InvalidRepoPath(_)));
 
     // Therefore, "pwned" file shouldn't be created.
@@ -1791,12 +1604,7 @@ fn test_check_out_malformed_file_path_windows(file_path_str: &str) {
 
     // Checkout should fail on Windows
     let ws = &mut test_workspace.workspace;
-    let result = ws.check_out(
-        repo.op_id().clone(),
-        None,
-        &commit,
-        &CheckoutOptions::empty_for_test(),
-    );
+    let result = ws.check_out(repo.op_id().clone(), None, &commit);
     if cfg!(windows) {
         assert_matches!(result, Err(CheckoutError::InvalidRepoPath(_)));
     } else {
@@ -1833,12 +1641,7 @@ fn test_check_out_reserved_file_path(file_path_str: &str) {
 
     // Checkout should fail.
     let ws = &mut test_workspace.workspace;
-    let result = ws.check_out(
-        repo.op_id().clone(),
-        None,
-        &commit1,
-        &CheckoutOptions::empty_for_test(),
-    );
+    let result = ws.check_out(repo.op_id().clone(), None, &commit1);
     assert_matches!(result, Err(CheckoutError::ReservedPathComponent { .. }));
 
     // Therefore, "pwned" file shouldn't be created.
@@ -1860,12 +1663,7 @@ fn test_check_out_reserved_file_path(file_path_str: &str) {
     }
 
     // Check out empty tree, which tries to remove the file.
-    let result = ws.check_out(
-        repo.op_id().clone(),
-        None,
-        &commit2,
-        &CheckoutOptions::empty_for_test(),
-    );
+    let result = ws.check_out(repo.op_id().clone(), None, &commit2);
     assert_matches!(result, Err(CheckoutError::ReservedPathComponent { .. }));
 
     // The existing file shouldn't be removed.
@@ -1894,12 +1692,7 @@ fn test_check_out_reserved_file_path_icase_fs(file_path_str: &str) {
 
     // Checkout should fail on icase fs.
     let ws = &mut test_workspace.workspace;
-    let result = ws.check_out(
-        repo.op_id().clone(),
-        None,
-        &commit1,
-        &CheckoutOptions::empty_for_test(),
-    );
+    let result = ws.check_out(repo.op_id().clone(), None, &commit1);
     if is_icase_fs {
         assert_matches!(result, Err(CheckoutError::ReservedPathComponent { .. }));
     } else {
@@ -1923,12 +1716,7 @@ fn test_check_out_reserved_file_path_icase_fs(file_path_str: &str) {
     std::fs::write(&disk_path, "").unwrap();
 
     // Check out empty tree, which tries to remove the file.
-    let result = ws.check_out(
-        repo.op_id().clone(),
-        None,
-        &commit2,
-        &CheckoutOptions::empty_for_test(),
-    );
+    let result = ws.check_out(repo.op_id().clone(), None, &commit2);
     if is_icase_fs {
         assert_matches!(result, Err(CheckoutError::ReservedPathComponent { .. }));
     } else {
@@ -1965,12 +1753,7 @@ fn test_check_out_reserved_file_path_hfs_plus(file_path_str: &str) {
 
     // Checkout should fail on HFS+-like fs.
     let ws = &mut test_workspace.workspace;
-    let result = ws.check_out(
-        repo.op_id().clone(),
-        None,
-        &commit1,
-        &CheckoutOptions::empty_for_test(),
-    );
+    let result = ws.check_out(repo.op_id().clone(), None, &commit1);
     if is_hfs_plus {
         assert_matches!(result, Err(CheckoutError::ReservedPathComponent { .. }));
     } else {
@@ -1994,12 +1777,7 @@ fn test_check_out_reserved_file_path_hfs_plus(file_path_str: &str) {
     std::fs::write(&disk_path, "").unwrap();
 
     // Check out empty tree, which tries to remove the file.
-    let result = ws.check_out(
-        repo.op_id().clone(),
-        None,
-        &commit2,
-        &CheckoutOptions::empty_for_test(),
-    );
+    let result = ws.check_out(repo.op_id().clone(), None, &commit2);
     if is_hfs_plus {
         assert_matches!(result, Err(CheckoutError::ReservedPathComponent { .. }));
     } else {
@@ -2041,12 +1819,7 @@ fn test_check_out_reserved_file_path_vfat(vfat_path_str: &str, file_path_strs: &
 
     // Checkout should fail on VFAT-like fs.
     let ws = &mut test_workspace.workspace;
-    let result = ws.check_out(
-        repo.op_id().clone(),
-        None,
-        &commit1,
-        &CheckoutOptions::empty_for_test(),
-    );
+    let result = ws.check_out(repo.op_id().clone(), None, &commit1);
     if is_vfat {
         assert_matches!(result, Err(CheckoutError::ReservedPathComponent { .. }));
     } else {
@@ -2072,12 +1845,7 @@ fn test_check_out_reserved_file_path_vfat(vfat_path_str: &str, file_path_strs: &
     }
 
     // Check out empty tree, which tries to remove the file.
-    let result = ws.check_out(
-        repo.op_id().clone(),
-        None,
-        &commit2,
-        &CheckoutOptions::empty_for_test(),
-    );
+    let result = ws.check_out(repo.op_id().clone(), None, &commit2);
     if is_vfat {
         assert_matches!(result, Err(CheckoutError::ReservedPathComponent { .. }));
     } else {
