@@ -18,7 +18,6 @@ use std::thread;
 use assert_matches::assert_matches;
 use jj_lib::repo::Repo as _;
 use jj_lib::working_copy::CheckoutError;
-use jj_lib::working_copy::CheckoutOptions;
 use jj_lib::working_copy::SnapshotOptions;
 use jj_lib::workspace::Workspace;
 use jj_lib::workspace::default_working_copy_factories;
@@ -51,13 +50,7 @@ fn test_concurrent_checkout() {
     // Check out tree1
     let ws1 = &mut test_workspace1.workspace;
     // The operation ID is not correct, but that doesn't matter for this test
-    ws1.check_out(
-        repo.op_id().clone(),
-        None,
-        &commit1,
-        &CheckoutOptions::empty_for_test(),
-    )
-    .unwrap();
+    ws1.check_out(repo.op_id().clone(), None, &commit1).unwrap();
 
     // Check out tree2 from another process (simulated by another workspace
     // instance)
@@ -72,23 +65,13 @@ fn test_concurrent_checkout() {
         // Reload commit from the store associated with the workspace
         let repo = ws2.repo_loader().load_at(repo.operation()).unwrap();
         let commit2 = repo.store().get_commit(commit2.id()).unwrap();
-        ws2.check_out(
-            repo.op_id().clone(),
-            Some(&tree_id1),
-            &commit2,
-            &CheckoutOptions::empty_for_test(),
-        )
-        .unwrap();
+        ws2.check_out(repo.op_id().clone(), Some(&tree_id1), &commit2)
+            .unwrap();
     }
 
     // Checking out another tree (via the first workspace instance) should now fail.
     assert_matches!(
-        ws1.check_out(
-            repo.op_id().clone(),
-            Some(&tree_id1),
-            &commit3,
-            &CheckoutOptions::empty_for_test()
-        ),
+        ws1.check_out(repo.op_id().clone(), Some(&tree_id1), &commit3,),
         Err(CheckoutError::ConcurrentCheckout)
     );
 
@@ -126,12 +109,7 @@ fn test_checkout_parallel() {
     let commit = commit_with_tree(repo.store(), tree.id());
     test_workspace
         .workspace
-        .check_out(
-            repo.op_id().clone(),
-            None,
-            &commit,
-            &CheckoutOptions::empty_for_test(),
-        )
+        .check_out(repo.op_id().clone(), None, &commit)
         .unwrap();
 
     thread::scope(|s| {
@@ -154,17 +132,14 @@ fn test_checkout_parallel() {
                 let repo = workspace.repo_loader().load_at(repo.operation()).unwrap();
                 let commit = repo.store().get_commit(commit.id()).unwrap();
                 // The operation ID is not correct, but that doesn't matter for this test
-                let stats = workspace
-                    .check_out(op_id, None, &commit, &CheckoutOptions::empty_for_test())
-                    .unwrap();
+                let stats = workspace.check_out(op_id, None, &commit).unwrap();
                 assert_eq!(stats.updated_files, 0);
                 assert_eq!(stats.added_files, 1);
                 assert_eq!(stats.removed_files, 1);
                 // Check that the working copy contains one of the trees. We may see a
                 // different tree than the one we just checked out, but since
-                // write_tree() should take the same lock as check_out(), write_tree(,
-                // &CheckoutOptions::empty_for_test()) should never produce a
-                // different tree.
+                // write_tree() should take the same lock as check_out(), write_tree()
+                // should never produce a different tree.
                 let mut locked_ws = workspace.start_working_copy_mutation().unwrap();
                 let (new_tree_id, _stats) = locked_ws
                     .locked_wc()
@@ -190,13 +165,7 @@ fn test_racy_checkout() {
     let mut num_matches = 0;
     for _ in 0..100 {
         let ws = &mut test_workspace.workspace;
-        ws.check_out(
-            op_id.clone(),
-            None,
-            &commit,
-            &CheckoutOptions::empty_for_test(),
-        )
-        .unwrap();
+        ws.check_out(op_id.clone(), None, &commit).unwrap();
         assert_eq!(
             std::fs::read(path.to_fs_path_unchecked(&workspace_root)).unwrap(),
             b"1".to_vec()
